@@ -18,17 +18,21 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _HAVE_CLUTTER_ACTOR_H
-#define _HAVE_CLUTTER_ACTOR_H
+#if !defined(__CLUTTER_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
+#error "Only <clutter/clutter.h> can be included directly."
+#endif
+
+#ifndef __CLUTTER_ACTOR_H__
+#define __CLUTTER_ACTOR_H__
 
 /* clutter-actor.h */
 
 #include <glib-object.h>
+#include <pango/pango.h>
+
 #include <clutter/clutter-color.h>
 #include <clutter/clutter-fixed.h>
 #include <clutter/clutter-types.h>
@@ -70,10 +74,10 @@ G_BEGIN_DECLS
  */
 #define CLUTTER_ACTOR_UNSET_FLAGS(a,f)  (((ClutterActor*)(a))->flags &= ~(f))
 
-#define CLUTTER_ACTOR_IS_MAPPED(e)      (((ClutterActor*)(e))->flags & CLUTTER_ACTOR_MAPPED)
-#define CLUTTER_ACTOR_IS_REALIZED(e)    (((ClutterActor*)(e))->flags & CLUTTER_ACTOR_REALIZED)
+#define CLUTTER_ACTOR_IS_MAPPED(e)      ((((ClutterActor*)(e))->flags & CLUTTER_ACTOR_MAPPED) != FALSE)
+#define CLUTTER_ACTOR_IS_REALIZED(e)    ((((ClutterActor*)(e))->flags & CLUTTER_ACTOR_REALIZED) != FALSE)
 #define CLUTTER_ACTOR_IS_VISIBLE(e)     (CLUTTER_ACTOR_IS_MAPPED (e) && CLUTTER_ACTOR_IS_REALIZED (e))
-#define CLUTTER_ACTOR_IS_REACTIVE(e)    (((ClutterActor*)(e))->flags & CLUTTER_ACTOR_REACTIVE)
+#define CLUTTER_ACTOR_IS_REACTIVE(e)    ((((ClutterActor*)(e))->flags & CLUTTER_ACTOR_REACTIVE) != FALSE)
 
 typedef struct _ClutterActorClass    ClutterActorClass;
 typedef struct _ClutterActorBox      ClutterActorBox;
@@ -186,6 +190,7 @@ struct _ClutterActor
  * @destroy: signal class handler for #ClutterActor::destroy
  * @pick: virtual function, used to draw an outline of the actor with
  *   the given color
+ * @queue_redraw: class handler for #ClutterActor::queue-redraw
  * @event: class handler for #ClutterActor::event
  * @button_press_event: class handler for #ClutterActor::button-press-event
  * @button_release_event: class handler for
@@ -223,6 +228,9 @@ struct _ClutterActorClass
   void (* pick)                 (ClutterActor          *actor,
                                  const ClutterColor    *color);
 
+  void (* queue_redraw)         (ClutterActor          *actor,
+                                 ClutterActor          *leaf_that_queued);
+
   /* size negotiation */
   void (* get_preferred_width)  (ClutterActor          *actor,
                                  ClutterUnit            for_height,
@@ -386,38 +394,33 @@ void                  clutter_actor_set_rotation              (ClutterActor
                                                                gint                   x,
                                                                gint                   y,
                                                                gint                   z);
-void                  clutter_actor_set_rotationx             (ClutterActor          *self,
-                                                               ClutterRotateAxis      axis,
-                                                               ClutterFixed           angle,
-                                                               gint                   x,
-                                                               gint                   y,
-                                                               gint                   z);
 void                  clutter_actor_set_rotationu             (ClutterActor          *self,
                                                                ClutterRotateAxis      axis,
                                                                gdouble                angle,
                                                                ClutterUnit            x,
                                                                ClutterUnit            y,
                                                                ClutterUnit            z);
+void                  clutter_actor_set_z_rotation_from_gravity (ClutterActor        *self,
+                                                               gdouble                angle,
+                                                               ClutterGravity         gravity);
 gdouble               clutter_actor_get_rotation              (ClutterActor          *self,
                                                                ClutterRotateAxis      axis,
                                                                gint                  *x,
                                                                gint                  *y,
                                                                gint                  *z);
-ClutterFixed          clutter_actor_get_rotationx             (ClutterActor          *self,
-                                                               ClutterRotateAxis      axis,
-                                                               gint                  *x,
-                                                               gint                  *y,
-                                                               gint                  *z);
 gdouble               clutter_actor_get_rotationu             (ClutterActor          *self,
                                                                ClutterRotateAxis      axis,
                                                                ClutterUnit           *x,
                                                                ClutterUnit           *y,
                                                                ClutterUnit           *z);
+ClutterGravity        clutter_actor_get_z_rotation_gravity    (ClutterActor          *self);
 
 void                  clutter_actor_set_opacity               (ClutterActor          *self,
                                                                guint8                 opacity);
 guint8                clutter_actor_get_opacity               (ClutterActor          *self);
 guint8                clutter_actor_get_paint_opacity         (ClutterActor          *self);
+gboolean              clutter_actor_get_paint_visibility      (ClutterActor          *self);
+
 
 void                  clutter_actor_set_name                  (ClutterActor          *self,
                                                                const gchar           *name);
@@ -468,18 +471,33 @@ void                  clutter_actor_set_depthu                (ClutterActor
                                                                ClutterUnit            depth);
 ClutterUnit           clutter_actor_get_depthu                (ClutterActor          *self);
 
-void                  clutter_actor_set_scalex                (ClutterActor          *self,
-                                                               ClutterFixed           scale_x,
-                                                               ClutterFixed           scale_y);
 void                  clutter_actor_set_scale                 (ClutterActor          *self,
                                                                gdouble                scale_x,
                                                                gdouble                scale_y);
-void                  clutter_actor_get_scalex                (ClutterActor          *self,
-                                                               ClutterFixed          *scale_x,
-                                                               ClutterFixed          *scale_y);
+void                  clutter_actor_set_scale_full            (ClutterActor          *self,
+                                                               gdouble                scale_x,
+                                                               gdouble                scale_y,
+                                                               int                    center_x,
+                                                               int                    center_y);
+void                  clutter_actor_set_scale_fullu           (ClutterActor          *self,
+                                                               gdouble                scale_x,
+                                                               gdouble                scale_y,
+                                                               ClutterUnit            center_x,
+                                                               ClutterUnit            center_y);
+void                  clutter_actor_set_scale_with_gravity    (ClutterActor          *self,
+                                                               gdouble                scale_x,
+                                                               gdouble                scale_y,
+                                                               ClutterGravity         gravity);
 void                  clutter_actor_get_scale                 (ClutterActor          *self,
                                                                gdouble               *scale_x,
                                                                gdouble               *scale_y);
+void                  clutter_actor_get_scale_center          (ClutterActor          *self,
+                                                               gint                  *center_x,
+                                                               gint                  *center_y);
+void                  clutter_actor_get_scale_centeru         (ClutterActor          *self,
+                                                               ClutterUnit           *center_x,
+                                                               ClutterUnit           *center_y);
+ClutterGravity        clutter_actor_get_scale_gravity         (ClutterActor          *self);
 
 void                  clutter_actor_move_by                   (ClutterActor          *self,
                                                                gint                   dx,
@@ -502,6 +520,12 @@ gboolean              clutter_actor_set_shader                (ClutterActor
                                                                ClutterShader         *shader);
 ClutterShader *       clutter_actor_get_shader                (ClutterActor          *self);
 void                  clutter_actor_set_shader_param          (ClutterActor          *self,
+                                                               const gchar           *param,
+                                                               const GValue          *value);
+void                  clutter_actor_set_shader_param_int      (ClutterActor          *self,
+                                                               const gchar           *param,
+                                                               gint                   value);
+void                  clutter_actor_set_shader_param_float    (ClutterActor          *self,
                                                                const gchar           *param,
                                                                gfloat                 value);
 
@@ -514,6 +538,7 @@ void     clutter_actor_move_anchor_point              (ClutterActor   *self,
 void     clutter_actor_get_anchor_point               (ClutterActor   *self,
                                                        gint           *anchor_x,
                                                        gint           *anchor_y);
+ClutterGravity clutter_actor_get_anchor_point_gravity (ClutterActor   *self);
 void     clutter_actor_set_anchor_pointu              (ClutterActor   *self,
                                                        ClutterUnit     anchor_x,
                                                        ClutterUnit     anchor_y);
@@ -537,21 +562,26 @@ gboolean clutter_actor_is_rotated                     (ClutterActor   *self);
 gboolean clutter_actor_is_scaled                      (ClutterActor   *self);
 gboolean clutter_actor_should_pick_paint              (ClutterActor   *self);
 
-void     clutter_actor_box_get_from_vertices          (ClutterVertex    vtx[4],
-				                       ClutterActorBox *box);
+void     clutter_actor_box_get_from_vertices          (ClutterVertex        vtx[4],
+				                       ClutterActorBox     *box);
 
-void clutter_actor_get_abs_allocation_vertices       (ClutterActor  *self,
-                                                      ClutterVertex  verts[4]);
+void clutter_actor_get_abs_allocation_vertices        (ClutterActor        *self,
+                                                       ClutterVertex        verts[4]);
 
-void clutter_actor_apply_transform_to_point          (ClutterActor        *self,
-                                                      const ClutterVertex *point,
-                                                      ClutterVertex       *vertex);
-void clutter_actor_apply_relative_transform_to_point (ClutterActor        *self,
-                                                      ClutterActor        *ancestor,
-                                                      const ClutterVertex *point,
-                                                      ClutterVertex       *vertex);
+void clutter_actor_apply_transform_to_point           (ClutterActor        *self,
+                                                       const ClutterVertex *point,
+                                                       ClutterVertex       *vertex);
+void clutter_actor_apply_relative_transform_to_point  (ClutterActor        *self,
+                                                       ClutterActor        *ancestor,
+                                                       const ClutterVertex *point,
+                                                       ClutterVertex       *vertex);
 
-gboolean clutter_actor_get_paint_visibility           (ClutterActor *self);
+void          clutter_actor_grab_key_focus            (ClutterActor        *self);
+
+PangoContext *clutter_actor_get_pango_context         (ClutterActor        *self);
+PangoContext *clutter_actor_create_pango_context      (ClutterActor        *self);
+PangoLayout * clutter_actor_create_pango_layout       (ClutterActor        *self,
+                                                       const gchar         *text);
 
 G_END_DECLS
 
