@@ -240,7 +240,7 @@ clutter_behaviour_get_property (GObject    *object,
 
 static void
 clutter_behaviour_alpha_notify_unimplemented (ClutterBehaviour *behaviour,
-                                              guint32           alpha_value)
+                                              gdouble           alpha_value)
 {
   g_warning ("ClutterBehaviourClass::alpha_notify not implemented for `%s'",
              g_type_name (G_TYPE_FROM_INSTANCE (behaviour)));
@@ -362,7 +362,7 @@ clutter_behaviour_apply (ClutterBehaviour *behave,
       return;
     }
 
-  priv->actors = g_slist_prepend (priv->actors, g_object_ref (actor));
+  priv->actors = g_slist_append (priv->actors, g_object_ref (actor));
   g_signal_connect (actor, "destroy",
                     G_CALLBACK (remove_actor_on_destroy),
                     behave);
@@ -457,7 +457,7 @@ clutter_behaviour_get_n_actors (ClutterBehaviour *behave)
  *
  * Gets an actor the behaviour was applied to referenced by index num.
  *
- * Return value: A Clutter actor or NULL if @index_ is invalid.
+ * Return value: (transfer none): A Clutter actor or NULL if @index_ is invalid.
  *
  * Since: 0.2
  */
@@ -507,7 +507,7 @@ clutter_behaviour_actors_foreach (ClutterBehaviour            *behave,
  *
  * Retrieves the #ClutterAlpha object bound to @behave.
  *
- * Return value: a #ClutterAlpha object, or %NULL if no alpha
+ * Return value: (transfer none): a #ClutterAlpha object, or %NULL if no alpha
  *   object has been bound to this behaviour.
  * 
  * Since: 0.2
@@ -537,11 +537,9 @@ notify_cb (GObject          *object,
 
   if (klass->alpha_notify)
     {
-      guint32 alpha_value;
+      gdouble alpha_value = clutter_alpha_get_alpha (behave->priv->alpha);
 
-      alpha_value = clutter_alpha_get_alpha (behave->priv->alpha);
-
-      CLUTTER_NOTE (BEHAVIOUR, "calling %s::alpha_notify (%p, %d)",
+      CLUTTER_NOTE (BEHAVIOUR, "calling %s::alpha_notify (%p, %.4f)",
                     g_type_name (G_TYPE_FROM_CLASS (klass)),
                     behave, alpha_value);
 
@@ -615,8 +613,8 @@ clutter_behaviour_set_alpha (ClutterBehaviour *behave,
  * for derived classes to use this in there alpha notify method but use 
  * #clutter_behaviour_actors_foreach as it avoids alot of needless allocations.
  *
- * Return value: a list of actors. You should free the returned list
- *   with g_slist_free() when finished using it.
+ * Return value: (transfer container) (element-type ClutterActor): a list of actors.
+ *   You should free the returned list with g_slist_free() when finished using it.
  * 
  * Since: 0.2
  */
@@ -633,7 +631,7 @@ clutter_behaviour_get_actors (ClutterBehaviour *behave)
   for (l = priv->actors; l != NULL; l = l->next)
     retval = g_slist_prepend (retval, l->data);
 
-  return retval;
+  return g_slist_reverse (retval);
 }
 
 /**
