@@ -40,6 +40,11 @@ clutter_modifier_type_get_type(void) {
         { CLUTTER_BUTTON3_MASK, "CLUTTER_BUTTON3_MASK", "button3-mask" },
         { CLUTTER_BUTTON4_MASK, "CLUTTER_BUTTON4_MASK", "button4-mask" },
         { CLUTTER_BUTTON5_MASK, "CLUTTER_BUTTON5_MASK", "button5-mask" },
+        { CLUTTER_SUPER_MASK, "CLUTTER_SUPER_MASK", "super-mask" },
+        { CLUTTER_HYPER_MASK, "CLUTTER_HYPER_MASK", "hyper-mask" },
+        { CLUTTER_META_MASK, "CLUTTER_META_MASK", "meta-mask" },
+        { CLUTTER_RELEASE_MASK, "CLUTTER_RELEASE_MASK", "release-mask" },
+        { CLUTTER_MODIFIER_MASK, "CLUTTER_MODIFIER_MASK", "modifier-mask" },
         { 0, NULL, NULL }
       };
       etype = g_flags_register_static (g_intern_static_string ("ClutterModifierType"), values);
@@ -158,6 +163,27 @@ clutter_init_error_get_type(void) {
     }
   return etype;
 }
+/* enumerations from "./clutter-path.h" */
+#include "./clutter-path.h"
+GType
+clutter_path_node_type_get_type(void) {
+  static GType etype = 0;
+  if (G_UNLIKELY (!etype))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_PATH_MOVE_TO, "CLUTTER_PATH_MOVE_TO", "move-to" },
+        { CLUTTER_PATH_LINE_TO, "CLUTTER_PATH_LINE_TO", "line-to" },
+        { CLUTTER_PATH_CURVE_TO, "CLUTTER_PATH_CURVE_TO", "curve-to" },
+        { CLUTTER_PATH_CLOSE, "CLUTTER_PATH_CLOSE", "close" },
+        { CLUTTER_PATH_REL_MOVE_TO, "CLUTTER_PATH_REL_MOVE_TO", "rel-move-to" },
+        { CLUTTER_PATH_REL_LINE_TO, "CLUTTER_PATH_REL_LINE_TO", "rel-line-to" },
+        { CLUTTER_PATH_REL_CURVE_TO, "CLUTTER_PATH_REL_CURVE_TO", "rel-curve-to" },
+        { 0, NULL, NULL }
+      };
+      etype = g_enum_register_static (g_intern_static_string ("ClutterPathNodeType"), values);
+    }
+  return etype;
+}
 /* enumerations from "./clutter-script.h" */
 #include "./clutter-script.h"
 GType
@@ -322,6 +348,65 @@ clutter_request_mode_get_type(void) {
     }
   return etype;
 }
+GType
+clutter_animation_mode_get_type(void) {
+  static GType etype = 0;
+  if (G_UNLIKELY (!etype))
+    {
+      static const GEnumValue values[] = {
+        { CLUTTER_CUSTOM_MODE, "CLUTTER_CUSTOM_MODE", "custom-mode" },
+        { CLUTTER_LINEAR, "CLUTTER_LINEAR", "linear" },
+        { CLUTTER_EASE_IN_QUAD, "CLUTTER_EASE_IN_QUAD", "ease-in-quad" },
+        { CLUTTER_EASE_OUT_QUAD, "CLUTTER_EASE_OUT_QUAD", "ease-out-quad" },
+        { CLUTTER_EASE_IN_OUT_QUAD, "CLUTTER_EASE_IN_OUT_QUAD", "ease-in-out-quad" },
+        { CLUTTER_EASE_IN_CUBIC, "CLUTTER_EASE_IN_CUBIC", "ease-in-cubic" },
+        { CLUTTER_EASE_OUT_CUBIC, "CLUTTER_EASE_OUT_CUBIC", "ease-out-cubic" },
+        { CLUTTER_EASE_IN_OUT_CUBIC, "CLUTTER_EASE_IN_OUT_CUBIC", "ease-in-out-cubic" },
+        { CLUTTER_EASE_IN_QUART, "CLUTTER_EASE_IN_QUART", "ease-in-quart" },
+        { CLUTTER_EASE_OUT_QUART, "CLUTTER_EASE_OUT_QUART", "ease-out-quart" },
+        { CLUTTER_EASE_IN_OUT_QUART, "CLUTTER_EASE_IN_OUT_QUART", "ease-in-out-quart" },
+        { CLUTTER_EASE_IN_QUINT, "CLUTTER_EASE_IN_QUINT", "ease-in-quint" },
+        { CLUTTER_EASE_OUT_QUINT, "CLUTTER_EASE_OUT_QUINT", "ease-out-quint" },
+        { CLUTTER_EASE_IN_OUT_QUINT, "CLUTTER_EASE_IN_OUT_QUINT", "ease-in-out-quint" },
+        { CLUTTER_EASE_IN_SINE, "CLUTTER_EASE_IN_SINE", "ease-in-sine" },
+        { CLUTTER_EASE_OUT_SINE, "CLUTTER_EASE_OUT_SINE", "ease-out-sine" },
+        { CLUTTER_EASE_IN_OUT_SINE, "CLUTTER_EASE_IN_OUT_SINE", "ease-in-out-sine" },
+        { CLUTTER_EASE_IN_EXPO, "CLUTTER_EASE_IN_EXPO", "ease-in-expo" },
+        { CLUTTER_EASE_OUT_EXPO, "CLUTTER_EASE_OUT_EXPO", "ease-out-expo" },
+        { CLUTTER_EASE_IN_OUT_EXPO, "CLUTTER_EASE_IN_OUT_EXPO", "ease-in-out-expo" },
+        { CLUTTER_EASE_IN_CIRC, "CLUTTER_EASE_IN_CIRC", "ease-in-circ" },
+        { CLUTTER_EASE_OUT_CIRC, "CLUTTER_EASE_OUT_CIRC", "ease-out-circ" },
+        { CLUTTER_EASE_IN_OUT_CIRC, "CLUTTER_EASE_IN_OUT_CIRC", "ease-in-out-circ" },
+        { CLUTTER_EASE_IN_ELASTIC, "CLUTTER_EASE_IN_ELASTIC", "ease-in-elastic" },
+        { CLUTTER_EASE_OUT_ELASTIC, "CLUTTER_EASE_OUT_ELASTIC", "ease-out-elastic" },
+        { CLUTTER_EASE_IN_OUT_ELASTIC, "CLUTTER_EASE_IN_OUT_ELASTIC", "ease-in-out-elastic" },
+        { CLUTTER_EASE_IN_BACK, "CLUTTER_EASE_IN_BACK", "ease-in-back" },
+        { CLUTTER_EASE_OUT_BACK, "CLUTTER_EASE_OUT_BACK", "ease-out-back" },
+        { CLUTTER_EASE_IN_OUT_BACK, "CLUTTER_EASE_IN_OUT_BACK", "ease-in-out-back" },
+        { CLUTTER_EASE_IN_BOUNCE, "CLUTTER_EASE_IN_BOUNCE", "ease-in-bounce" },
+        { CLUTTER_EASE_OUT_BOUNCE, "CLUTTER_EASE_OUT_BOUNCE", "ease-out-bounce" },
+        { CLUTTER_EASE_IN_OUT_BOUNCE, "CLUTTER_EASE_IN_OUT_BOUNCE", "ease-in-out-bounce" },
+        { CLUTTER_ANIMATION_LAST, "CLUTTER_ANIMATION_LAST", "animation-last" },
+        { 0, NULL, NULL }
+      };
+      etype = g_enum_register_static (g_intern_static_string ("ClutterAnimationMode"), values);
+    }
+  return etype;
+}
+GType
+clutter_font_flags_get_type(void) {
+  static GType etype = 0;
+  if (G_UNLIKELY (!etype))
+    {
+      static const GFlagsValue values[] = {
+        { CLUTTER_FONT_MIPMAPPING, "CLUTTER_FONT_MIPMAPPING", "mipmapping" },
+        { CLUTTER_FONT_HINTING, "CLUTTER_FONT_HINTING", "hinting" },
+        { 0, NULL, NULL }
+      };
+      etype = g_flags_register_static (g_intern_static_string ("ClutterFontFlags"), values);
+    }
+  return etype;
+}
 
 /* Generated data ends here */
 
