@@ -21,8 +21,12 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _HAVE_CLUTTER_EVENT_H
-#define _HAVE_CLUTTER_EVENT_H
+#if !defined(__CLUTTER_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
+#error "Only <clutter/clutter.h> can be included directly."
+#endif
+
+#ifndef __CLUTTER_EVENT_H__
+#define __CLUTTER_EVENT_H__
 
 #include <glib-object.h>
 #include <clutter/clutter-types.h>
@@ -64,6 +68,11 @@ G_BEGIN_DECLS
  * @CLUTTER_BUTTON3_MASK: Mask applied by the third pointer button
  * @CLUTTER_BUTTON4_MASK: Mask applied by the fourth pointer button
  * @CLUTTER_BUTTON5_MASK: Mask applied by the fifth pointer button
+ * @CLUTTER_SUPER_MASK: Mask applied by the Super key
+ * @CLUTTER_HYPER_MASK: Mask applied by the Hyper key
+ * @CLUTTER_META_MASK: Mask applied by the Meta key
+ * @CLUTTER_RELEASE_MASK: Mask applied during release
+ * @CLUTTER_MODIFIER_MASK: A mask covering all modifier types
  *
  * Masks applied to a #ClutterEvent by modifiers.
  *
@@ -82,7 +91,17 @@ typedef enum {
   CLUTTER_BUTTON2_MASK  = 1 << 9,
   CLUTTER_BUTTON3_MASK  = 1 << 10,
   CLUTTER_BUTTON4_MASK  = 1 << 11,
-  CLUTTER_BUTTON5_MASK  = 1 << 12
+  CLUTTER_BUTTON5_MASK  = 1 << 12,
+
+  /* bits 15 to 25 are currently unused; bit 29 is used internally */
+
+  CLUTTER_SUPER_MASK    = 1 << 26,
+  CLUTTER_HYPER_MASK    = 1 << 27,
+  CLUTTER_META_MASK     = 1 << 28,
+
+  CLUTTER_RELEASE_MASK  = 1 << 30,
+
+  CLUTTER_MODIFIER_MASK = 0x5c001fff
 } ClutterModifierType;
 
 /**
@@ -220,6 +239,7 @@ struct _ClutterAnyEvent
  * @keyval: raw key value
  * @hardware_keycode: raw hardware key value
  * @unicode_value: Unicode representation
+ * @device: reserved for future use
  *
  * Key event
  *
@@ -236,6 +256,7 @@ struct _ClutterKeyEvent
   guint keyval;
   guint16 hardware_keycode;
   gunichar unicode_value;
+  ClutterInputDevice *device;
 };
 
 /**
@@ -245,8 +266,8 @@ struct _ClutterKeyEvent
  * @flags: event flags
  * @stage: event source stage
  * @source: event source actor
- * @x: event X coordinate
- * @y: event Y coordinate
+ * @x: event X coordinate, relative to the stage
+ * @y: event Y coordinate, relative to the stage
  * @modifier_state: button modifiers
  * @button: event button
  * @click_count: number of button presses within the default time
@@ -254,7 +275,11 @@ struct _ClutterKeyEvent
  * @axes: reserved for future use
  * @device: reserved for future use
  *
- * Button event
+ * Button event.
+ *
+ * The event coordinates are relative to the stage that received the
+ * event, and can be transformed into actor-relative coordinates by
+ * using clutter_actor_transform_stage_point().
  *
  * Since: 0.2
  */
@@ -284,6 +309,7 @@ struct _ClutterButtonEvent
  * @x: event X coordinate
  * @y: event Y coordinate
  * @related: actor related to the crossing
+ * @device: reserved for future use
  *
  * Event for the movement of the pointer across different actors
  *
@@ -427,7 +453,7 @@ ClutterModifierType clutter_event_get_state     (ClutterEvent       *event);
 void                clutter_event_get_coords    (ClutterEvent       *event,
                                                  gint               *x,
                                                  gint               *y);
-gint                clutter_event_get_device_id (ClutterEvent *event);
+gint                clutter_event_get_device_id (ClutterEvent       *event);
 ClutterActor*       clutter_event_get_source    (ClutterEvent       *event);
 
 guint               clutter_key_event_symbol    (ClutterKeyEvent    *keyev);
@@ -438,8 +464,10 @@ guint32             clutter_button_event_button (ClutterButtonEvent *buttev);
 
 guint32             clutter_keysym_to_unicode   (guint               keyval);
 
-ClutterStage*       clutter_event_get_stage     (ClutterEvent *event);
+ClutterStage*       clutter_event_get_stage     (ClutterEvent       *event);
+
+guint32             clutter_get_current_event_time (void);
 
 G_END_DECLS
 
-#endif
+#endif /* __CLUTTER_EVENT_H__ */
