@@ -86,7 +86,9 @@ clutter_group_paint (ClutterActor *actor)
   ClutterGroupPrivate *priv = CLUTTER_GROUP (actor)->priv;
   GList               *child_item;
 
-  CLUTTER_NOTE (PAINT, "ClutterGroup paint enter");
+  CLUTTER_NOTE (PAINT, "ClutterGroup paint enter '%s'",
+                clutter_actor_get_name (actor) ? clutter_actor_get_name (actor)
+                                               : "unknown");
 
   for (child_item = priv->children;
        child_item != NULL;
@@ -100,7 +102,9 @@ clutter_group_paint (ClutterActor *actor)
 	clutter_actor_paint (child);
     }
 
-  CLUTTER_NOTE (PAINT, "ClutterGroup paint leave");
+  CLUTTER_NOTE (PAINT, "ClutterGroup paint leave '%s'",
+                clutter_actor_get_name (actor) ? clutter_actor_get_name (actor)
+                                               : "unknown");
 }
 
 static void
@@ -502,6 +506,9 @@ clutter_group_real_raise (ClutterContainer *container,
     {
       clutter_actor_set_depth (actor, clutter_actor_get_depth (sibling));
     }
+
+  if (CLUTTER_ACTOR_IS_VISIBLE (container))
+    clutter_actor_queue_redraw (CLUTTER_ACTOR (container));
 }
 
 static void
@@ -541,6 +548,9 @@ clutter_group_real_lower (ClutterContainer *container,
     {
       clutter_actor_set_depth (actor, clutter_actor_get_depth (sibling));
     }
+
+  if (CLUTTER_ACTOR_IS_VISIBLE (container))
+    clutter_actor_queue_redraw (CLUTTER_ACTOR (container));
 }
 
 static gint
@@ -563,7 +573,7 @@ clutter_group_real_sort_depth_order (ClutterContainer *container)
 
   priv->children = g_list_sort (priv->children, sort_z_order);
 
-  if (CLUTTER_ACTOR_IS_VISIBLE (CLUTTER_ACTOR (self)))
+  if (CLUTTER_ACTOR_IS_VISIBLE (self))
     clutter_actor_queue_redraw (CLUTTER_ACTOR (self));
 
 }
@@ -788,7 +798,7 @@ clutter_group_get_n_children (ClutterGroup *self)
  *
  * Gets a groups child held at @index_ in stack.
  *
- * Return value: A Clutter actor or NULL if @index_ is invalid.
+ * Return value: (transfer none): A Clutter actor or NULL if @index_ is invalid.
  *
  * Since: 0.2
  **/
