@@ -50,23 +50,29 @@
 #include "clutter-frame-source.h"
 
 #include "cogl/cogl.h"
+#include "pango/cogl-pango.h"
 
 /* main context */
-static ClutterMainContext *ClutterCntx  = NULL;
+static ClutterMainContext *ClutterCntx       = NULL;
 
 /* main lock and locking/unlocking functions */
-static GMutex *clutter_threads_mutex    = NULL;
-static GCallback clutter_threads_lock   = NULL;
-static GCallback clutter_threads_unlock = NULL;
+static GMutex *clutter_threads_mutex         = NULL;
+static GCallback clutter_threads_lock        = NULL;
+static GCallback clutter_threads_unlock      = NULL;
 
-static gboolean clutter_is_initialized  = FALSE;
-static gboolean clutter_show_fps        = FALSE;
-static gboolean clutter_fatal_warnings  = FALSE;
+/* command line options */
+static gboolean clutter_is_initialized       = FALSE;
+static gboolean clutter_show_fps             = FALSE;
+static gboolean clutter_fatal_warnings       = FALSE;
+static gboolean clutter_disable_mipmap_text  = FALSE;
+static gboolean clutter_use_fuzzy_picking    = FALSE;
 
-static guint clutter_default_fps        = 60;
+static guint clutter_default_fps             = 60;
 
-static guint clutter_main_loop_level    = 0;
-static GSList *main_loops               = NULL;
+static PangoDirection clutter_text_direction = PANGO_DIRECTION_LTR;
+
+static guint clutter_main_loop_level         = 0;
+static GSList *main_loops                    = NULL;
 
 guint clutter_debug_flags = 0;  /* global clutter debug flag */
 
@@ -86,6 +92,7 @@ static const GDebugKey clutter_debug_keys[] = {
   { "script", CLUTTER_DEBUG_SCRIPT },
   { "shader", CLUTTER_DEBUG_SHADER },
   { "multistage", CLUTTER_DEBUG_MULTISTAGE },
+  { "animation", CLUTTER_DEBUG_ANIMATION }
 };
 #endif /* CLUTTER_ENABLE_DEBUG */
 
@@ -149,7 +156,7 @@ _clutter_stage_maybe_setup_viewport (ClutterStage *stage)
       guint width, height;
 
       clutter_actor_get_size (CLUTTER_ACTOR (stage), &width, &height);
-      clutter_stage_get_perspectivex (stage, &perspective);
+      clutter_stage_get_perspective (stage, &perspective);
 
       CLUTTER_NOTE (PAINT, "Setting up the viewport");
 
@@ -297,16 +304,16 @@ _clutter_id_to_color (guint id, ClutterColor *col)
    * driver / hw implementation.
    */
   if (ctx->fb_r_mask_used != ctx->fb_r_mask)
-    red = red * 2 + 1;
+    red = red * 2;
   if (ctx->fb_g_mask_used != ctx->fb_g_mask)
-    green = green * 2 + 1;
+    green = green * 2;
   if (ctx->fb_b_mask_used != ctx->fb_b_mask)
-    blue  = blue  * 2 + 1;
+    blue  = blue  * 2;
 
   /* shift up to be full 8bit values */
-  red   = (red   << (8 - ctx->fb_r_mask)) | (0xff >> (ctx->fb_r_mask + 1));
-  green = (green << (8 - ctx->fb_g_mask)) | (0xff >> (ctx->fb_g_mask + 1));
-  blue  = (blue  << (8 - ctx->fb_b_mask)) | (0xff >> (ctx->fb_b_mask + 1));
+  red   = (red   << (8 - ctx->fb_r_mask)) | (0x7f >> (ctx->fb_r_mask_used));
+  green = (green << (8 - ctx->fb_g_mask)) | (0x7f >> (ctx->fb_g_mask_used));
+  blue  = (blue  << (8 - ctx->fb_b_mask)) | (0x7f >> (ctx->fb_b_mask_used));
 
   col->red   = red;
   col->green = green;
@@ -351,8 +358,9 @@ _clutter_do_pick (ClutterStage   *stage,
   ClutterMainContext *context;
   guchar              pixel[4];
   GLint               viewport[4];
-  ClutterColor        white = { 0xff, 0xff, 0xff, 0xff };
+  CoglColor           white;
   guint32             id;
+  GLboolean           dither_was_on;
 
   context = clutter_context_get_default ();
 
@@ -361,9 +369,12 @@ _clutter_do_pick (ClutterStage   *stage,
   /* needed for when a context switch happens */
   _clutter_stage_maybe_setup_viewport (stage);
 
-  cogl_paint_init (&white);
+  cogl_color_set_from_4ub (&white, 255, 255, 255, 255);
+  cogl_disable_fog ();
+  cogl_clear (&white);
 
   /* Disable dithering (if any) when doing the painting in pick mode */
+  dither_was_on = glIsEnabled (GL_DITHER);
   glDisable (GL_DITHER);
 
   /* Render the entire scence in pick mode - just single colored silhouette's
@@ -381,12 +392,13 @@ _clutter_do_pick (ClutterStage   *stage,
   */
   glFinish();
 
-  /* glEnable (GL_DITHER); we never enabled this originally, so its
-     probably not safe to then enable it */
-
   /* Read the color of the screen co-ords pixel */
   glReadPixels (x, viewport[3] - y -1, 1, 1, GL_RGBA, GL_UNSIGNED_BYTE, pixel);
 
+  /* Restore whether GL_DITHER was enabled */
+  if (dither_was_on)
+    glEnable (GL_DITHER);
+
   if (pixel[0] == 0xff && pixel[1] == 0xff && pixel[2] == 0xff)
     return CLUTTER_ACTOR (stage);
 
@@ -395,23 +407,100 @@ _clutter_do_pick (ClutterStage   *stage,
   return clutter_get_actor_by_gid (id);
 }
 
+static PangoDirection
+clutter_get_text_direction (void)
+{
+  PangoDirection dir = PANGO_DIRECTION_LTR;
+  const gchar *direction;
+
+  direction = g_getenv ("CLUTTER_TEXT_DIRECTION");
+  if (direction && *direction != '\0')
+    {
+      if (strcmp (direction, "rtl") == 0)
+        dir = PANGO_DIRECTION_RTL;
+      else if (strcmp (direction, "ltr") == 0)
+        dir = PANGO_DIRECTION_LTR;
+    }
+  else
+    {
+      /* Translate to default:RTL if you want your widgets
+       * to be RTL, otherwise translate to default:LTR.
+       *
+       * Do *not* translate it to "predefinito:LTR": if it
+       * it isn't default:LTR or default:RTL it will not work
+       */
+      char *e = _("default:LTR");
+
+      if (strcmp (e, "default:RTL") == 0)
+        dir = PANGO_DIRECTION_RTL;
+      else if (strcmp (e, "default:LTR") == 0)
+        dir = PANGO_DIRECTION_LTR;
+      else
+        g_warning ("Whoever translated default:LTR did so wrongly.");
+    }
+
+  return dir;
+}
+
+static void
+update_pango_context (ClutterBackend *backend,
+                      PangoContext   *context)
+{
+  PangoFontDescription *font_desc;
+  const cairo_font_options_t *font_options;
+  const gchar *font_name;
+  gdouble resolution;
+
+  /* update the text direction */
+  pango_context_set_base_dir (context, clutter_text_direction);
+
+  /* get the configuration for the PangoContext from the backend */
+  font_name = clutter_backend_get_font_name (backend);
+  font_options = clutter_backend_get_font_options (backend);
+  resolution = clutter_backend_get_resolution (backend);
+
+  font_desc = pango_font_description_from_string (font_name);
+
+  if (resolution < 0)
+    resolution = 96.0; /* fall back */
+
+  pango_context_set_font_description (context, font_desc);
+  pango_cairo_context_set_font_options (context, font_options);
+  pango_cairo_context_set_resolution (context, resolution);
+
+  pango_font_description_free (font_desc);
+}
+
+PangoContext *
+_clutter_context_get_pango_context (ClutterMainContext *self)
+{
+  if (G_UNLIKELY (self->pango_context == NULL))
+    {
+      PangoContext *context;
+
+      context = cogl_pango_font_map_create_context (self->font_map);
+      self->pango_context = context;
+
+      g_signal_connect (self->backend, "resolution-changed",
+                        G_CALLBACK (update_pango_context),
+                        self->pango_context);
+      g_signal_connect (self->backend, "font-changed",
+                        G_CALLBACK (update_pango_context),
+                        self->pango_context);
+    }
+
+  update_pango_context (self->backend, self->pango_context);
+
+  return self->pango_context;
+}
+
 PangoContext *
 _clutter_context_create_pango_context (ClutterMainContext *self)
 {
   PangoContext *context;
-  gdouble resolution;
-  cairo_font_options_t *font_options;
 
-  resolution = clutter_backend_get_resolution (self->backend);
-  if (resolution < 0)
-    resolution = 96.0; /* fall back */
-
-  context = pango_clutter_font_map_create_context (self->font_map);
-
-  pango_cairo_context_set_resolution (context, resolution);
-
-  font_options = clutter_backend_get_font_options (self->backend);
-  pango_cairo_context_set_font_options (context, font_options);
+  context = cogl_pango_font_map_create_context (self->font_map);
+  update_pango_context (self->backend, context);
 
   return context;
 }
@@ -830,7 +919,7 @@ clutter_threads_add_timeout (guint       interval,
  * clutter_threads_add_frame_source_full:
  * @priority: the priority of the frame source. Typically this will be in the
  *            range between #G_PRIORITY_DEFAULT and #G_PRIORITY_HIGH.
- * @interval: the time between calls to the function, in milliseconds
+ * @fps: the number of times per second to call the function
  * @func: function to call
  * @data: data to pass to the function
  * @notify: function to call when the timeout source is removed
@@ -859,7 +948,7 @@ clutter_threads_add_timeout (guint       interval,
  */
 guint
 clutter_threads_add_frame_source_full (gint           priority,
-				       guint          interval,
+				       guint          fps,
 				       GSourceFunc    func,
 				       gpointer       data,
 				       GDestroyNotify notify)
@@ -874,14 +963,14 @@ clutter_threads_add_frame_source_full (gint           priority,
   dispatch->notify = notify;
 
   return clutter_frame_source_add_full (priority,
-					interval,
+					fps,
 					clutter_threads_dispatch, dispatch,
 					clutter_threads_dispatch_free);
 }
 
 /**
  * clutter_threads_add_frame_source:
- * @interval: the time between calls to the function, in milliseconds
+ * @fps: the number of times per second to call the function
  * @func: function to call
  * @data: data to pass to the function
  *
@@ -892,14 +981,14 @@ clutter_threads_add_frame_source_full (gint           priority,
  * Since: 0.8
  */
 guint
-clutter_threads_add_frame_source (guint       interval,
+clutter_threads_add_frame_source (guint       fps,
 				  GSourceFunc func,
 				  gpointer    data)
 {
   g_return_val_if_fail (func != NULL, 0);
 
   return clutter_threads_add_frame_source_full (G_PRIORITY_DEFAULT,
-						interval,
+						fps,
 						func, data,
 						NULL);
 }
@@ -998,6 +1087,17 @@ clutter_get_timestamp (void)
 #endif
 }
 
+static gboolean
+clutter_arg_direction_cb (const char *key,
+                          const char *value,
+                          gpointer    user_data)
+{
+  clutter_text_direction =
+    (strcmp (value, "rtl") == 0) ? PANGO_DIRECTION_RTL
+                                 : PANGO_DIRECTION_LTR;
+
+  return TRUE;
+}
 
 #ifdef CLUTTER_ENABLE_DEBUG
 static gboolean
@@ -1058,20 +1158,8 @@ clutter_init_real (GError **error)
   /*
    * Call backend post parse hooks.
    */
-  if (CLUTTER_BACKEND_GET_CLASS (backend)->post_parse)
-    if (!CLUTTER_BACKEND_GET_CLASS (backend)->post_parse (backend, error))
-      return CLUTTER_INIT_ERROR_BACKEND;
-
-  /*
-   * Resolution requires display to be open, so can only be queried after
-   * the post_parse hooks run.
-   */
-  ctx->font_map = PANGO_CLUTTER_FONT_MAP (pango_clutter_font_map_new ());
-
-  resolution = clutter_backend_get_resolution (ctx->backend);
-  pango_clutter_font_map_set_resolution (ctx->font_map, resolution);
-
-  pango_clutter_font_map_set_use_mipmapping (ctx->font_map, TRUE);
+  if (!_clutter_backend_post_parse (backend, error))
+    return CLUTTER_INIT_ERROR_BACKEND;
 
   /* Stage will give us a GL Context etc */
   stage = clutter_stage_get_default ();
@@ -1087,6 +1175,8 @@ clutter_init_real (GError **error)
       return CLUTTER_INIT_ERROR_INTERNAL;
     }
 
+  clutter_stage_set_title (CLUTTER_STAGE (stage), g_get_prgname ());
+
   clutter_actor_realize (stage);
 
   if (!CLUTTER_ACTOR_IS_REALIZED (stage))
@@ -1105,6 +1195,23 @@ clutter_init_real (GError **error)
    * start issueing cogl commands
   */
 
+  /*
+   * Resolution requires display to be open, so can only be queried after
+   * the post_parse hooks run.
+   *
+   * NB: cogl_pango requires a Cogl context.
+   */
+  ctx->font_map = COGL_PANGO_FONT_MAP (cogl_pango_font_map_new ());
+
+  resolution = clutter_backend_get_resolution (ctx->backend);
+  cogl_pango_font_map_set_resolution (ctx->font_map, resolution);
+
+  if (G_LIKELY (!clutter_disable_mipmap_text))
+    cogl_pango_font_map_set_use_mipmapping (ctx->font_map, TRUE);
+
+  clutter_text_direction = clutter_get_text_direction ();
+
+
   /* Figure out framebuffer masks used for pick */
   cogl_get_bitmasks (&ctx->fb_r_mask, &ctx->fb_g_mask, &ctx->fb_b_mask, NULL);
 
@@ -1112,10 +1219,8 @@ clutter_init_real (GError **error)
   ctx->fb_g_mask_used = ctx->fb_g_mask;
   ctx->fb_b_mask_used = ctx->fb_b_mask;
 
-#ifndef HAVE_CLUTTER_FRUITY
-  /* We always do fuzzy picking for the fruity backend */
-  if (g_getenv ("CLUTTER_FUZZY_PICK") != NULL)
-#endif
+  /* XXX - describe what "fuzzy picking" is */
+  if (clutter_use_fuzzy_picking)
     {
       ctx->fb_r_mask_used--;
       ctx->fb_g_mask_used--;
@@ -1128,8 +1233,6 @@ clutter_init_real (GError **error)
   /* finally features - will call to backend and cogl */
   _clutter_feature_init ();
 
-  clutter_stage_set_title (CLUTTER_STAGE (stage), g_get_prgname ());
-
   clutter_is_initialized = TRUE;
   ctx->is_initialized = TRUE;
 
@@ -1138,16 +1241,25 @@ clutter_init_real (GError **error)
 
 static GOptionEntry clutter_args[] = {
   { "clutter-show-fps", 0, 0, G_OPTION_ARG_NONE, &clutter_show_fps,
-    "Show frames per second", NULL },
+    N_("Show frames per second"), NULL },
   { "clutter-default-fps", 0, 0, G_OPTION_ARG_INT, &clutter_default_fps,
-    "Default frame rate", "FPS" },
+    N_("Default frame rate"), "FPS" },
   { "g-fatal-warnings", 0, 0, G_OPTION_ARG_NONE, &clutter_fatal_warnings,
-    "Make all warnings fatal", NULL },
+    N_("Make all warnings fatal"), NULL },
+  { "clutter-text-direction", 0, 0, G_OPTION_ARG_CALLBACK,
+    clutter_arg_direction_cb,
+    N_("Direction for the text"), "DIRECTION" },
+  { "clutter-disable-mipmapped-text", 0, 0, G_OPTION_ARG_NONE,
+    &clutter_disable_mipmap_text,
+    N_("Disable mipmapping on text"), NULL },
+  { "clutter-use-fuzzy-picking", 0, 0, G_OPTION_ARG_NONE,
+    &clutter_use_fuzzy_picking,
+    N_("Use 'fuzzy' picking"), NULL },
 #ifdef CLUTTER_ENABLE_DEBUG
   { "clutter-debug", 0, 0, G_OPTION_ARG_CALLBACK, clutter_arg_debug_cb,
-    "Clutter debugging flags to set", "FLAGS" },
+    N_("Clutter debugging flags to set"), "FLAGS" },
   { "clutter-no-debug", 0, 0, G_OPTION_ARG_CALLBACK, clutter_arg_no_debug_cb,
-    "Clutter debugging flags to unset", "FLAGS" },
+    N_("Clutter debugging flags to unset"), "FLAGS" },
 #endif /* CLUTTER_ENABLE_DEBUG */
   { NULL, },
 };
@@ -1204,10 +1316,20 @@ pre_parse_hook (GOptionContext  *context,
       clutter_default_fps = CLAMP (default_fps, 1, 1000);
     }
 
-  if (CLUTTER_BACKEND_GET_CLASS (backend)->pre_parse)
-    return CLUTTER_BACKEND_GET_CLASS (backend)->pre_parse (backend, error);
+  env_string = g_getenv ("CLUTTER_DISABLE_MIPMAPPED_TEXT");
+  if (env_string)
+    clutter_disable_mipmap_text = TRUE;
 
-  return TRUE;
+#ifdef HAVE_CLUTTER_FRUITY
+  /* we always enable fuzzy picking in the "fruity" backend */
+  clutter_use_fuzzy_picking = TRUE;
+#else
+  env_string = g_getenv ("CLUTTER_FUZZY_PICK");
+  if (env_string)
+    clutter_use_fuzzy_picking = TRUE;
+#endif /* HAVE_CLUTTER_FRUITY */
+
+  return _clutter_backend_pre_parse (backend, error);
 }
 
 /* post_parse_hook: initialise the context and data structures
@@ -1277,7 +1399,7 @@ post_parse_hook (GOptionContext  *context,
  * Clutter #GOptionGroup has returned %TRUE, Clutter is guaranteed to be
  * initialized.
  *
- * Return value: a #GOptionGroup for the commandline arguments
+ * Return value: (transfer full): a #GOptionGroup for the commandline arguments
  *   recognized by Clutter
  *
  * Since: 0.2
@@ -1293,13 +1415,14 @@ clutter_get_option_group (void)
   context = clutter_context_get_default ();
 
   group = g_option_group_new ("clutter",
-                              "Clutter Options",
-                              "Show Clutter Options",
+                              _("Clutter Options"),
+                              _("Show Clutter Options"),
                               NULL,
                               NULL);
 
   g_option_group_set_parse_hooks (group, pre_parse_hook, post_parse_hook);
   g_option_group_add_entries (group, clutter_args);
+  g_option_group_set_translation_domain (group, GETTEXT_PACKAGE);
 
   /* add backend-specific options */
   _clutter_backend_add_options (context->backend, group);
@@ -1319,7 +1442,7 @@ clutter_get_option_group (void)
  * function when needing to set foreign display connection with
  * clutter_x11_set_display(), or with gtk_clutter_init().
  *
- * Return value: a #GOptionGroup for the commandline arguments
+ * Return value: (transfer full): a #GOptionGroup for the commandline arguments
  *   recognized by Clutter
  *
  * Since: 0.8.2
@@ -1340,10 +1463,15 @@ clutter_get_option_group_without_init (void)
   return group;
 }
 
+/* Note that the gobject-introspection annotations for the argc/argv
+ * parameters do not produce the right result; however, they do
+ * allow the common case of argc=NULL, argv=NULL to work.
+ */
+
 /**
  * clutter_init_with_args:
- * @argc: a pointer to the number of command line arguments
- * @argv: a pointer to the array of command line arguments
+ * @argc: (inout): a pointer to the number of command line arguments
+ * @argv: (array length=argc) (inout): a pointer to the array of command line arguments
  * @parameter_string: a string which is displayed in the
  *   first line of <option>--help</option> output, after
  *   <literal><replaceable>programname</replaceable> [OPTION...]</literal>
@@ -1371,9 +1499,9 @@ clutter_get_option_group_without_init (void)
 ClutterInitError
 clutter_init_with_args (int            *argc,
                         char         ***argv,
-                        char           *parameter_string,
+                        const char     *parameter_string,
                         GOptionEntry   *entries,
-                        char           *translation_domain,
+                        const char     *translation_domain,
                         GError        **error)
 {
   GOptionContext *context;
@@ -1393,9 +1521,12 @@ clutter_init_with_args (int            *argc,
       if (argc && *argc > 0 && *argv)
 	g_set_prgname ((*argv)[0]);
 
-      group   = clutter_get_option_group ();
       context = g_option_context_new (parameter_string);
 
+      group = clutter_get_option_group ();
+      g_option_context_add_group (context, group);
+
+      group = cogl_get_option_group ();
       g_option_context_add_group (context, group);
 
       if (entries)
@@ -1429,7 +1560,7 @@ clutter_parse_args (int    *argc,
                     char ***argv)
 {
   GOptionContext *option_context;
-  GOptionGroup   *clutter_group;
+  GOptionGroup   *clutter_group, *cogl_group;
   GError         *error = NULL;
   gboolean        ret = TRUE;
 
@@ -1445,6 +1576,9 @@ clutter_parse_args (int    *argc,
   clutter_group = clutter_get_option_group ();
   g_option_context_set_main_group (option_context, clutter_group);
 
+  cogl_group = cogl_get_option_group ();
+  g_option_context_add_group (option_context, cogl_group);
+
   if (!g_option_context_parse (option_context, argc, argv, &error))
     {
       if (error)
@@ -1463,8 +1597,8 @@ clutter_parse_args (int    *argc,
 
 /**
  * clutter_init:
- * @argc: The number of arguments in @argv
- * @argv: A pointer to an array of arguments.
+ * @argc: (inout): The number of arguments in @argv
+ * @argv: (array length=argc) (inout): A pointer to an array of arguments.
  *
  * It will initialise everything needed to operate with Clutter and
  * parses some standard command line options. @argc and @argv are
@@ -1600,34 +1734,33 @@ static inline void
 emit_event (ClutterEvent *event,
             gboolean      is_key_event)
 {
-#define MAX_EVENT_DEPTH 512
+  static gboolean      lock = FALSE;
 
-  static ClutterActor **event_tree = NULL;
-  static gboolean       lock = FALSE;
-
-  ClutterActor         *actor;
-  gint                  i = 0, n_tree_events = 0;
+  GPtrArray *event_tree = NULL;
+  ClutterActor *actor;
+  gint i = 0;
 
   if (!event->any.source)
     {
-      g_warning ("No event source set, discarding event");
+      CLUTTER_NOTE (EVENT, "No source set, discarding event");
       return;
     }
 
   /* reentrancy check */
   if (lock != FALSE)
-    return;
+    {
+      g_warning ("Tried emitting event during event delivery, bailing out.n");
+      return;
+    }
 
   lock = TRUE;
 
-  /* Sorry Mr Bassi. */
-  if (G_UNLIKELY (event_tree == NULL))
-    event_tree = g_new0 (ClutterActor *, MAX_EVENT_DEPTH);
+  event_tree = g_ptr_array_sized_new (64);
 
   actor = event->any.source;
 
   /* Build 'tree' of emitters for the event */
-  while (actor && n_tree_events < MAX_EVENT_DEPTH)
+  while (actor)
     {
       ClutterActor *parent;
 
@@ -1637,30 +1770,29 @@ emit_event (ClutterEvent *event,
           parent == NULL ||         /* stage gets all events */
           is_key_event)             /* keyboard events are always emitted */
         {
-          event_tree[n_tree_events++] = g_object_ref (actor);
+          g_ptr_array_add (event_tree, g_object_ref (actor));
         }
 
       actor = parent;
     }
 
   /* Capture */
-  for (i = n_tree_events-1; i >= 0; i--)
-    if (clutter_actor_event (event_tree[i], event, TRUE))
+  for (i = event_tree->len - 1; i >= 0; i--)
+    if (clutter_actor_event (g_ptr_array_index (event_tree, i), event, TRUE))
       goto done;
 
   /* Bubble */
-  for (i = 0; i < n_tree_events; i++)
-    if (clutter_actor_event (event_tree[i], event, FALSE))
+  for (i = 0; i < event_tree->len; i++)
+    if (clutter_actor_event (g_ptr_array_index (event_tree, i), event, FALSE))
       goto done;
 
 done:
+  for (i = 0; i < event_tree->len; i++)
+    g_object_unref (g_ptr_array_index (event_tree, i));
 
-  for (i = 0; i < n_tree_events; i++)
-    g_object_unref (event_tree[i]);
+  g_ptr_array_free (event_tree, TRUE);
 
   lock = FALSE;
-
-#undef MAX_EVENT_DEPTH
 }
 
 /*
@@ -1718,85 +1850,127 @@ unset_motion_last_actor (ClutterActor *actor, ClutterInputDevice *dev)
     dev->motion_last_actor = NULL;
 }
 
-static inline void
-generate_enter_leave_events (ClutterEvent *event)
+static ClutterInputDevice * clutter_event_get_device (ClutterEvent *event);
+
+/* This function should perhaps be public and in clutter-event.c ?
+ */
+static ClutterInputDevice *
+clutter_event_get_device (ClutterEvent *event)
+{
+  g_return_val_if_fail (event != NULL, NULL);
+
+  switch (event->type)
+    {
+    case CLUTTER_NOTHING:
+    case CLUTTER_STAGE_STATE:
+    case CLUTTER_DESTROY_NOTIFY:
+    case CLUTTER_CLIENT_MESSAGE:
+    case CLUTTER_DELETE:
+    case CLUTTER_ENTER:
+    case CLUTTER_LEAVE:
+      return NULL;
+      break;
+    case CLUTTER_BUTTON_PRESS:
+    case CLUTTER_BUTTON_RELEASE:
+      return event->button.device;
+    case CLUTTER_MOTION:
+      return event->motion.device;
+    case CLUTTER_SCROLL:
+      return event->scroll.device;
+      break;
+    case CLUTTER_KEY_PRESS:
+    case CLUTTER_KEY_RELEASE:
+      break;
+    }
+  return NULL;
+}
+
+static void
+set_motion_last_actor (ClutterActor       *motion_current_actor,
+                       ClutterInputDevice *device)
 {
   ClutterMainContext *context              = ClutterCntx;
-  ClutterActor       *motion_current_actor = event->motion.source;
   ClutterActor       *last_actor           = context->motion_last_actor;
 
-  if (event->motion.device != NULL)
-    last_actor = event->motion.device->motion_last_actor;
-
-  if (last_actor != motion_current_actor)
-    {
-      if (motion_current_actor)
-        {
-          ClutterEvent cev;
-
-          cev.crossing.device  = event->motion.device;
-
-          if (context->motion_last_actor)
-            {
-              cev.crossing.type    = CLUTTER_LEAVE;
-              cev.crossing.time    = event->any.time;
-              cev.crossing.flags   = 0;
-              cev.crossing.x       = event->motion.x;
-              cev.crossing.y       = event->motion.y;
-              cev.crossing.source  = last_actor;
-              cev.crossing.stage   = event->any.stage;
-
-              /* unref in free  */
-              cev.crossing.related = motion_current_actor;
-
-              g_queue_push_head (context->events_queue,
-                                 clutter_event_copy (&cev));
-            }
-
-          cev.crossing.type    = CLUTTER_ENTER;
-          cev.crossing.time    = event->any.time;
-          cev.crossing.flags   = 0;
-          cev.crossing.x       = event->motion.x;
-          cev.crossing.y       = event->motion.y;
-          cev.crossing.source  = motion_current_actor;
-          cev.crossing.stage   = event->any.stage;
-
-          if (context->motion_last_actor)
-            cev.crossing.related = last_actor;
-          else
-            {
-              /* the previous actor we were getting events from seems to have
-               * vanished
-               */
-              cev.crossing.related = NULL;
-            }
-
-          g_queue_push_head (context->events_queue,
-			     clutter_event_copy (&cev));
-        }
-    }
+  if (device != NULL)
+    last_actor = device->motion_last_actor;
 
   if (last_actor && last_actor != motion_current_actor)
     {
       g_signal_handlers_disconnect_by_func
                        (last_actor,
                         G_CALLBACK (unset_motion_last_actor),
-                        event->motion.device);
+                        device);
     }
 
   if (motion_current_actor && last_actor != motion_current_actor)
     {
       g_signal_connect (motion_current_actor, "destroy",
                         G_CALLBACK (unset_motion_last_actor),
-                        event->motion.device);
+                        device);
     }
 
-  if (event->motion.device != NULL)
-    event->motion.device->motion_last_actor = motion_current_actor;
+  if (device != NULL)
+    device->motion_last_actor = motion_current_actor;
   else
     context->motion_last_actor = motion_current_actor;
 }
 
+static inline void
+generate_enter_leave_events (ClutterEvent *event)
+{
+  ClutterMainContext *context              = ClutterCntx;
+  ClutterActor       *motion_current_actor = event->motion.source;
+  ClutterActor       *last_actor           = context->motion_last_actor;
+  ClutterInputDevice *device               = clutter_event_get_device (event);
+
+  if (device != NULL)
+    last_actor = device->motion_last_actor;
+
+  if (last_actor != motion_current_actor)
+    {
+      if (motion_current_actor)
+        {
+          gint         x, y;
+          ClutterEvent cev;
+
+          cev.crossing.device  = device;
+          clutter_event_get_coords (event, &x, &y);
+
+          if (context->motion_last_actor)
+            {
+              cev.crossing.type    = CLUTTER_LEAVE;
+              cev.crossing.time    = event->any.time;
+              cev.crossing.flags   = 0;
+              cev.crossing.x       = x;
+              cev.crossing.y       = y;
+              cev.crossing.source  = last_actor;
+              cev.crossing.stage   = event->any.stage;
+              cev.crossing.related = motion_current_actor;
+
+              emit_pointer_event (&cev, device);
+            }
+
+          cev.crossing.type    = CLUTTER_ENTER;
+          cev.crossing.time    = event->any.time;
+          cev.crossing.flags   = 0;
+          cev.crossing.x       = x;
+          cev.crossing.y       = y;
+          cev.crossing.source  = motion_current_actor;
+          cev.crossing.stage   = event->any.stage;
+
+          if (context->motion_last_actor)
+            cev.crossing.related = last_actor;
+          else
+            cev.crossing.related = NULL;
+
+          emit_pointer_event (&cev, device);
+        }
+    }
+
+  set_motion_last_actor (motion_current_actor, device);
+}
+
 /**
  * clutter_do_event
  * @event: a #ClutterEvent.
@@ -1828,14 +2002,31 @@ clutter_do_event (ClutterEvent *event)
 
   CLUTTER_TIMESTAMP (EVENT, "Event received");
 
+  context->last_event_time = clutter_event_get_time (event);
+
   switch (event->type)
     {
       case CLUTTER_NOTHING:
         event->any.source = stage;
         break;
 
-      case CLUTTER_ENTER:
       case CLUTTER_LEAVE:
+        /* The source is set for generated events, not for events
+         * resulting from the cursor leaving the stage
+         */
+        if (event->any.source == NULL)
+          {
+            ClutterActor *last_actor = context->motion_last_actor;
+
+            if (event->crossing.device != NULL)
+              last_actor = event->crossing.device->motion_last_actor;
+
+            event->any.source = last_actor;
+
+            set_motion_last_actor (NULL, event->crossing.device);
+          }
+        /* flow through */
+      case CLUTTER_ENTER:
         emit_pointer_event (event, event->crossing.device);
         break;
 
@@ -2012,12 +2203,10 @@ clutter_do_event (ClutterEvent *event)
           CLUTTER_NOTE (EVENT, "Reactive event received at %i, %i - actor: %p",
                         x, y, actor);
 
-          if (event->type == CLUTTER_MOTION)
-            {
-              /* Generate enter leave events (if any) */
-              generate_enter_leave_events (event);
-            }
-          else /* (button event) */
+          /* Create, enter/leave events if needed */
+          generate_enter_leave_events (event);
+
+          if (event->type != CLUTTER_MOTION)
             {
               /* Generate click count */
               event_click_count_generate (event);
@@ -2062,8 +2251,8 @@ clutter_do_event (ClutterEvent *event)
  *
  * Retrieves the #ClutterActor with @id.
  *
- * Return value: the actor with the passed id or %NULL. The returned
- *   actor does not have its reference count increased.
+ * Return value: (transfer none): the actor with the passed id or %NULL.
+ *   The returned actor does not have its reference count increased.
  *
  * Since: 0.6
  */
@@ -2090,6 +2279,9 @@ clutter_base_init (void)
 
       initialised = TRUE;
 
+      bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+      bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
       /* initialise GLib type system */
       g_type_init ();
 
@@ -2169,8 +2361,14 @@ on_pointer_grab_weak_notify (gpointer data,
  *
  * Grabs pointer events, after the grab is done all pointer related events
  * (press, motion, release, enter, leave and scroll) are delivered to this
- * actor directly. The source set in the event will be the actor that would
- * have received the event if the pointer grab was not in effect.
+ * actor directly without passing through both capture and bubble phases of
+ * the event delivery chain. The source set in the event will be the actor
+ * that would have received the event if the pointer grab was not in effect.
+ *
+ * <note><para>Grabs completely override the entire event delivery chain
+ * done by Clutter. Pointer grabs should only be used as a last resource;
+ * using the #ClutterActor::captured-event signal should always be the
+ * preferred way to intercept event delivery to reactive actors.</para></note>
  *
  * If you wish to grab all the pointer events for a specific input device,
  * you should use clutter_grab_pointer_for_device().
@@ -2293,7 +2491,7 @@ clutter_ungrab_pointer_for_device (gint id)
  *
  * Queries the current pointer grab of clutter.
  *
- * Return value: the actor currently holding the pointer grab, or NULL if there is no grab.
+ * Return value: (transfer none): the actor currently holding the pointer grab, or NULL if there is no grab.
  *
  * Since: 0.6
  */
@@ -2323,10 +2521,17 @@ on_keyboard_grab_weak_notify (gpointer data,
  * clutter_grab_keyboard:
  * @actor: a #ClutterActor
  *
- * Grabs keyboard events, after the grab is done keyboard events ("key-press-event"
- * and "key-release-event") are delivered to this actor directly. The source
- * set in the event will be the actor that would have received the event if the
- * keyboard grab was not in effect.
+ * Grabs keyboard events, after the grab is done keyboard
+ * events (#ClutterActor::key-press-event and #ClutterActor::key-release-event)
+ * are delivered to this actor directly. The source set in the event will be
+ * the actor that would have received the event if the keyboard grab was not
+ * in effect.
+ *
+ * Like pointer grabs, keyboard grabs should only be used as a last
+ * resource.
+ *
+ * See also clutter_stage_set_key_focus() and clutter_actor_grab_key_focus()
+ * to perform a "soft" key grab and assign key focus to a specific actor.
  *
  * Since: 0.6
  */
@@ -2378,7 +2583,7 @@ clutter_ungrab_keyboard (void)
  *
  * Queries the current keyboard grab of clutter.
  *
- * Return value: the actor currently holding the keyboard grab, or NULL if there is no grab.
+ * Return value: (transfer none): the actor currently holding the keyboard grab, or NULL if there is no grab.
  *
  * Since: 0.6
  */
@@ -2460,49 +2665,95 @@ void
 clutter_clear_glyph_cache (void)
 {
   if (CLUTTER_CONTEXT ()->font_map)
-    pango_clutter_font_map_clear_glyph_cache (CLUTTER_CONTEXT ()->font_map);
+    cogl_pango_font_map_clear_glyph_cache (CLUTTER_CONTEXT ()->font_map);
 }
 
 /**
- * clutter_set_use_mipmapped_text:
- * @value: %TRUE to enable mipmapping or %FALSE to disable.
+ * clutter_set_font_flags:
+ * @flags: The new flags
  *
- * Sets whether subsequent text rendering operations will use
- * mipmapped textures or not. Using mipmapped textures will improve
- * the quality for scaled down text but will use more texture memory.
+ * Sets the font quality options for subsequent text rendering
+ * operations.
  *
- * Since: 0.8
+ * Using mipmapped textures will improve the quality for scaled down
+ * text but will use more texture memory.
+ *
+ * Enabling hinting improves text quality for static text but may
+ * introduce some artifacts if the text is animated.
+ *
+ * Since: 1.0
  */
 void
-clutter_set_use_mipmapped_text (gboolean value)
+clutter_set_font_flags (ClutterFontFlags flags)
 {
+  ClutterFontFlags old_flags, changed_flags;
+  const cairo_font_options_t *font_options;
+  cairo_font_options_t *new_font_options;
+  ClutterBackend *backend;
+
+  backend = clutter_get_default_backend ();
+
   if (CLUTTER_CONTEXT ()->font_map)
-    pango_clutter_font_map_set_use_mipmapping (CLUTTER_CONTEXT ()->font_map,
-					       value);
+    cogl_pango_font_map_set_use_mipmapping (CLUTTER_CONTEXT ()->font_map,
+					    (flags
+                                             & CLUTTER_FONT_MIPMAPPING) != 0);
+
+  old_flags = clutter_get_font_flags ();
+
+  font_options = clutter_backend_get_font_options (backend);
+  new_font_options = cairo_font_options_copy (font_options);
+
+  /* Only set the font options that have actually changed so we don't
+     override a detailed setting from the backend */
+  changed_flags = old_flags ^ flags;
+
+  if ((changed_flags & CLUTTER_FONT_HINTING))
+    cairo_font_options_set_hint_style (new_font_options,
+                                       (flags & CLUTTER_FONT_HINTING)
+                                       ? CAIRO_HINT_STYLE_FULL
+                                       : CAIRO_HINT_STYLE_NONE);
+
+  clutter_backend_set_font_options (backend, new_font_options);
+
+  cairo_font_options_destroy (new_font_options);
+
+  if (CLUTTER_CONTEXT ()->pango_context)
+    update_pango_context (backend, CLUTTER_CONTEXT ()->pango_context);
 }
 
 /**
- * clutter_get_use_mipmapped_text:
+ * clutter_get_font_flags:
  *
- * Gets whether mipmapped textures are used in text operations.
- * See clutter_set_use_mipmapped_text().
+ * Gets the current font flags for rendering text. See
+ * clutter_set_font_flags().
  *
- * Return value: %TRUE if text operations should use mipmapped
- *   textures
+ * Return value: The font flags
  *
- * Since: 0.8
+ * Since: 1.0
  */
-gboolean
-clutter_get_use_mipmapped_text (void)
+ClutterFontFlags
+clutter_get_font_flags (void)
 {
-  PangoClutterFontMap *font_map = NULL;
+  ClutterMainContext *ctxt = CLUTTER_CONTEXT ();
+  CoglPangoFontMap *font_map = NULL;
+  const cairo_font_options_t *font_options;
+  ClutterFontFlags flags = 0;
 
   font_map = CLUTTER_CONTEXT ()->font_map;
 
-  if (font_map)
-    return pango_clutter_font_map_get_use_mipmapping (font_map);
+  if (G_LIKELY (font_map)
+      && cogl_pango_font_map_get_use_mipmapping (font_map))
+    flags |= CLUTTER_FONT_MIPMAPPING;
 
-  return FALSE;
+  font_options = clutter_backend_get_font_options (ctxt->backend);
+
+  if ((cairo_font_options_get_hint_style (font_options)
+       != CAIRO_HINT_STYLE_DEFAULT)
+      && (cairo_font_options_get_hint_style (font_options)
+          != CAIRO_HINT_STYLE_NONE))
+    flags |= CLUTTER_FONT_HINTING;
+
+  return flags;
 }
 
 /**
@@ -2511,7 +2762,7 @@ clutter_get_use_mipmapped_text (void)
  *
  * Retrieves the #ClutterInputDevice from its id.
  *
- * Return value: a #ClutterInputDevice, or %NULL
+ * Return value: (transfer none): a #ClutterInputDevice, or %NULL
  *
  * Since: 0.8
  */
@@ -2536,3 +2787,24 @@ clutter_get_input_device_for_id (gint id)
 
   return NULL;
 }
+
+/**
+ * clutter_get_font_map:
+ *
+ * Retrieves the #PangoFontMap instance used by Clutter.
+ * You can use the global font map object with the COGL
+ * Pango API.
+ *
+ * Return value: (transfer none): the #PangoFontMap instance. The returned
+ *   value is owned by Clutter and it should never be unreferenced.
+ *
+ * Since: 1.0
+ */
+PangoFontMap *
+clutter_get_font_map (void)
+{
+  if (CLUTTER_CONTEXT ()->font_map)
+    return PANGO_FONT_MAP (CLUTTER_CONTEXT ()->font_map);
+
+  return NULL;
+}
