@@ -18,16 +18,19 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _HAVE_CLUTTER_MAIN_H
-#define _HAVE_CLUTTER_MAIN_H
+#if !defined(__CLUTTER_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
+#error "Only <clutter/clutter.h> can be included directly."
+#endif
+
+#ifndef __CLUTTER_MAIN_H__
+#define __CLUTTER_MAIN_H__
 
 #include <clutter/clutter-actor.h>
 #include <clutter/clutter-stage.h>
+#include <pango/pango.h>
 
 G_BEGIN_DECLS
 
@@ -84,9 +87,9 @@ ClutterInitError clutter_init             (int          *argc,
                                            char       ***argv);
 ClutterInitError clutter_init_with_args   (int          *argc,
                                            char       ***argv,
-                                           char         *parameter_string,
+                                           const char   *parameter_string,
                                            GOptionEntry *entries,
-                                           char         *translation_domain,
+                                           const char   *translation_domain,
                                            GError      **error);
 GOptionGroup *   clutter_get_option_group (void);
 GOptionGroup *   clutter_get_option_group_without_init (void);
@@ -125,12 +128,12 @@ guint            clutter_threads_add_timeout_full   (gint           priority,
                                                      GSourceFunc    func,
                                                      gpointer       data,
                                                      GDestroyNotify notify);
-guint            clutter_threads_add_frame_source   (guint          interval,
+guint            clutter_threads_add_frame_source   (guint          fps,
 						     GSourceFunc    func,
 						     gpointer       data);
 guint            clutter_threads_add_frame_source_full
                                                     (gint           priority,
-						     guint          interval,
+						     guint          fps,
 						     GSourceFunc    func,
 						     gpointer       data,
 						     GDestroyNotify notify);
@@ -152,8 +155,8 @@ void             clutter_ungrab_keyboard             (void);
 ClutterActor *   clutter_get_keyboard_grab           (void);
 
 void             clutter_clear_glyph_cache           (void);
-void             clutter_set_use_mipmapped_text      (gboolean      value);
-gboolean         clutter_get_use_mipmapped_text      (void);
+void             clutter_set_font_flags              (ClutterFontFlags flags);
+ClutterFontFlags clutter_get_font_flags              (void);
 
 ClutterInputDevice*  clutter_get_input_device_for_id (gint id);
 
@@ -162,7 +165,8 @@ void             clutter_grab_pointer_for_device     (ClutterActor  *actor,
 
 void             clutter_ungrab_pointer_for_device   (gint id);
 
+PangoFontMap *   clutter_get_font_map                (void);
 
 G_END_DECLS
 
-#endif /* _HAVE_CLUTTER_MAIN_H */
+#endif /* _CLUTTER_MAIN_H__ */
