@@ -87,7 +87,87 @@ clutter_marshal_BOOLEAN__BOXED (GClosure     *closure,
   g_value_set_boolean (return_value, v_return);
 }
 
-/* UINT:VOID (./clutter-marshal.list:2) */
+/* BOOLEAN:STRING,UINT,ENUM (./clutter-marshal.list:2) */
+void
+clutter_marshal_BOOLEAN__STRING_UINT_ENUM (GClosure     *closure,
+                                           GValue       *return_value G_GNUC_UNUSED,
+                                           guint         n_param_values,
+                                           const GValue *param_values,
+                                           gpointer      invocation_hint G_GNUC_UNUSED,
+                                           gpointer      marshal_data)
+{
+  typedef gboolean (*GMarshalFunc_BOOLEAN__STRING_UINT_ENUM) (gpointer     data1,
+                                                              gpointer     arg_1,
+                                                              guint        arg_2,
+                                                              gint         arg_3,
+                                                              gpointer     data2);
+  register GMarshalFunc_BOOLEAN__STRING_UINT_ENUM callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gboolean v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 4);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_BOOLEAN__STRING_UINT_ENUM) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       g_marshal_value_peek_string (param_values + 1),
+                       g_marshal_value_peek_uint (param_values + 2),
+                       g_marshal_value_peek_enum (param_values + 3),
+                       data2);
+
+  g_value_set_boolean (return_value, v_return);
+}
+
+/* DOUBLE:VOID (./clutter-marshal.list:3) */
+void
+clutter_marshal_DOUBLE__VOID (GClosure     *closure,
+                              GValue       *return_value G_GNUC_UNUSED,
+                              guint         n_param_values,
+                              const GValue *param_values,
+                              gpointer      invocation_hint G_GNUC_UNUSED,
+                              gpointer      marshal_data)
+{
+  typedef gdouble (*GMarshalFunc_DOUBLE__VOID) (gpointer     data1,
+                                                gpointer     data2);
+  register GMarshalFunc_DOUBLE__VOID callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
+  gdouble v_return;
+
+  g_return_if_fail (return_value != NULL);
+  g_return_if_fail (n_param_values == 1);
+
+  if (G_CCLOSURE_SWAP_DATA (closure))
+    {
+      data1 = closure->data;
+      data2 = g_value_peek_pointer (param_values + 0);
+    }
+  else
+    {
+      data1 = g_value_peek_pointer (param_values + 0);
+      data2 = closure->data;
+    }
+  callback = (GMarshalFunc_DOUBLE__VOID) (marshal_data ? marshal_data : cc->callback);
+
+  v_return = callback (data1,
+                       data2);
+
+  g_value_set_double (return_value, v_return);
+}
+
+/* UINT:VOID (./clutter-marshal.list:4) */
 void
 clutter_marshal_UINT__VOID (GClosure     *closure,
                             GValue       *return_value G_GNUC_UNUSED,
@@ -124,11 +204,11 @@ clutter_marshal_UINT__VOID (GClosure     *closure,
   g_value_set_uint (return_value, v_return);
 }
 
-/* VOID:BOXED (./clutter-marshal.list:3) */
+/* VOID:BOXED (./clutter-marshal.list:5) */
 
-/* VOID:INT (./clutter-marshal.list:4) */
+/* VOID:INT (./clutter-marshal.list:6) */
 
-/* VOID:INT64,INT64,FLOAT,BOOLEAN (./clutter-marshal.list:5) */
+/* VOID:INT64,INT64,FLOAT,BOOLEAN (./clutter-marshal.list:7) */
 void
 clutter_marshal_VOID__INT64_INT64_FLOAT_BOOLEAN (GClosure     *closure,
                                                  GValue       *return_value G_GNUC_UNUSED,
@@ -169,7 +249,7 @@ clutter_marshal_VOID__INT64_INT64_FLOAT_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,INT (./clutter-marshal.list:6) */
+/* VOID:INT,INT (./clutter-marshal.list:8) */
 void
 clutter_marshal_VOID__INT_INT (GClosure     *closure,
                                GValue       *return_value G_GNUC_UNUSED,
@@ -206,7 +286,7 @@ clutter_marshal_VOID__INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:INT,INT,INT,INT (./clutter-marshal.list:7) */
+/* VOID:INT,INT,INT,INT (./clutter-marshal.list:9) */
 void
 clutter_marshal_VOID__INT_INT_INT_INT (GClosure     *closure,
                                        GValue       *return_value G_GNUC_UNUSED,
@@ -247,9 +327,9 @@ clutter_marshal_VOID__INT_INT_INT_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT (./clutter-marshal.list:8) */
+/* VOID:OBJECT (./clutter-marshal.list:10) */
 
-/* VOID:OBJECT,OBJECT,PARAM (./clutter-marshal.list:9) */
+/* VOID:OBJECT,OBJECT,PARAM (./clutter-marshal.list:11) */
 void
 clutter_marshal_VOID__OBJECT_OBJECT_PARAM (GClosure     *closure,
                                            GValue       *return_value G_GNUC_UNUSED,
@@ -288,7 +368,7 @@ clutter_marshal_VOID__OBJECT_OBJECT_PARAM (GClosure     *closure,
             data2);
 }
 
-/* VOID:OBJECT,POINTER (./clutter-marshal.list:10) */
+/* VOID:OBJECT,POINTER (./clutter-marshal.list:12) */
 void
 clutter_marshal_VOID__OBJECT_POINTER (GClosure     *closure,
                                       GValue       *return_value G_GNUC_UNUSED,
@@ -325,7 +405,7 @@ clutter_marshal_VOID__OBJECT_POINTER (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,BOOLEAN,BOOLEAN (./clutter-marshal.list:11) */
+/* VOID:STRING,BOOLEAN,BOOLEAN (./clutter-marshal.list:13) */
 void
 clutter_marshal_VOID__STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
                                               GValue       *return_value G_GNUC_UNUSED,
@@ -364,7 +444,7 @@ clutter_marshal_VOID__STRING_BOOLEAN_BOOLEAN (GClosure     *closure,
             data2);
 }
 
-/* VOID:STRING,INT (./clutter-marshal.list:12) */
+/* VOID:STRING,INT (./clutter-marshal.list:14) */
 void
 clutter_marshal_VOID__STRING_INT (GClosure     *closure,
                                   GValue       *return_value G_GNUC_UNUSED,
@@ -401,5 +481,7 @@ clutter_marshal_VOID__STRING_INT (GClosure     *closure,
             data2);
 }
 
-/* VOID:VOID (./clutter-marshal.list:13) */
+/* VOID:UINT (./clutter-marshal.list:15) */
+
+/* VOID:VOID (./clutter-marshal.list:16) */
 
