@@ -20,15 +20,16 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#if !defined(__CLUTTER_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
+#error "Only <clutter/clutter.h> can be included directly."
+#endif
+
 #ifndef __CLUTTER_MODEL_H__
 #define __CLUTTER_MODEL_H__
 
-#include <glib.h>
 #include <glib-object.h>
 
 G_BEGIN_DECLS
@@ -300,6 +301,7 @@ struct _ClutterModelIter
  *   iterator belongs to
  * @get_row: Virtual function for getting the row to which the iterator
  *   points
+ * @copy: Virtual function for copying a #ClutterModelIter.
  *
  * Class for #ClutterModelIter instances.
  *
@@ -325,9 +327,11 @@ struct _ClutterModelIterClass
   ClutterModelIter *(* next)      (ClutterModelIter *iter);
   ClutterModelIter *(* prev)      (ClutterModelIter *iter);
 
-  ClutterModel*     (* get_model) (ClutterModelIter *iter);
+  ClutterModel *    (* get_model) (ClutterModelIter *iter);
   guint             (* get_row)   (ClutterModelIter *iter);
 
+  ClutterModelIter *(* copy)      (ClutterModelIter *iter);
+
   /*< private >*/
   /* padding for future */
   void (*_clutter_model_iter_1) (void);
@@ -361,6 +365,7 @@ ClutterModelIter *clutter_model_iter_prev       (ClutterModelIter *iter);
 ClutterModel *    clutter_model_iter_get_model  (ClutterModelIter *iter);
 guint             clutter_model_iter_get_row    (ClutterModelIter *iter);
 
+ClutterModelIter *clutter_model_iter_copy       (ClutterModelIter *iter);
 
 G_END_DECLS
 
