@@ -18,18 +18,20 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#if !defined(__CLUTTER_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
+#error "Only <clutter/clutter.h> can be included directly."
+#endif
+
 #ifndef __CLUTTER_STAGE_H__
 #define __CLUTTER_STAGE_H__
 
+#include <clutter/clutter-types.h>
 #include <clutter/clutter-group.h>
 #include <clutter/clutter-color.h>
 #include <clutter/clutter-event.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
 
 G_BEGIN_DECLS
 
@@ -60,7 +62,7 @@ G_BEGIN_DECLS
 /**
  * CLUTTER_STAGE_WIDTH:
  *
- * Macro that evaluates to the current stage width
+ * Macro that evaluates to the width of the default stage
  *
  * Since: 0.2
  */
@@ -70,7 +72,7 @@ G_BEGIN_DECLS
 /**
  * CLUTTER_STAGE_HEIGHT:
  *
- * Macro that evaluates to the current stage height
+ * Macro that evaluates to the height of the default stage
  *
  * Since: 0.2
  */
@@ -80,10 +82,17 @@ G_BEGIN_DECLS
 typedef struct _ClutterPerspective  ClutterPerspective;
 typedef struct _ClutterFog          ClutterFog;
 
-typedef struct _ClutterStage        ClutterStage;
 typedef struct _ClutterStageClass   ClutterStageClass;
 typedef struct _ClutterStagePrivate ClutterStagePrivate;
 
+/**
+ * ClutterStage:
+ *
+ * The #ClutterStage structure contains only private data
+ * and should be accessed using the provided API
+ *
+ * Since: 0.1
+ */
 struct _ClutterStage
 {
   /*< private >*/
@@ -91,6 +100,17 @@ struct _ClutterStage
 
   ClutterStagePrivate *priv;
 };
+/**
+ * ClutterStageClass:
+ * @fullscreen: handler for the #ClutterStage::fullscreen signal
+ * @unfullscreen: handler for the #ClutterStage::unfullscreen signal
+ * @activate: handler for the #ClutterStage::activate signal
+ * @deactivate: handler for the #ClutterStage::deactive signal
+ *
+ * The #ClutterStageClass structure contains only private data
+ *
+ * Since: 0.1
+ */
 
 struct _ClutterStageClass
 {
@@ -98,26 +118,11 @@ struct _ClutterStageClass
   ClutterGroupClass parent_class;
 
   /*< public >*/
-  /* vfuncs, not signals */
-  void          (* set_fullscreen)     (ClutterStage *stage,
-                                        gboolean      fullscreen);
-  void          (* set_cursor_visible) (ClutterStage *stage,
-                                        gboolean      visible);
-  GdkPixbuf*    (* draw_to_pixbuf)     (ClutterStage *stage,
-                                        gint          x,
-                                        gint          y,
-                                        gint          width,
-                                        gint          height);
-  void          (* set_title)          (ClutterStage *stage,
-                                        const gchar  *title);
-  void          (* set_user_resize)    (ClutterStage *stage,
-                                        gboolean      value);
-
-  /* events */
-  void     (* fullscreen)      (ClutterStage           *stage);
-  void     (* unfullscreen)    (ClutterStage           *stage);
-  void     (* activate)        (ClutterStage           *stage);
-  void     (* deactivate)      (ClutterStage           *stage);
+  /* signals */
+  void (* fullscreen)   (ClutterStage *stage);
+  void (* unfullscreen) (ClutterStage *stage);
+  void (* activate)     (ClutterStage *stage);
+  void (* deactivate)   (ClutterStage *stage);
 
   /*< private >*/
   /* padding for future expansion */
@@ -143,30 +148,27 @@ struct _ClutterStageClass
  */
 struct _ClutterPerspective
 {
-  ClutterFixed fovy;
-  ClutterFixed aspect;
-  ClutterFixed z_near;
-  ClutterFixed z_far;
+  gfloat fovy;
+  gfloat aspect;
+  gfloat z_near;
+  gfloat z_far;
 };
 
 /**
  * ClutterFog:
- * @density: density of the fog
  * @z_near: starting distance from the viewer to the near clipping
  *   plane (always positive)
  * @z_far: final distance from the viewer to the far clipping
  *   plane (always positive)
  *
- * Fog settings used to create the depth cueing effect. #ClutterFog is
- * useful only when using the fixed point API.
+ * Fog settings used to create the depth cueing effect.
  *
  * Since: 0.6
  */
 struct _ClutterFog
 {
-  ClutterFixed density;
-  ClutterFixed z_near;
-  ClutterFixed z_far;
+  gfloat z_near;
+  gfloat z_far;
 };
 
 GType         clutter_perspective_get_type    (void) G_GNUC_CONST;
@@ -174,24 +176,16 @@ GType         clutter_fog_get_type            (void) G_GNUC_CONST;
 GType         clutter_stage_get_type          (void) G_GNUC_CONST;
 
 ClutterActor *clutter_stage_get_default       (void);
+ClutterActor *clutter_stage_new               (void);
+
 void          clutter_stage_set_color         (ClutterStage       *stage,
                                                const ClutterColor *color);
 void          clutter_stage_get_color         (ClutterStage       *stage,
                                                ClutterColor       *color);
-void          clutter_stage_set_perspectivex  (ClutterStage       *stage,
-			                       ClutterPerspective *perspective);
-void          clutter_stage_get_perspectivex  (ClutterStage       *stage,
-			                       ClutterPerspective *perspective);
 void          clutter_stage_set_perspective   (ClutterStage       *stage,
-					       gfloat              fovy,
-					       gfloat              aspect,
-					       gfloat              z_near,
-					       gfloat              z_far);
+			                       ClutterPerspective *perspective);
 void          clutter_stage_get_perspective   (ClutterStage       *stage,
-					       gfloat             *fovy,
-					       gfloat             *aspect,
-					       gfloat             *z_near,
-					       gfloat             *z_far);
+			                       ClutterPerspective *perspective);
 void          clutter_stage_fullscreen        (ClutterStage       *stage);
 void          clutter_stage_unfullscreen      (ClutterStage       *stage);
 void          clutter_stage_show_cursor       (ClutterStage       *stage);
@@ -200,7 +194,7 @@ void          clutter_stage_hide_cursor       (ClutterStage       *stage);
 ClutterActor *clutter_stage_get_actor_at_pos  (ClutterStage       *stage,
                                                gint                x,
                                                gint                y);
-GdkPixbuf *   clutter_stage_snapshot          (ClutterStage       *stage,
+guchar *      clutter_stage_read_pixels       (ClutterStage       *stage,
                                                gint                x,
                                                gint                y,
                                                gint                width,
@@ -218,25 +212,20 @@ void                  clutter_stage_set_use_fog        (ClutterStage *stage,
                                                         gboolean      fog);
 gboolean              clutter_stage_get_use_fog        (ClutterStage *stage);
 void                  clutter_stage_set_fog            (ClutterStage *stage,
-                                                        gdouble       density,
-                                                        gdouble       z_near,
-                                                        gdouble       z_far);
+                                                        ClutterFog   *fog);
 void                  clutter_stage_get_fog            (ClutterStage *stage,
-                                                        gdouble      *density,
-                                                        gdouble      *z_near,
-                                                        gdouble      *z_far);
-void                  clutter_stage_set_fogx           (ClutterStage *stage,
                                                         ClutterFog   *fog);
-void                  clutter_stage_get_fogx           (ClutterStage *stage,
-                                                        ClutterFog   *fog);
-gdouble               clutter_stage_get_resolution     (ClutterStage *stage);
-ClutterFixed          clutter_stage_get_resolutionx    (ClutterStage *stage);
 
-/* New experiental calls */
 void                  clutter_stage_set_key_focus      (ClutterStage *stage,
                                                         ClutterActor *actor);
 ClutterActor *        clutter_stage_get_key_focus      (ClutterStage *stage);
 
+/* New experiental calls */
+void                  clutter_stage_ensure_current     (ClutterStage *stage);
+void                  clutter_stage_queue_redraw       (ClutterStage *stage);
+gboolean              clutter_stage_is_default         (ClutterStage *stage);
+void                  clutter_stage_ensure_viewport    (ClutterStage *stage);
+
 /* Commodity macro */
 #define clutter_stage_add(stage,actor)                  G_STMT_START {  \
   if (CLUTTER_IS_STAGE ((stage)) && CLUTTER_IS_ACTOR ((actor)))         \
