@@ -40,33 +40,34 @@
  * your application.
  *
  * One way to visualise a timeline is as a path with marks along its length.
- * When creating a timeline of @n_frames via clutter_timeline_new(), then the
- * number of frames can be seen as the paths length, and each unit of length
- * (each frame) is delimited by a mark.
+ * When creating a timeline of N frames via clutter_timeline_new(), then the
+ * number of frames can be seen as the paths length, and each unit of
+ * length (each frame) is delimited by a mark.
  *
- * For a non looping timeline there will be (n_frames + 1) marks along its
+ * For a non looping timeline there will be (N frames + 1) marks along its
  * length. For a looping timeline, the two ends are joined with one mark.
  * Technically this mark represents two discrete frame numbers, but for a
  * looping timeline the start and end frame numbers are considered equivalent.
  *
- * When you create a timeline it starts with
- * clutter_timeline_get_current_frame() == 0.
+ * When you create a timeline it will be initialized so that the current
+ * frame, as returned by clutter_timeline_get_current_frame(), will be 0.
  *
- * After starting a timeline, the first timeout is for current_frame_num == 1
- * (Notably it isn't 0 since there is a delay before the first timeout signals
- * so re-asserting the starting frame (0) wouldn't make sense.)
- * Notably, this implies that actors you intend to be affected by the
- * timeline's progress, should be manually primed/positioned for frame 0 which
- * will be displayed before the first timeout. (If you are not careful about
+ * After starting a timeline, the first timeout is for frame number
+ * one (notably it isn't zero since there is a delay before the first
+ * #ClutterTimeline::new-frame signal, so re-asserting the frame number
+ * zero wouldn't make sense).
+ *
+ * This implies that actors you intend to be affected by the timeline's
+ * progress should be manually primed or positioned for frame zero which
+ * will be displayed before the first timeout (if you are not careful about
  * this point you will likely see flashes of incorrect actor state in your
- * program)
+ * program).
  *
- * For a non looping timeline the last timeout would be for
- * current_frame_num == @n_frames
+ * For a non looping timeline the last timeout would be for the number
+ * of frames in the timeline, as returned by clutter_timeline_get_n_frames().
  *
- * For a looping timeline the timeout for current_frame_num == @n_frames would
- * be followed by a timeout for current_frame_num == 1 (remember frame 0 is
- * considered == frame (@n_frames)).
+ * For a looping timeline the timeout for the last frame would be followed
+ * by a timeout for frame number 1.
  *
  * There may be times when a system is not able to meet the frame rate
  * requested for a timeline, and in this case the frame number will be
@@ -74,9 +75,8 @@
  * the time that the timeline was started, not from the time of the last
  * timeout, so a given timeline should basically elapse in the same - real
  * world - time on any given system. An invariable here though is that
- * current_frame_num == @n_frames will always be signaled, but notably frame 1
- * can be interpolated past and so never signaled.
- *
+ * the last frame will always be signaled, but notably frame number 1 can
+ * be interpolated past and thus never signaled.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -93,8 +93,6 @@
 
 G_DEFINE_TYPE (ClutterTimeline, clutter_timeline, G_TYPE_OBJECT);
 
-#define FPS_TO_INTERVAL(f) (1000 / (f))
-
 struct _ClutterTimelinePrivate
 {
   ClutterTimelineDirection direction;
@@ -111,7 +109,7 @@ struct _ClutterTimelinePrivate
   gint skipped_frames;
 
   GTimeVal prev_frame_timeval;
-  guint  msecs_delta;
+  guint msecs_delta;
 
   GHashTable *markers_by_frame;
   GHashTable *markers_by_name;
@@ -148,9 +146,9 @@ enum
   LAST_SIGNAL
 };
 
-static guint               timeline_signals[LAST_SIGNAL] = { 0 };
-static gint                timeline_use_pool = -1;
-static ClutterTimeoutPool *timeline_pool = NULL;
+static guint               timeline_signals[LAST_SIGNAL] = { 0, };
+static gint                timeline_use_pool             = -1;
+static ClutterTimeoutPool *timeline_pool                 = NULL;
 
 static inline void
 timeline_pool_init (void)
@@ -174,7 +172,7 @@ timeline_pool_init (void)
 }
 
 static guint
-timeout_add (guint          interval,
+timeout_add (guint          fps,
              GSourceFunc    func,
              gpointer       data,
              GDestroyNotify notify)
@@ -185,13 +183,13 @@ timeout_add (guint          interval,
     {
       g_assert (timeline_pool != NULL);
       res = clutter_timeout_pool_add (timeline_pool,
-                                      interval,
+                                      fps,
                                       func, data, notify);
     }
   else
     {
       res = clutter_threads_add_frame_source_full (CLUTTER_PRIORITY_TIMELINE,
-						   interval,
+						   fps,
 						   func, data, notify);
     }
 
@@ -254,21 +252,27 @@ clutter_timeline_set_property (GObject      *object,
     case PROP_FPS:
       clutter_timeline_set_speed (timeline, g_value_get_uint (value));
       break;
+
     case PROP_NUM_FRAMES:
       clutter_timeline_set_n_frames (timeline, g_value_get_uint (value));
       break;
+
     case PROP_LOOP:
       priv->loop = g_value_get_boolean (value);
       break;
+
     case PROP_DELAY:
       priv->delay = g_value_get_uint (value);
       break;
+
     case PROP_DURATION:
       clutter_timeline_set_duration (timeline, g_value_get_uint (value));
       break;
+
     case PROP_DIRECTION:
       clutter_timeline_set_direction (timeline, g_value_get_enum (value));
       break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -292,21 +296,27 @@ clutter_timeline_get_property (GObject    *object,
     case PROP_FPS:
       g_value_set_uint (value, priv->fps);
       break;
+
     case PROP_NUM_FRAMES:
       g_value_set_uint (value, priv->n_frames);
       break;
+
     case PROP_LOOP:
       g_value_set_boolean (value, priv->loop);
       break;
+
     case PROP_DELAY:
       g_value_set_uint (value, priv->delay);
       break;
+
     case PROP_DURATION:
       g_value_set_uint (value, clutter_timeline_get_duration (timeline));
       break;
+
     case PROP_DIRECTION:
       g_value_set_enum (value, priv->direction);
       break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -318,8 +328,11 @@ clutter_timeline_finalize (GObject *object)
 {
   ClutterTimelinePrivate *priv = CLUTTER_TIMELINE (object)->priv;
 
-  g_hash_table_destroy (priv->markers_by_frame);
-  g_hash_table_destroy (priv->markers_by_name);
+  if (priv->markers_by_frame)
+    g_hash_table_destroy (priv->markers_by_frame);
+
+  if (priv->markers_by_name)
+    g_hash_table_destroy (priv->markers_by_name);
 
   G_OBJECT_CLASS (clutter_timeline_parent_class)->finalize (object);
 }
@@ -351,6 +364,7 @@ static void
 clutter_timeline_class_init (ClutterTimelineClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GParamSpec *pspec;
 
   timeline_pool_init ();
 
@@ -364,42 +378,45 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
   /**
    * ClutterTimeline:fps:
    *
-   * Timeline frames per second. Because of the nature of the main
-   * loop used by Clutter this is to be considered a best approximation.
+   * Number of frames per second. Because of the nature of the main
+   * loop used by Clutter, we can only accept a granularity of one
+   * frame per millisecond.
+   *
+   * This value is to be considered a best approximation.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_FPS,
-                                   g_param_spec_uint ("fps",
-                                                      "Frames Per Second",
-                                                      "Timeline frames per second",
-                                                      1, G_MAXUINT,
-                                                      60,
-                                                      CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_uint ("fps",
+                             "Frames Per Second",
+                             "Frames per second",
+                             1, 1000,
+                             60,
+                             CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_FPS, pspec);
+
   /**
    * ClutterTimeline:num-frames:
    *
    * Total number of frames for the timeline.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_NUM_FRAMES,
-                                   g_param_spec_uint ("num-frames",
-                                                      "Total number of frames",
-                                                      "Timelines total number of frames",
-                                                      1, G_MAXUINT,
-                                                      1,
-                                                      CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_uint ("num-frames",
+                             "Total number of frames",
+                             "Total number of frames",
+                             1, G_MAXUINT,
+                             1,
+                             CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_NUM_FRAMES, pspec);
+
   /**
    * ClutterTimeline:loop:
    *
    * Whether the timeline should automatically rewind and restart.
    */
-  g_object_class_install_property (object_class,
-                                   PROP_LOOP,
-                                   g_param_spec_boolean ("loop",
-                                                         "Loop",
-                                                         "Should the timeline automatically restart",
-                                                         FALSE,
-                                                         CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_boolean ("loop",
+                                "Loop",
+                                "Should the timeline automatically restart",
+                                FALSE,
+                                CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_LOOP, pspec);
+
   /**
    * ClutterTimeline:delay:
    *
@@ -408,14 +425,14 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
    *
    * Since: 0.4
    */
-  g_object_class_install_property (object_class,
-                                   PROP_DELAY,
-                                   g_param_spec_uint ("delay",
-                                                      "Delay",
-                                                      "Delay before start",
-                                                      0, G_MAXUINT,
-                                                      0,
-                                                      CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_uint ("delay",
+                             "Delay",
+                             "Delay before start",
+                             0, G_MAXUINT,
+                             0,
+                             CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_DELAY, pspec);
+
   /**
    * ClutterTimeline:duration:
    *
@@ -424,14 +441,14 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
    *
    * Since: 0.6
    */
-  g_object_class_install_property (object_class,
-                                   PROP_DURATION,
-                                   g_param_spec_uint ("duration",
-                                                      "Duration",
-                                                      "Duration of the timeline in milliseconds",
-                                                      0, G_MAXUINT,
-                                                      1000,
-                                                      CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_uint ("duration",
+                             "Duration",
+                             "Duration of the timeline in milliseconds",
+                             0, G_MAXUINT,
+                             1000,
+                             CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_DURATION, pspec);
+
   /**
    * ClutterTimeline:direction:
    *
@@ -440,14 +457,13 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
    *
    * Since: 0.6
    */
-  g_object_class_install_property (object_class,
-                                   PROP_DIRECTION,
-                                   g_param_spec_enum ("direction",
-                                                      "Direction",
-                                                      "Direction of the timeline",
-                                                      CLUTTER_TYPE_TIMELINE_DIRECTION,
-                                                      CLUTTER_TIMELINE_FORWARD,
-                                                      CLUTTER_PARAM_READWRITE));
+  pspec = g_param_spec_enum ("direction",
+                             "Direction",
+                             "Direction of the timeline",
+                             CLUTTER_TYPE_TIMELINE_DIRECTION,
+                             CLUTTER_TIMELINE_FORWARD,
+                             CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (object_class, PROP_DIRECTION, pspec);
 
   /**
    * ClutterTimeline::new-frame:
@@ -459,7 +475,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
    * timeline is reached.
    */
   timeline_signals[NEW_FRAME] =
-    g_signal_new ("new-frame",
+    g_signal_new (I_("new-frame"),
 		  G_TYPE_FROM_CLASS (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterTimelineClass, new_frame),
@@ -475,7 +491,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
    * number of frames specified by the ClutterTimeline:num-frames property.
    */
   timeline_signals[COMPLETED] =
-    g_signal_new ("completed",
+    g_signal_new (I_("completed"),
 		  G_TYPE_FROM_CLASS (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterTimelineClass, completed),
@@ -492,7 +508,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
    * expired.
    */
   timeline_signals[STARTED] =
-    g_signal_new ("started",
+    g_signal_new (I_("started"),
 		  G_TYPE_FROM_CLASS (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterTimelineClass, started),
@@ -506,7 +522,7 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
    * The ::paused signal is emitted when clutter_timeline_pause() is invoked.
    */
   timeline_signals[PAUSED] =
-    g_signal_new ("paused",
+    g_signal_new (I_("paused"),
 		  G_TYPE_FROM_CLASS (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (ClutterTimelineClass, paused),
@@ -545,9 +561,10 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
    * Since: 0.8
    */
   timeline_signals[MARKER_REACHED] =
-    g_signal_new ("marker-reached",
+    g_signal_new (I_("marker-reached"),
                   G_TYPE_FROM_CLASS (object_class),
-                  G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE | G_SIGNAL_DETAILED | G_SIGNAL_NO_HOOKS,
+                  G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE |
+                  G_SIGNAL_DETAILED | G_SIGNAL_NO_HOOKS,
                   G_STRUCT_OFFSET (ClutterTimelineClass, marker_reached),
                   NULL, NULL,
                   clutter_marshal_VOID__STRING_INT,
@@ -561,18 +578,13 @@ clutter_timeline_init (ClutterTimeline *self)
 {
   ClutterTimelinePrivate *priv;
 
-  self->priv = priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
-                                                   CLUTTER_TYPE_TIMELINE,
-                                                   ClutterTimelinePrivate);
+  self->priv = priv =
+    G_TYPE_INSTANCE_GET_PRIVATE (self, CLUTTER_TYPE_TIMELINE,
+                                 ClutterTimelinePrivate);
 
   priv->fps = clutter_get_default_frame_rate ();
   priv->n_frames = 0;
   priv->msecs_delta = 0;
-
-  priv->markers_by_frame = g_hash_table_new (NULL, NULL);
-  priv->markers_by_name = g_hash_table_new_full (g_str_hash, g_str_equal,
-                                                 NULL,
-                                                 timeline_marker_free);
 }
 
 static void
@@ -584,12 +596,18 @@ emit_frame_signal (ClutterTimeline *timeline)
   g_signal_emit (timeline, timeline_signals[NEW_FRAME], 0,
                  priv->current_frame_num);
 
+  /* shortcircuit here if we don't have any marker installed */
+  if (priv->markers_by_name == NULL ||
+      priv->markers_by_frame == NULL)
+    return;
+
   for (i = priv->skipped_frames; i >= 0; i--)
     {
-      gint frame_num = priv->current_frame_num
-        + (priv->direction == CLUTTER_TIMELINE_FORWARD ? -i : i);
+      gint frame_num;
       GSList *markers, *l;
 
+      frame_num = priv->current_frame_num
+                + (priv->direction == CLUTTER_TIMELINE_FORWARD ? -i : i);
       markers = g_hash_table_lookup (priv->markers_by_frame,
                                      GUINT_TO_POINTER (frame_num));
       for (l = markers; l; l = l->next)
@@ -606,13 +624,24 @@ emit_frame_signal (ClutterTimeline *timeline)
     }
 }
 
+static gboolean
+is_complete (ClutterTimeline *timeline)
+{
+  ClutterTimelinePrivate *priv = timeline->priv;
+
+  return ((priv->direction == CLUTTER_TIMELINE_FORWARD) &&
+          (priv->current_frame_num >= priv->n_frames)) ||
+         ((priv->direction == CLUTTER_TIMELINE_BACKWARD) &&
+          (priv->current_frame_num <= 0));
+}
+
 static gboolean
 timeline_timeout_func (gpointer data)
 {
   ClutterTimeline        *timeline = data;
   ClutterTimelinePrivate *priv;
   GTimeVal                timeval;
-  guint                   n_frames;
+  guint                   n_frames, speed;
   gulong                  msecs;
 
   priv = timeline->priv;
@@ -639,7 +668,11 @@ timeline_timeout_func (gpointer data)
   msecs = (timeval.tv_sec - priv->prev_frame_timeval.tv_sec) * 1000;
   msecs += (timeval.tv_usec - priv->prev_frame_timeval.tv_usec) / 1000;
   priv->msecs_delta = msecs;
-  n_frames = msecs / (1000 / priv->fps);
+
+  /* we need to avoid fps > 1000 */
+  speed = MAX (1000 / priv->fps, 1);
+
+  n_frames = msecs / speed;
   if (n_frames == 0)
     n_frames = 1;
 
@@ -660,12 +693,7 @@ timeline_timeout_func (gpointer data)
     priv->current_frame_num -= n_frames;
 
   /* If we have not reached the end of the timeline: */
-  if (!(
-      ((priv->direction == CLUTTER_TIMELINE_FORWARD) &&
-       (priv->current_frame_num >= priv->n_frames)) ||
-      ((priv->direction == CLUTTER_TIMELINE_BACKWARD) &&
-       (priv->current_frame_num <= 0))
-       ))
+  if (!is_complete (timeline))
     {
       /* Emit the signal */
       emit_frame_signal (timeline);
@@ -728,7 +756,7 @@ timeline_timeout_func (gpointer data)
           /* We remove the timeout now, so that the completed signal handler
            * may choose to re-start the timeline
            *
-           * ** Perhaps we should remove this earlier, and regardless
+           * XXX Perhaps we should remove this earlier, and regardless
            * of priv->loop. Are we limiting the things that could be done in
            * the above new-frame signal handler */
           timeout_remove (priv->timeout_id);
@@ -784,10 +812,10 @@ timeline_timeout_func (gpointer data)
 
 static guint
 timeline_timeout_add (ClutterTimeline *timeline,
-                      guint          interval,
-                      GSourceFunc    func,
-                      gpointer       data,
-                      GDestroyNotify notify)
+                      guint            fps,
+                      GSourceFunc      func,
+                      gpointer         data,
+                      GDestroyNotify   notify)
 {
   ClutterTimelinePrivate *priv;
   GTimeVal timeval;
@@ -799,10 +827,11 @@ timeline_timeout_add (ClutterTimeline *timeline,
       g_get_current_time (&timeval);
       priv->prev_frame_timeval = timeval;
     }
+
   priv->skipped_frames   = 0;
   priv->msecs_delta      = 0;
 
-  return timeout_add (interval, func, data, notify);
+  return timeout_add (fps, func, data, notify);
 }
 
 static gboolean
@@ -814,7 +843,7 @@ delay_timeout_func (gpointer data)
   priv->delay_id = 0;
 
   priv->timeout_id = timeline_timeout_add (timeline,
-                                           FPS_TO_INTERVAL (priv->fps),
+                                           priv->fps,
                                            timeline_timeout_func,
                                            timeline, NULL);
 
@@ -853,7 +882,7 @@ clutter_timeline_start (ClutterTimeline *timeline)
   else
     {
       priv->timeout_id = timeline_timeout_add (timeline,
-                                               FPS_TO_INTERVAL (priv->fps),
+                                               priv->fps,
                                                timeline_timeout_func,
                                                timeline, NULL);
 
@@ -949,9 +978,9 @@ clutter_timeline_get_loop (ClutterTimeline *timeline)
  * @timeline: A #ClutterTimeline
  *
  * Rewinds #ClutterTimeline to the first frame if its direction is
- * CLUTTER_TIMELINE_FORWARD and the last frame if it is
- * CLUTTER_TIMELINE_BACKWARD.
- **/
+ * %CLUTTER_TIMELINE_FORWARD and the last frame if it is
+ * %CLUTTER_TIMELINE_BACKWARD.
+ */
 void
 clutter_timeline_rewind (ClutterTimeline *timeline)
 {
@@ -972,8 +1001,8 @@ clutter_timeline_rewind (ClutterTimeline *timeline)
  * @timeline: A #ClutterTimeline
  * @n_frames: Number of frames to skip
  *
- * Advance timeline by requested number of frames.
- **/
+ * Advance timeline by the requested number of frames.
+ */
 void
 clutter_timeline_skip (ClutterTimeline *timeline,
                        guint            n_frames)
@@ -1005,8 +1034,13 @@ clutter_timeline_skip (ClutterTimeline *timeline,
  * @timeline: A #ClutterTimeline
  * @frame_num: Frame number to advance to
  *
- * Advance timeline to requested frame number
- **/
+ * Advance timeline to the requested frame number.
+ *
+ * <note><para>The @timeline will not emit the #ClutterTimeline::new-frame
+ * signal for @frame_num. The first ::new-frame signal after the call to
+ * clutter_timeline_advance() will be emitted for a frame following
+ * @frame_num.</para></note>
+ */
 void
 clutter_timeline_advance (ClutterTimeline *timeline,
                           guint            frame_num)
@@ -1026,8 +1060,8 @@ clutter_timeline_advance (ClutterTimeline *timeline,
  *
  * Request the current frame number of the timeline.
  *
- * Return Value: current frame number
- **/
+ * Return value: current frame number
+ */
 gint
 clutter_timeline_get_current_frame (ClutterTimeline *timeline)
 {
@@ -1042,8 +1076,8 @@ clutter_timeline_get_current_frame (ClutterTimeline *timeline)
  *
  * Request the total number of frames for the #ClutterTimeline.
  *
- * Return Value: Number of frames for this #ClutterTimeline.
- **/
+ * Return value: the number of frames
+ */
 guint
 clutter_timeline_get_n_frames (ClutterTimeline *timeline)
 {
@@ -1072,8 +1106,6 @@ clutter_timeline_set_n_frames (ClutterTimeline *timeline,
 
   if (priv->n_frames != n_frames)
     {
-      g_object_ref (timeline);
-
       g_object_freeze_notify (G_OBJECT (timeline));
 
       priv->n_frames = n_frames;
@@ -1082,17 +1114,17 @@ clutter_timeline_set_n_frames (ClutterTimeline *timeline,
       g_object_notify (G_OBJECT (timeline), "duration");
 
       g_object_thaw_notify (G_OBJECT (timeline));
-      g_object_unref (timeline);
     }
 }
 
 /**
  * clutter_timeline_set_speed:
  * @timeline: A #ClutterTimeline
- * @fps: New speed of timeline as frames per second
+ * @fps: New speed of timeline as frames per second,
+ *    between 1 and 1000
  *
- * Set the speed in frames per second of the timeline.
- **/
+ * Sets the speed of @timeline in frames per second.
+ */
 void
 clutter_timeline_set_speed (ClutterTimeline *timeline,
                             guint            fps)
@@ -1100,7 +1132,7 @@ clutter_timeline_set_speed (ClutterTimeline *timeline,
   ClutterTimelinePrivate *priv;
 
   g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
-  g_return_if_fail (fps > 0);
+  g_return_if_fail (fps > 0 && fps <= 1000);
 
   priv = timeline->priv;
 
@@ -1116,7 +1148,7 @@ clutter_timeline_set_speed (ClutterTimeline *timeline,
           timeout_remove (priv->timeout_id);
 
           priv->timeout_id = timeline_timeout_add (timeline,
-                                                   FPS_TO_INTERVAL (priv->fps),
+                                                   priv->fps,
                                                    timeline_timeout_func,
                                                    timeline, NULL);
         }
@@ -1152,9 +1184,9 @@ clutter_timeline_get_speed (ClutterTimeline *timeline)
  * clutter_timeline_is_playing:
  * @timeline: A #ClutterTimeline
  *
- * Query state of a #ClutterTimeline instance.
+ * Queries state of a #ClutterTimeline.
  *
- * Return Value: TRUE if timeline is currently playing, FALSE if not.
+ * Return value: %TRUE if timeline is currently playing
  */
 gboolean
 clutter_timeline_is_playing (ClutterTimeline *timeline)
@@ -1175,7 +1207,7 @@ clutter_timeline_is_playing (ClutterTimeline *timeline)
  *
  * Return Value: a new #ClutterTimeline, cloned from @timeline
  *
- * Since 0.4
+ * Since: 0.4
  */
 ClutterTimeline *
 clutter_timeline_clone (ClutterTimeline *timeline)
@@ -1229,7 +1261,7 @@ clutter_timeline_new (guint n_frames,
 		      guint fps)
 {
   g_return_val_if_fail (n_frames > 0, NULL);
-  g_return_val_if_fail (fps > 0, NULL);
+  g_return_val_if_fail (fps > 0 && fps <= 1000, NULL);
 
   return g_object_new (CLUTTER_TYPE_TIMELINE,
 		       "fps", fps,
@@ -1329,6 +1361,7 @@ clutter_timeline_set_duration (ClutterTimeline *timeline,
   if (n_frames < 1)
     n_frames = 1;
 
+  /* this will notify :duration as well */
   clutter_timeline_set_n_frames (timeline, n_frames);
 }
 
@@ -1345,9 +1378,13 @@ clutter_timeline_set_duration (ClutterTimeline *timeline,
 gdouble
 clutter_timeline_get_progress (ClutterTimeline *timeline)
 {
-  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), 0.);
+  ClutterTimelinePrivate *priv;
 
-  return CLUTTER_FIXED_TO_DOUBLE (clutter_timeline_get_progressx (timeline));
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), 0.0);
+
+  priv = timeline->priv;
+
+  return (gdouble) priv->current_frame_num / (gdouble) priv->n_frames;
 }
 
 /**
@@ -1360,20 +1397,10 @@ clutter_timeline_get_progress (ClutterTimeline *timeline)
  *
  * Since: 0.6
  */
-ClutterFixed
+CoglFixed
 clutter_timeline_get_progressx (ClutterTimeline *timeline)
 {
-  ClutterTimelinePrivate *priv;
-  ClutterFixed progress;
-
-  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), 0);
-
-  priv = timeline->priv;
-
-  progress = clutter_qdivx (CLUTTER_INT_TO_FIXED (priv->current_frame_num),
-			    CLUTTER_INT_TO_FIXED (priv->n_frames));
-
-  return progress;
+  return COGL_FIXED_FROM_DOUBLE (clutter_timeline_get_progress (timeline));
 }
 
 /**
@@ -1390,7 +1417,8 @@ clutter_timeline_get_progressx (ClutterTimeline *timeline)
 ClutterTimelineDirection
 clutter_timeline_get_direction (ClutterTimeline *timeline)
 {
-  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), CLUTTER_TIMELINE_FORWARD);
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline),
+                        CLUTTER_TIMELINE_FORWARD);
 
   return timeline->priv->direction;
 }
@@ -1476,6 +1504,16 @@ clutter_timeline_add_marker_internal (ClutterTimeline *timeline,
   TimelineMarker *marker;
   GSList *markers;
 
+  /* create the hash tables that will hold the markers */
+  if (G_UNLIKELY (priv->markers_by_name == NULL ||
+                  priv->markers_by_frame == NULL))
+    {
+      priv->markers_by_name = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                     NULL,
+                                                     timeline_marker_free);
+      priv->markers_by_frame = g_hash_table_new (NULL, NULL);
+    }
+
   marker = g_hash_table_lookup (priv->markers_by_name, marker_name);
   if (G_UNLIKELY (marker))
     {
@@ -1572,9 +1610,9 @@ clutter_timeline_add_marker_at_time (ClutterTimeline *timeline,
  * negative integer, all the markers attached to @timeline will be
  * returned.
  *
- * Return value: a newly allocated, %NULL terminated string array
- *   containing the names of the markers. Use g_strfreev() when
- *   done.
+ * Return value: (array zero-terminated=1 length=n_markers): a newly
+ *   allocated, %NULL terminated string array containing the names of
+ *   the markers. Use g_strfreev() when done.
  *
  * Since: 0.8
  */
@@ -1591,6 +1629,15 @@ clutter_timeline_list_markers (ClutterTimeline *timeline,
 
   priv = timeline->priv;
 
+  if (G_UNLIKELY (priv->markers_by_name == NULL ||
+                  priv->markers_by_frame == NULL))
+    {
+      if (n_markers)
+        *n_markers = 0;
+
+      return NULL;
+    }
+
   if (frame_num < 0)
     {
       GList *markers, *l;
@@ -1628,6 +1675,11 @@ clutter_timeline_list_markers (ClutterTimeline *timeline,
  *
  * Advances @timeline to the frame of the given @marker_name.
  *
+ * <note><para>Like clutter_timeline_advance(), this function will not
+ * emit the #ClutterTimeline::new-frame for the frame where @marker_name
+ * is set, nor it will emit #ClutterTimeline::marker-reached for
+ * @marker_name.</para></note>
+ *
  * Since: 0.8
  */
 void
@@ -1642,6 +1694,13 @@ clutter_timeline_advance_to_marker (ClutterTimeline *timeline,
 
   priv = timeline->priv;
 
+  if (G_UNLIKELY (priv->markers_by_name == NULL ||
+                  priv->markers_by_frame == NULL))
+    {
+      g_warning ("No marker named `%s' found.", marker_name);
+      return;
+    }
+
   marker = g_hash_table_lookup (priv->markers_by_name, marker_name);
   if (!marker)
     {
@@ -1674,6 +1733,13 @@ clutter_timeline_remove_marker (ClutterTimeline *timeline,
 
   priv = timeline->priv;
 
+  if (G_UNLIKELY (priv->markers_by_name == NULL ||
+                  priv->markers_by_frame == NULL))
+    {
+      g_warning ("No marker named `%s' found.", marker_name);
+      return;
+    }
+
   marker = g_hash_table_lookup (priv->markers_by_name, marker_name);
   if (!marker)
     {
@@ -1728,6 +1794,10 @@ clutter_timeline_has_marker (ClutterTimeline *timeline,
   g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), FALSE);
   g_return_val_if_fail (marker_name != NULL, FALSE);
 
+  if (G_UNLIKELY (timeline->priv->markers_by_name == NULL ||
+                  timeline->priv->markers_by_frame == NULL))
+    return FALSE;
+
   return NULL != g_hash_table_lookup (timeline->priv->markers_by_name,
                                       marker_name);
 }
