@@ -18,40 +18,27 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _HAVE_CLUTTER_TIMELINE_H
-#define _HAVE_CLUTTER_TIMELINE_H
+#if !defined(__CLUTTER_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
+#error "Only <clutter/clutter.h> can be included directly."
+#endif
+
+#ifndef __CLUTTER_TIMELINE_H__
+#define __CLUTTER_TIMELINE_H__
 
 #include <glib-object.h>
 #include <clutter/clutter-fixed.h>
 
 G_BEGIN_DECLS
 
-#define CLUTTER_TYPE_TIMELINE (clutter_timeline_get_type ())
-
-#define CLUTTER_TIMELINE(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
-  CLUTTER_TYPE_TIMELINE, ClutterTimeline))
-
-#define CLUTTER_TIMELINE_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST ((klass), \
-  CLUTTER_TYPE_TIMELINE, ClutterTimelineClass))
-
-#define CLUTTER_IS_TIMELINE(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
-  CLUTTER_TYPE_TIMELINE))
-
-#define CLUTTER_IS_TIMELINE_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_TYPE ((klass), \
-  CLUTTER_TYPE_TIMELINE))
-
-#define CLUTTER_TIMELINE_GET_CLASS(obj) \
-  (G_TYPE_INSTANCE_GET_CLASS ((obj), \
-  CLUTTER_TYPE_TIMELINE, ClutterTimelineClass))
+#define CLUTTER_TYPE_TIMELINE                   (clutter_timeline_get_type ())
+#define CLUTTER_TIMELINE(obj)                   (G_TYPE_CHECK_INSTANCE_CAST ((obj), CLUTTER_TYPE_TIMELINE, ClutterTimeline))
+#define CLUTTER_TIMELINE_CLASS(klass)           (G_TYPE_CHECK_CLASS_CAST ((klass), CLUTTER_TYPE_TIMELINE, ClutterTimelineClass))
+#define CLUTTER_IS_TIMELINE(obj)                (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CLUTTER_TYPE_TIMELINE))
+#define CLUTTER_IS_TIMELINE_CLASS(klass)        (G_TYPE_CHECK_CLASS_TYPE ((klass), CLUTTER_TYPE_TIMELINE))
+#define CLUTTER_TIMELINE_GET_CLASS(obj)         (G_TYPE_INSTANCE_GET_CLASS ((obj), CLUTTER_TYPE_TIMELINE, ClutterTimelineClass))
 
 /**
  * ClutterTimelineDirection:
@@ -71,6 +58,14 @@ typedef struct _ClutterTimeline        ClutterTimeline;
 typedef struct _ClutterTimelineClass   ClutterTimelineClass; 
 typedef struct _ClutterTimelinePrivate ClutterTimelinePrivate;
 
+/**
+ * ClutterTimeline:
+ *
+ * The #ClutterTimeline structure contains only private data
+ * and should be accessed using the provided API
+ *
+ * Since: 0.2
+ */
 struct _ClutterTimeline
 {
   /*< private >*/
@@ -78,6 +73,18 @@ struct _ClutterTimeline
   ClutterTimelinePrivate *priv;
 };
 
+/**
+ * ClutterTimelineClass:
+ * @started: handler for the #ClutterTimeline::started signal
+ * @completed: handler for the #ClutterTimeline::completed signal
+ * @paused: handler for the #ClutterTimeline::paused signal
+ * @new_frame: handler for the #ClutterTimeline::new-frame signal
+ * @marker_reached: handler for the #ClutterTimeline::marker-reached signal
+ *
+ * The #ClutterTimelineClass structure contains only private data
+ *
+ * Since: 0.2
+ */
 struct _ClutterTimelineClass
 {
   /*< private >*/
@@ -132,7 +139,7 @@ void             clutter_timeline_advance               (ClutterTimeline *timeli
                                                          guint            frame_num);
 gint             clutter_timeline_get_current_frame     (ClutterTimeline *timeline);
 gdouble          clutter_timeline_get_progress          (ClutterTimeline *timeline);
-ClutterFixed     clutter_timeline_get_progressx         (ClutterTimeline *timeline);
+CoglFixed        clutter_timeline_get_progressx         (ClutterTimeline *timeline);
 void             clutter_timeline_set_n_frames          (ClutterTimeline *timeline,
                                                          guint            n_frames);
 guint            clutter_timeline_get_n_frames          (ClutterTimeline *timeline);
@@ -161,4 +168,4 @@ void             clutter_timeline_advance_to_marker     (ClutterTimeline *timeli
 
 G_END_DECLS
 
-#endif /* _HAVE_CLUTTER_TIMELINE_H */
+#endif /* _CLUTTER_TIMELINE_H__ */
