@@ -101,6 +101,117 @@
 #include "clutter-units.h"
 #include "clutter-private.h"
 
+#define DPI_FALLBACK    (96.0)
+
+#define FLOAT_EPSILON   (1e-30)
+
+/**
+ * clutter_units_mm:
+ * @mm: millimeters to convert
+ *
+ * Converts a value in millimeters to #ClutterUnit<!-- -->s at
+ * the current DPI.
+ *
+ * Return value: the value in units
+ *
+ * Since: 1.0
+ */
+ClutterUnit
+clutter_units_mm (gdouble mm)
+{
+  ClutterBackend *backend;
+  gdouble dpi;
+
+  backend = clutter_get_default_backend ();
+  dpi = clutter_backend_get_resolution (backend);
+  if (dpi < 0)
+    dpi = DPI_FALLBACK;
+
+  return mm * dpi / 25.4;
+}
+
+/**
+ * clutter_units_pt:
+ * @pt: typographic points to convert
+ *
+ * Converts a value in typographic points to #ClutterUnit<!-- -->s
+ * at the current DPI.
+ *
+ * Return value: the value in units
+ *
+ * Since: 1.0
+ */
+ClutterUnit
+clutter_units_pt (gdouble pt)
+{
+  ClutterBackend *backend;
+  gdouble dpi;
+
+  backend = clutter_get_default_backend ();
+  dpi = clutter_backend_get_resolution (backend);
+  if (dpi < 0)
+    dpi = DPI_FALLBACK;
+
+  return pt * dpi / 72.0;
+}
+
+/**
+ * clutter_units_em:
+ * @em: em to convert
+ *
+ * Converts a value in em to #ClutterUnit<!-- -->s at the
+ * current DPI
+ *
+ * Return value: the value in units
+ *
+ * Since: 1.0
+ */
+ClutterUnit
+clutter_units_em (gdouble em)
+{
+  ClutterBackend *backend;
+
+  backend = clutter_get_default_backend ();
+
+  return em * _clutter_backend_get_units_per_em (backend);
+}
+
+/**
+ * clutter_units_pixels:
+ * @px: pixels to convert
+ *
+ * Converts a value in pixels to #ClutterUnit<!-- -->s
+ *
+ * Return value: the value in units
+ *
+ * Since: 1.0
+ */
+ClutterUnit
+clutter_units_pixels (gint px)
+{
+  return CLUTTER_UNITS_FROM_INT (px);
+}
+
+/**
+ * clutter_units_to_pixels:
+ * @units: units to convert
+ *
+ * Converts a value in #ClutterUnit<!-- -->s to pixels
+ *
+ * Return value: the value in pixels
+ *
+ * Since: 1.0
+ */
+gint
+clutter_units_to_pixels (ClutterUnit units)
+{
+  return CLUTTER_UNITS_TO_INT (units);
+}
+
+/*
+ * GValue and GParamSpec integration
+ */
+
 static GTypeInfo _info = {
  0,
  NULL,
@@ -119,14 +230,14 @@ static GTypeFundamentalInfo _finfo = { 0, };
 static void
 clutter_value_init_unit (GValue *value)
 {
-  value->data[0].v_int = 0;
+  value->data[0].v_float = 0.0;
 }
 
 static void
 clutter_value_copy_unit (const GValue *src,
                          GValue       *dest)
 {
-  dest->data[0].v_int = src->data[0].v_int;
+  dest->data[0].v_float = src->data[0].v_float;
 }
 
 static gchar *
@@ -135,7 +246,7 @@ clutter_value_collect_unit (GValue      *value,
                             GTypeCValue *collect_values,
                             guint        collect_flags)
 {
-  value->data[0].v_int = collect_values[0].v_int;
+  value->data[0].v_float = collect_values[0].v_double;
 
   return NULL;
 }
@@ -146,13 +257,13 @@ clutter_value_lcopy_unit (const GValue *value,
                           GTypeCValue  *collect_values,
                           guint         collect_flags)
 {
-  gint32 *units_p = collect_values[0].v_pointer;
+  gfloat *units_p = collect_values[0].v_pointer;
 
   if (!units_p)
     return g_strdup_printf ("value location for `%s' passed as NULL",
                             G_VALUE_TYPE_NAME (value));
 
-  *units_p = value->data[0].v_int;
+  *units_p = value->data[0].v_float;
 
   return NULL;
 }
@@ -161,22 +272,52 @@ static void
 clutter_value_transform_unit_int (const GValue *src,
                                   GValue       *dest)
 {
-  dest->data[0].v_int = CLUTTER_UNITS_TO_INT (src->data[0].v_int);
+  dest->data[0].v_int = CLUTTER_UNITS_TO_INT (src->data[0].v_float);
 }
 
 static void
 clutter_value_transform_int_unit (const GValue *src,
                                   GValue       *dest)
 {
-  dest->data[0].v_int = CLUTTER_UNITS_FROM_INT (src->data[0].v_int);
+  dest->data[0].v_float = CLUTTER_UNITS_FROM_INT (src->data[0].v_int);
 }
 
+static void
+clutter_value_transform_unit_float (const GValue *src,
+                                    GValue       *dest)
+{
+  dest->data[0].v_float = CLUTTER_UNITS_TO_FLOAT (src->data[0].v_float);
+}
+
+static void
+clutter_value_transform_float_unit (const GValue *src,
+                                    GValue       *dest)
+{
+  dest->data[0].v_float = CLUTTER_UNITS_FROM_FLOAT (src->data[0].v_float);
+}
+
+#if 0
+static void
+clutter_value_transform_unit_fixed (const GValue *src,
+                                    GValue       *dest)
+{
+  dest->data[0].v_int = CLUTTER_UNITS_TO_FIXED (src->data[0].v_float);
+}
+
+static void
+clutter_value_transform_fixed_unit (const GValue *src,
+                                    GValue       *dest)
+{
+  dest->data[0].v_float = CLUTTER_UNITS_FROM_FIXED (src->data[0].v_int);
+}
+#endif
+
 static const GTypeValueTable _clutter_unit_value_table = {
   clutter_value_init_unit,
   NULL,
   clutter_value_copy_unit,
   NULL,
-  "i",
+  "d",
   clutter_value_collect_unit,
   "p",
   clutter_value_lcopy_unit
@@ -199,6 +340,11 @@ clutter_unit_get_type (void)
                                        clutter_value_transform_unit_int);
       g_value_register_transform_func (G_TYPE_INT, _clutter_unit_type,
                                        clutter_value_transform_int_unit);
+
+      g_value_register_transform_func (_clutter_unit_type, G_TYPE_FLOAT,
+                                       clutter_value_transform_unit_float);
+      g_value_register_transform_func (G_TYPE_FLOAT, _clutter_unit_type,
+                                       clutter_value_transform_float_unit);
     }
 
   return _clutter_unit_type;
@@ -219,7 +365,7 @@ clutter_value_set_unit (GValue      *value,
 {
   g_return_if_fail (CLUTTER_VALUE_HOLDS_UNIT (value));
 
-  value->data[0].v_int = units;
+  value->data[0].v_float = units;
 }
 
 /**
@@ -237,7 +383,7 @@ clutter_value_get_unit (const GValue *value)
 {
   g_return_val_if_fail (CLUTTER_VALUE_HOLDS_UNIT (value), 0);
 
-  return value->data[0].v_int;
+  return value->data[0].v_float;
 }
 
 static void
@@ -254,7 +400,7 @@ static void
 param_unit_set_default (GParamSpec *pspec,
                         GValue     *value)
 {
-  value->data[0].v_int = CLUTTER_PARAM_SPEC_UNIT (pspec)->default_value;
+  value->data[0].v_float = CLUTTER_PARAM_SPEC_UNIT (pspec)->default_value;
 }
 
 static gboolean
@@ -262,26 +408,15 @@ param_unit_validate (GParamSpec *pspec,
                      GValue     *value)
 {
   ClutterParamSpecUnit *uspec = CLUTTER_PARAM_SPEC_UNIT (pspec);
-  gint oval = CLUTTER_UNITS_TO_INT (value->data[0].v_int);
-  gint min, max, val;
+  gfloat oval = value->data[0].v_float;
 
   g_assert (CLUTTER_IS_PARAM_SPEC_UNIT (pspec));
 
-  /* we compare the integer part of the value because the minimum
-   * and maximum values cover just that part of the representation
-   */
-  min = uspec->minimum; 
-  max = uspec->maximum;
-  val = CLUTTER_UNITS_TO_INT (value->data[0].v_int);
+  value->data[0].v_float = CLAMP (value->data[0].v_float,
+                                  uspec->minimum,
+                                  uspec->maximum);
 
-  val = CLAMP (val, min, max);
-  if (val != oval)
-    {
-      value->data[0].v_int = val;
-      return TRUE;
-    }
-
-  return FALSE;
+  return value->data[0].v_float != oval;
 }
 
 static gint
@@ -289,10 +424,12 @@ param_unit_values_cmp (GParamSpec   *pspec,
                        const GValue *value1,
                        const GValue *value2)
 {
-  if (value1->data[0].v_int < value2->data[0].v_int)
-    return -1;
+  gfloat epsilon = FLOAT_EPSILON;
+
+  if (value1->data[0].v_float < value2->data[0].v_float)
+    return - (value2->data[0].v_float - value1->data[0].v_float > epsilon);
   else
-    return value1->data[0].v_int > value2->data[0].v_int;
+    return value1->data[0].v_float - value2->data[0].v_float > epsilon;
 }
 
 GType
