@@ -4,9 +4,11 @@
  *
  * An OpenGL based 'interactive canvas' library.
  *
- * Authored By Tomas Frydrych  <tf@openedhand.com>
+ * Authored By: Tomas Frydrych  <tf@openedhand.com>
+ *              Emmanuele Bassu  <ebassi@linux.intel.com>
  *
- * Copyright (C) 2007 OpenedHand
+ * Copyright (C) 2007, 2008 OpenedHand
+ * Copyright (C) 2009 Intel Corp.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,13 +21,15 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef _HAVE_CLUTTER_UNITS_H
-#define _HAVE_CLUTTER_UNITS_H
+#if !defined(__CLUTTER_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
+#error "Only <clutter/clutter.h> can be included directly.h"
+#endif
+
+#ifndef __CLUTTER_UNITS_H__
+#define __CLUTTER_UNITS_H__
 
 #include <glib-object.h>
 #include <clutter/clutter-fixed.h>
@@ -35,27 +39,88 @@ G_BEGIN_DECLS
 /**
  * ClutterUnit:
  *
- * Device independent unit used by Clutter. The value held can be transformed
- * into other units, likes pixels.
+ * Device independent unit used by Clutter. The value held can be
+ * transformed into other units, likes pixels
  *
  * Since: 0.4
  */
-typedef gint32 ClutterUnit;
+typedef float ClutterUnit;
 
-/*
- * Currently CLUTTER_UNIT maps directly onto ClutterFixed. Nevertheless, the
- * _FROM_FIXED and _TO_FIXED macros should always be used in case that we
- * decide to change this relationship in the future.
+/**
+ * CLUTTER_UNITS_FROM_INT:
+ * @x: integer value
+ *
+ * Converts @x from an integer value to #ClutterUnit<!-- -->s
+ *
+ * Since: 0.6
  */
+#define CLUTTER_UNITS_FROM_INT(x)        ((float)(x))
 
-#define CLUTTER_UNITS_FROM_INT(x)        CLUTTER_INT_TO_FIXED ((x))
-#define CLUTTER_UNITS_TO_INT(x)          CLUTTER_FIXED_TO_INT ((x))
+/**
+ * CLUTTER_UNITS_TO_INT:
+ * @x: value in #ClutterUnit<!-- -->s
+ *
+ * Converts @x from a #ClutterUnit value into an integer
+ *
+ * Since: 0.6
+ */
+#define CLUTTER_UNITS_TO_INT(x)          ((int)(x))
 
-#define CLUTTER_UNITS_FROM_FLOAT(x)      CLUTTER_FLOAT_TO_FIXED ((x))
-#define CLUTTER_UNITS_TO_FLOAT(x)        CLUTTER_FIXED_TO_FLOAT ((x))
+/**
+ * CLUTTER_UNITS_FROM_FLOAT:
+ * @x: float value
+ *
+ * Converts @x from a floating point value to #ClutterUnit<!-- -->s
+ *
+ * Since: 0.6
+ */
+#define CLUTTER_UNITS_FROM_FLOAT(x)      (x)
 
-#define CLUTTER_UNITS_FROM_FIXED(x)      (x)
-#define CLUTTER_UNITS_TO_FIXED(x)        (x)
+/**
+ * CLUTTER_UNITS_TO_FLOAT:
+ * @x: value in #ClutterUnit<!-- -->s
+ *
+ * Converts @x from a #ClutterUnit value into a float
+ *
+ * Since: 0.6
+ */
+#define CLUTTER_UNITS_TO_FLOAT(x)        (x)
+
+/**
+ * CLUTTER_UNITS_FROM_FIXED:
+ * @x: #CoglFixed value
+ *
+ * Converts @x from a fixed point value to #ClutterUnit<!-- -->s
+ *
+ * Since: 0.6
+ */
+#define CLUTTER_UNITS_FROM_FIXED(x)      (COGL_FIXED_TO_FLOAT (x))
+
+/**
+ * CLUTTER_UNITS_TO_FIXED:
+ * @x: value in #ClutterUnit<!-- -->s
+ *
+ * Converts @x from a #ClutterUnit value into a #CoglFixed
+ *
+ * Since: 0.6
+ */
+#define CLUTTER_UNITS_TO_FIXED(x)        (COGL_FIXED_FROM_FLOAT (x))
+
+/**
+ * CLUTTER_UNITS_FORMAT:
+ *
+ * Format string that should be used for scanning and printing units.
+ * It is a string literal, but it does not include the percent sign to
+ * allow precision and length modifiers between the percent sign and
+ * the format:
+ *
+ * |[
+ *   g_print ("%" CLUTTER_UNITS_FORMAT, units);
+ * ]|
+ *
+ * Since: 1.0
+ */
+#define CLUTTER_UNITS_FORMAT             "f"
 
 /**
  * CLUTTER_UNITS_FROM_DEVICE:
@@ -65,7 +130,7 @@ typedef gint32 ClutterUnit;
  *
  * Since: 0.6
  */
-#define CLUTTER_UNITS_FROM_DEVICE(x)     CLUTTER_UNITS_FROM_INT ((x))
+#define CLUTTER_UNITS_FROM_DEVICE(x)    (clutter_units_pixels ((x)))
 
 /**
  * CLUTTER_UNITS_TO_DEVICE:
@@ -75,10 +140,7 @@ typedef gint32 ClutterUnit;
  *
  * Since: 0.6
  */
-#define CLUTTER_UNITS_TO_DEVICE(x)       CLUTTER_UNITS_TO_INT ((x))
-
-#define CLUTTER_UNITS_TMP_FROM_DEVICE(x) (x)
-#define CLUTTER_UNITS_TMP_TO_DEVICE(x)   (x)
+#define CLUTTER_UNITS_TO_DEVICE(x)      (clutter_units_to_pixels ((x)))
 
 /**
  * CLUTTER_UNITS_FROM_PANGO_UNIT:
@@ -88,7 +150,7 @@ typedef gint32 ClutterUnit;
  *
  * Since: 0.6
  */
-#define CLUTTER_UNITS_FROM_PANGO_UNIT(x) ((x) << 6)
+#define CLUTTER_UNITS_FROM_PANGO_UNIT(x) ((float)((x) / 1024.0))
 
 /**
  * CLUTTER_UNITS_TO_PANGO_UNIT:
@@ -98,19 +160,7 @@ typedef gint32 ClutterUnit;
  *
  * Since: 0.6
  */
-#define CLUTTER_UNITS_TO_PANGO_UNIT(x)   ((x) >> 6)
-
-#define CLUTTER_UNITS_FROM_STAGE_WIDTH_PERCENTAGE(x) \
-  ((clutter_actor_get_widthu (clutter_stage_get_default ()) * x) / 100)
-
-#define CLUTTER_UNITS_FROM_STAGE_HEIGHT_PERCENTAGE(x) \
-  ((clutter_actor_get_heightu (clutter_stage_get_default ()) * x) / 100)
-
-#define CLUTTER_UNITS_FROM_PARENT_WIDTH_PERCENTAGE(a, x) \
-  ((clutter_actor_get_widthu (clutter_actor_get_parent (a)) * x) / 100)
-
-#define CLUTTER_UNITS_FROM_PARENT_HEIGHT_PERCENTAGE(a, x) \
-  ((clutter_actor_get_heightu (clutter_actor_get_parent (a)) * x) / 100)
+#define CLUTTER_UNITS_TO_PANGO_UNIT(x)   ((int)((x) * 1024))
 
 /**
  * CLUTTER_UNITS_FROM_MM:
@@ -120,11 +170,7 @@ typedef gint32 ClutterUnit;
  *
  * Since: 0.6
  */
-#define CLUTTER_UNITS_FROM_MM(x) \
-  (CLUTTER_UNITS_FROM_FLOAT ((((x) * clutter_stage_get_resolution ((ClutterStage *) clutter_stage_get_default ())) / 25.4)))
-
-#define CLUTTER_UNITS_FROM_MMX(x) \
-  (CFX_DIV (CFX_MUL ((x), clutter_stage_get_resolutionx ((ClutterStage *) clutter_stage_get_default ())), 0x196666))
+#define CLUTTER_UNITS_FROM_MM(x)        (clutter_units_mm (x))
 
 /**
  * CLUTTER_UNITS_FROM_POINTS:
@@ -134,11 +180,24 @@ typedef gint32 ClutterUnit;
  *
  * Since: 0.6
  */
-#define CLUTTER_UNITS_FROM_POINTS(x) \
-  CLUTTER_UNITS_FROM_FLOAT ((((x) * clutter_stage_get_resolution ((ClutterStage *) clutter_stage_get_default ())) / 72.0))
+#define CLUTTER_UNITS_FROM_POINTS(x)    (clutter_units_pt (x))
 
-#define CLUTTER_UNITS_FROM_POINTSX(x) \
-  (CFX_MUL ((x), clutter_stage_get_resolutionx ((ClutterStage *) clutter_stage_get_default ())) / 72)
+/**
+ * CLUTTER_UNITS_FROM_EM:
+ * @x: a value in em
+ *
+ * Converts a value in em into #ClutterUnit<!-- -->s
+ *
+ * Since: 1.0
+ */
+#define CLUTTER_UNITS_FROM_EM(x)        (clutter_units_em (x))
+
+ClutterUnit clutter_units_mm        (gdouble     mm);
+ClutterUnit clutter_units_pt        (gdouble     pt);
+ClutterUnit clutter_units_em        (gdouble     em);
+ClutterUnit clutter_units_pixels    (gint        px);
+
+gint        clutter_units_to_pixels (ClutterUnit units);
 
 #define CLUTTER_TYPE_UNIT                 (clutter_unit_get_type ())
 #define CLUTTER_TYPE_PARAM_UNIT           (clutter_param_unit_get_type ())
@@ -152,7 +211,7 @@ typedef gint32 ClutterUnit;
  *
  * Since: 0.8
  */
-#define CLUTTER_MAXUNIT         G_MAXINT16
+#define CLUTTER_MAXUNIT         (G_MAXFLOAT)
 
 /**
  * CLUTTER_MINUNIT:
@@ -161,7 +220,7 @@ typedef gint32 ClutterUnit;
  *
  * Since: 0.8
  */
-#define CLUTTER_MINUNIT         G_MININT16
+#define CLUTTER_MINUNIT         (-G_MAXFLOAT)
 
 /**
  * CLUTTER_VALUE_HOLDS_UNIT:
@@ -213,4 +272,4 @@ GParamSpec *clutter_param_spec_unit (const gchar *name,
 
 G_END_DECLS
 
-#endif /* _HAVE_CLUTTER_UNITS_H */
+#endif /* __CLUTTER_UNITS_H__ */
