@@ -309,6 +309,7 @@ typedef GLuint COGLuint;
 #define CGL_MAX_TEXTURE_STACK_DEPTH GL_MAX_TEXTURE_STACK_DEPTH
 #define CGL_MAX_VIEWPORT_DIMS GL_MAX_VIEWPORT_DIMS
 #define CGL_MAX_CLIENT_ATTRIB_STACK_DEPTH GL_MAX_CLIENT_ATTRIB_STACK_DEPTH
+#define CGL_MAX_COMBINED_TEXTURE_IMAGE_UNITS GL_MAX_TEXTURE_UNITS
 #define CGL_ATTRIB_STACK_DEPTH GL_ATTRIB_STACK_DEPTH
 #define CGL_CLIENT_ATTRIB_STACK_DEPTH GL_CLIENT_ATTRIB_STACK_DEPTH
 #define CGL_COLOR_CLEAR_VALUE GL_COLOR_CLEAR_VALUE
@@ -667,6 +668,9 @@ typedef GLuint COGLuint;
 #define CGL_COMPRESSED_SLUMINANCE_EXT GL_COMPRESSED_SLUMINANCE_EXT
 #define CGL_COMPRESSED_SLUMINANCE_ALPHA_EXT GL_COMPRESSED_SLUMINANCE_ALPHA_EXT
 
+#define CGL_DOT3_RGB GL_DOT3_RGB
+#define CGL_DOT3_RGBA GL_DOT3_RGBA
+
 /* extras */
 
 #define CGL_TEXTURE_2D GL_TEXTURE_2D
@@ -837,11 +841,199 @@ typedef void
    GLenum                pname,
    GLint                *params);
 
+typedef void
+  (APIENTRYP		 COGL_PFNGLVERTEXATTRIBPOINTERARBPROC)
+  (GLuint		 index,
+   GLint		 size,
+   GLenum		 type,
+   GLboolean		 normalized,
+   GLsizei		 stride,
+   const GLvoid		*pointer);
+
+typedef void
+  (APIENTRYP		 COGL_PFNGLENABLEVERTEXATTRIBARRAYARBPROC)
+  (GLuint		 index);
+
+typedef void
+  (APIENTRYP		 COGL_PFNGLDISABLEVERTEXATTRIBARRAYARBPROC)
+  (GLuint		 index);
+
+typedef void
+  (APIENTRYP		 COGL_PFNGLGENBUFFERSARBPROC)
+  (GLuint		 n,
+   GLuint		*buffers);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLBINDBUFFERARBPROC)
+  (GLenum		 target, 
+   GLuint		 buffer);
+
+typedef void
+  (APIENTRYP		 COGL_PFNGLBUFFERDATAARBPROC)
+  (GLenum		 target, 
+   GLsizeiptr		 size, 
+   const GLvoid		*data, 
+   GLenum		 usage);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLBUFFERSUBDATAARBPROC)
+  (GLenum		 target, 
+   GLintptr		 offset, 
+   GLsizeiptr		 size,
+   const GLvoid		*data);
+
+typedef void *
+  (APIENTRYP		 COGL_PFNGLMAPBUFFERARBPROC)
+  (GLenum		 target, 
+   GLenum		 access);
+
+typedef GLboolean
+  (APIENTRYP		 COGL_PFNGLUNMAPBUFFERARBPROC)
+  (GLenum		 target);
+
+typedef void
+  (APIENTRYP		 COGL_PFNGLDELETEBUFFERSARBPROC)
+  (GLsizei		 n,
+   const GLuint		*buffers);
+
 typedef void
   (APIENTRYP             COGL_PFNGLUNIFORM1FARBPROC)
   (GLint                 location,
    GLfloat               v0);
 
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM2FARBPROC)
+  (GLint                 location,
+   GLfloat               v0,
+   GLfloat               v1);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM3FARBPROC)
+  (GLint                 location,
+   GLfloat               v0,
+   GLfloat               v1,
+   GLfloat               v2);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM4FARBPROC)
+  (GLint                 location,
+   GLfloat               v0,
+   GLfloat               v1,
+   GLfloat               v2,
+   GLfloat               v3);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM1FVARBPROC)
+  (GLint                 location,
+   GLsizei               count,
+   const GLfloat *       value);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM2FVARBPROC)
+  (GLint                 location,
+   GLsizei               count,
+   const GLfloat *       value);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM3FVARBPROC)
+  (GLint                 location,
+   GLsizei               count,
+   const GLfloat *       value);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM4FVARBPROC)
+  (GLint                 location,
+   GLsizei               count,
+   const GLfloat *       value);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM1IARBPROC)
+  (GLint                 location,
+   GLint                 v0);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM2IARBPROC)
+  (GLint                 location,
+   GLint                 v0,
+   GLint                 v1);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM3IARBPROC)
+  (GLint                 location,
+   GLint                 v0,
+   GLint                 v1,
+   GLint                 v2);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM4IARBPROC)
+  (GLint                 location,
+   GLint                 v0,
+   GLint                 v1,
+   GLint                 v2,
+   GLint                 v3);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM1IVARBPROC)
+  (GLint                 location,
+   GLsizei               count,
+   const GLint *         value);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM2IVARBPROC)
+  (GLint                 location,
+   GLsizei               count,
+   const GLint *         value);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM3IVARBPROC)
+  (GLint                 location,
+   GLsizei               count,
+   const GLint *         value);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORM4IVARBPROC)
+  (GLint                 location,
+   GLsizei               count,
+   const GLint *         value);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORMMATRIX2FVARBPROC)
+  (GLint                 location,
+   GLsizei               count,
+   GLboolean             transpose,
+   const GLfloat        *value);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORMMATRIX3FVARBPROC)
+  (GLint                 location,
+   GLsizei               count,
+   GLboolean             transpose,
+   const GLfloat        *value);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLUNIFORMMATRIX4FVARBPROC)
+  (GLint                 location,
+   GLsizei               count,
+   GLboolean             transpose,
+   const GLfloat        *value);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLDRAWRANGEELEMENTSPROC)
+  (GLenum                mode,
+   GLuint                start,
+   GLuint                end,
+   GLsizei               count,
+   GLenum                type,
+   const GLvoid         *indices);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLACTIVETEXTUREPROC)
+  (GLenum                texture);
+
+typedef void
+  (APIENTRYP             COGL_PFNGLCLIENTACTIVETEXTUREPROC)
+  (GLenum                texture);
+
 G_END_DECLS
 
 #endif
