@@ -27,12 +27,15 @@
 #define __COGL_CONTEXT_H
 
 #include "cogl-primitives.h"
+#include "cogl-clip-stack.h"
+#include "cogl-matrix-stack.h"
+#include "cogl-current-matrix.h"
 
 typedef struct
 {
   GLfloat v[3];
   GLfloat t[2];
-  GLfloat c[4];
+  GLubyte c[4];
 } CoglTextureGLVertex;
 
 typedef struct
@@ -40,36 +43,48 @@ typedef struct
   /* Features cache */
   CoglFeatureFlags  feature_flags;
   gboolean          features_cached;
-  GLint             num_stencil_bits;
-  
+
   /* Enable cache */
   gulong            enable_flags;
   guint8            color_alpha;
-  COGLenum          blend_src_factor;
-  COGLenum          blend_dst_factor;
-  
-  /* Primitives */
-  CoglFixedVec2     path_start;
-  CoglFixedVec2     path_pen;
-  CoglFloatVec2    *path_nodes;
-  guint             path_nodes_cap;
-  guint             path_nodes_size;
-  CoglFixedVec2     path_nodes_min;
-  CoglFixedVec2     path_nodes_max;
+
+  gboolean          enable_backface_culling;
+
+  gboolean          indirect;
+
+  /* Client-side matrix stack or NULL if none */
+  CoglMatrixMode    matrix_mode;
+  CoglMatrixStack  *modelview_stack;
 
   /* Cache of inverse projection matrix */
-  GLfloat           inverse_projection[16];
-  
+  float            inverse_projection[16];
+
+  /* Materials */
+  GArray           *material_handles;
+  GArray           *material_layer_handles;
+  CoglHandle        default_material;
+  CoglHandle	    source_material;
+
   /* Textures */
   GArray	   *texture_handles;
-  GArray           *texture_vertices;
-  GArray           *texture_indices;
-  /* The gl texture number that the above vertices apply to. This to
-     detect when a different slice is encountered so that the vertices
-     can be flushed */
-  GLuint            texture_current;
-  GLenum            texture_target;
-  GLenum            texture_wrap_mode;
+  CoglHandle        default_gl_texture_2d_tex;
+  CoglHandle        default_gl_texture_rect_tex;
+
+
+  /* Batching geometry... */
+  /* We journal the texture rectangles we want to submit to OpenGL so
+   * we have an oppertunity to optimise the final order so that we
+   * can batch things together. */
+  GArray           *journal;
+  GArray           *logged_vertices;
+  GArray           *static_indices;
+  GArray           *polygon_vertices;
+
+  /* Some simple caching, to minimize state changes... */
+  CoglHandle	    current_material;
+  gulong            current_material_flags;
+  GArray           *current_layers;
+  guint             n_texcoord_arrays_enabled;
 
   /* Framebuffer objects */
   GArray           *fbo_handles;
@@ -80,7 +95,22 @@ typedef struct
 
   /* Programs */
   GArray           *program_handles;
-  
+
+  /* Clip stack */
+  CoglClipStackState clip;
+
+  /* Vertex buffers */
+  GArray           *vertex_buffer_handles;
+
+  /* Primitives */
+  floatVec2         path_start;
+  floatVec2         path_pen;
+  GArray           *path_nodes;
+  guint             last_path;
+  floatVec2         path_nodes_min;
+  floatVec2         path_nodes_max;
+  CoglHandle        stencil_material;
+
   /* Relying on glext.h to define these */
   COGL_PFNGLGENRENDERBUFFERSEXTPROC                pf_glGenRenderbuffersEXT;
   COGL_PFNGLDELETERENDERBUFFERSEXTPROC             pf_glDeleteRenderbuffersEXT;
@@ -94,7 +124,7 @@ typedef struct
   COGL_PFNGLDELETEFRAMEBUFFERSEXTPROC              pf_glDeleteFramebuffersEXT;
   COGL_PFNGLBLITFRAMEBUFFEREXTPROC                 pf_glBlitFramebufferEXT;
   COGL_PFNGLRENDERBUFFERSTORAGEMULTISAMPLEEXTPROC  pf_glRenderbufferStorageMultisampleEXT;
-  
+
   COGL_PFNGLCREATEPROGRAMOBJECTARBPROC             pf_glCreateProgramObjectARB;
   COGL_PFNGLCREATESHADEROBJECTARBPROC              pf_glCreateShaderObjectARB;
   COGL_PFNGLSHADERSOURCEARBPROC                    pf_glShaderSourceARB;
@@ -106,9 +136,43 @@ typedef struct
   COGL_PFNGLDELETEOBJECTARBPROC                    pf_glDeleteObjectARB;
   COGL_PFNGLGETINFOLOGARBPROC                      pf_glGetInfoLogARB;
   COGL_PFNGLGETOBJECTPARAMETERIVARBPROC            pf_glGetObjectParameterivARB;
+
+  COGL_PFNGLVERTEXATTRIBPOINTERARBPROC		   pf_glVertexAttribPointerARB;
+  COGL_PFNGLENABLEVERTEXATTRIBARRAYARBPROC	   pf_glEnableVertexAttribArrayARB;
+  COGL_PFNGLDISABLEVERTEXATTRIBARRAYARBPROC	   pf_glDisableVertexAttribArrayARB;
+
+  COGL_PFNGLGENBUFFERSARBPROC			   pf_glGenBuffersARB;
+  COGL_PFNGLBINDBUFFERARBPROC			   pf_glBindBufferARB;
+  COGL_PFNGLBUFFERDATAARBPROC			   pf_glBufferDataARB;
+  COGL_PFNGLBUFFERSUBDATAARBPROC		   pf_glBufferSubDataARB;
+  COGL_PFNGLMAPBUFFERARBPROC			   pf_glMapBufferARB;
+  COGL_PFNGLUNMAPBUFFERARBPROC			   pf_glUnmapBufferARB;
+  COGL_PFNGLDELETEBUFFERSARBPROC		   pf_glDeleteBuffersARB;
+
   COGL_PFNGLUNIFORM1FARBPROC                       pf_glUniform1fARB;
-  
+  COGL_PFNGLUNIFORM2FARBPROC                       pf_glUniform2fARB;
+  COGL_PFNGLUNIFORM3FARBPROC                       pf_glUniform3fARB;
+  COGL_PFNGLUNIFORM4FARBPROC                       pf_glUniform4fARB;
+  COGL_PFNGLUNIFORM1FVARBPROC                      pf_glUniform1fvARB;
+  COGL_PFNGLUNIFORM2FVARBPROC                      pf_glUniform2fvARB;
+  COGL_PFNGLUNIFORM3FVARBPROC                      pf_glUniform3fvARB;
+  COGL_PFNGLUNIFORM4FVARBPROC                      pf_glUniform4fvARB;
+  COGL_PFNGLUNIFORM1IARBPROC                       pf_glUniform1iARB;
+  COGL_PFNGLUNIFORM2IARBPROC                       pf_glUniform2iARB;
+  COGL_PFNGLUNIFORM3IARBPROC                       pf_glUniform3iARB;
+  COGL_PFNGLUNIFORM4IARBPROC                       pf_glUniform4iARB;
+  COGL_PFNGLUNIFORM1IVARBPROC                      pf_glUniform1ivARB;
+  COGL_PFNGLUNIFORM2IVARBPROC                      pf_glUniform2ivARB;
+  COGL_PFNGLUNIFORM3IVARBPROC                      pf_glUniform3ivARB;
+  COGL_PFNGLUNIFORM4IVARBPROC                      pf_glUniform4ivARB;
+  COGL_PFNGLUNIFORMMATRIX2FVARBPROC                pf_glUniformMatrix2fvARB;
+  COGL_PFNGLUNIFORMMATRIX3FVARBPROC                pf_glUniformMatrix3fvARB;
+  COGL_PFNGLUNIFORMMATRIX4FVARBPROC                pf_glUniformMatrix4fvARB;
+
   COGL_PFNGLDRAWRANGEELEMENTSPROC                  pf_glDrawRangeElements;
+
+  COGL_PFNGLACTIVETEXTUREPROC                      pf_glActiveTexture;
+  COGL_PFNGLCLIENTACTIVETEXTUREPROC                pf_glClientActiveTexture;
 } CoglContext;
 
 CoglContext *
@@ -119,6 +183,6 @@ _cogl_context_get_default ();
 CoglContext *ctxvar = _cogl_context_get_default (); \
 if (ctxvar == NULL) return retval;
 
-#define NO_RETVAL 
+#define NO_RETVAL
 
 #endif /* __COGL_CONTEXT_H */
