@@ -26,7 +26,12 @@
 #ifndef __COGL_FIXED_VERTEX_SHADER_H
 #define __COGL_FIXED_VERTEX_SHADER_H
 
-extern const char cogl_fixed_vertex_shader_start[];
+extern const char cogl_fixed_vertex_shader_per_vertex_attribs[];
+extern const char cogl_fixed_vertex_shader_transform_matrices[];
+extern const char cogl_fixed_vertex_shader_output_variables[];
+extern const char cogl_fixed_vertex_shader_fogging_options[];
+extern const char cogl_fixed_vertex_shader_main_start[];
+extern const char cogl_fixed_vertex_shader_frag_color_start[];
 extern const char cogl_fixed_vertex_shader_fog_start[];
 extern const char cogl_fixed_vertex_shader_fog_exp[];
 extern const char cogl_fixed_vertex_shader_fog_exp2[];
