@@ -31,13 +31,19 @@
 #include <string.h>
 #include <math.h>
 
+/* We don't want to get the remaps from the gl* functions to the
+   cogl_wrap_gl* functions in this file because we need to be able to
+   call the base version */
+#define COGL_GLES2_WRAPPER_NO_REMAP 1
+
 #include "cogl.h"
 #include "cogl-gles2-wrapper.h"
 #include "cogl-fixed-vertex-shader.h"
 #include "cogl-fixed-fragment-shader.h"
 #include "cogl-context.h"
-#include "cogl-shader.h"
+#include "cogl-shader-private.h"
 #include "cogl-program.h"
+#include "cogl-internal.h"
 
 #define _COGL_GET_GLES2_WRAPPER(wvar, retval)			\
   CoglGles2Wrapper *wvar;					\
@@ -66,8 +72,9 @@
   while (0)
 
 #define COGL_GLES2_WRAPPER_VERTEX_ATTRIB    0
-#define COGL_GLES2_WRAPPER_TEX_COORD_ATTRIB 1
-#define COGL_GLES2_WRAPPER_COLOR_ATTRIB     2
+#define COGL_GLES2_WRAPPER_COLOR_ATTRIB     1
+#define COGL_GLES2_WRAPPER_NORMAL_ATTRIB    2
+
 
 static GLuint
 cogl_gles2_wrapper_create_shader (GLenum type, const char *source)
@@ -100,32 +107,71 @@ cogl_gles2_wrapper_create_shader (GLenum type, const char *source)
   return shader;
 }
 
+static void
+initialize_texture_units (CoglGles2Wrapper *w)
+{
+  /* We save the active texture unit since we may need to temporarily
+   * change this to initialise each new texture unit and we want to
+   * restore the active unit afterwards */
+  int initial_active_unit = w->active_texture_unit;
+  GLint prev_mode;
+  int i;
+
+  /* We will need to set the matrix mode to GL_TEXTURE to
+   * initialise any new texture units, so we save the current
+   * mode for restoring afterwards */
+  GE( cogl_wrap_glGetIntegerv (CGL_MATRIX_MODE, &prev_mode));
+
+  for (i = 0; i < COGL_GLES2_MAX_TEXTURE_UNITS; i++)
+    {
+      CoglGles2WrapperTextureUnit *new_unit;
+
+      new_unit = w->texture_units + i;
+      memset (new_unit, 0, sizeof (CoglGles2WrapperTextureUnit));
+
+      w->active_texture_unit = i;
+      GE( cogl_wrap_glMatrixMode (GL_TEXTURE));
+      GE( cogl_wrap_glLoadIdentity ());
+    }
+
+  GE( cogl_wrap_glMatrixMode ((GLenum) prev_mode));
+
+  w->settings.texture_units = 0;
+
+  w->active_texture_unit = initial_active_unit;
+}
+
 void
 cogl_gles2_wrapper_init (CoglGles2Wrapper *wrapper)
 {
-  GLfixed default_fog_color[4] = { 0, 0, 0, 0 };
+  GLfloat default_fog_color[4] = { 0, 0, 0, 0 };
 
   memset (wrapper, 0, sizeof (CoglGles2Wrapper));
 
   /* Initialize the stacks */
-  cogl_wrap_glMatrixMode (GL_TEXTURE);
-  cogl_wrap_glLoadIdentity ();
   cogl_wrap_glMatrixMode (GL_PROJECTION);
   cogl_wrap_glLoadIdentity ();
   cogl_wrap_glMatrixMode (GL_MODELVIEW);
   cogl_wrap_glLoadIdentity ();
 
+  /* The gl*ActiveTexture wrappers will initialise the texture
+   * stack for the texture unit when it's first activated */
+  cogl_wrap_glActiveTexture (GL_TEXTURE0);
+  cogl_wrap_glClientActiveTexture (GL_TEXTURE0);
+
   /* Initialize the fogging options */
   cogl_wrap_glDisable (GL_FOG);
-  cogl_wrap_glFogx (GL_FOG_MODE, GL_LINEAR);
-  cogl_wrap_glFogx (GL_FOG_DENSITY, CFX_ONE);
-  cogl_wrap_glFogx (GL_FOG_START, 0);
-  cogl_wrap_glFogx (GL_FOG_END, 1);
-  cogl_wrap_glFogxv (GL_FOG_COLOR, default_fog_color);
+  cogl_wrap_glFogf (GL_FOG_MODE, GL_LINEAR);
+  cogl_wrap_glFogf (GL_FOG_DENSITY, 1.0);
+  cogl_wrap_glFogf (GL_FOG_START, 0);
+  cogl_wrap_glFogf (GL_FOG_END, 1);
+  cogl_wrap_glFogfv (GL_FOG_COLOR, default_fog_color);
 
   /* Initialize alpha testing */
   cogl_wrap_glDisable (GL_ALPHA_TEST);
   cogl_wrap_glAlphaFunc (GL_ALWAYS, 0.0f);
+
+  initialize_texture_units (wrapper);
 }
 
 static gboolean
@@ -134,14 +180,11 @@ cogl_gles2_settings_equal (const CoglGles2WrapperSettings *a,
 			   gboolean vertex_tests,
 			   gboolean fragment_tests)
 {
+  if (a->texture_units != b->texture_units)
+    return FALSE;
+
   if (fragment_tests)
     {
-      if (a->texture_2d_enabled != b->texture_2d_enabled)
-	return FALSE;
-      
-      if (a->texture_2d_enabled && a->alpha_only != b->alpha_only)
-	return FALSE;
-
       if (a->alpha_test_enabled != b->alpha_test_enabled)
 	return FALSE;
       if (a->alpha_test_enabled && a->alpha_test_func != b->alpha_test_func)
@@ -164,6 +207,8 @@ cogl_gles2_get_vertex_shader (const CoglGles2WrapperSettings *settings)
   GLuint shader_obj;
   CoglGles2WrapperShader *shader;
   GSList *node;
+  int i;
+  int n_texture_units = 0;
 
   _COGL_GET_GLES2_WRAPPER (w, NULL);
 
@@ -176,7 +221,51 @@ cogl_gles2_get_vertex_shader (const CoglGles2WrapperSettings *settings)
       return (CoglGles2WrapperShader *) node->data;
 
   /* Otherwise create a new shader */
-  shader_source = g_string_new (cogl_fixed_vertex_shader_start);
+  shader_source = g_string_new (cogl_fixed_vertex_shader_per_vertex_attribs);
+
+  for (i = 0; i < COGL_GLES2_MAX_TEXTURE_UNITS; i++)
+    if (COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (settings->texture_units, i))
+      g_string_append_printf (shader_source,
+			      "attribute vec4 multi_tex_coord_attrib%d;\n",
+			      i);
+
+  /* Find the biggest enabled texture unit index */
+  for (i = 0; i < COGL_GLES2_MAX_TEXTURE_UNITS; i++)
+    if (COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (settings->texture_units, i))
+      n_texture_units = i + 1;
+
+  g_string_append (shader_source, cogl_fixed_vertex_shader_transform_matrices);
+  g_string_append (shader_source, cogl_fixed_vertex_shader_output_variables);
+
+  if (n_texture_units > 0)
+    {
+      g_string_append_printf (shader_source,
+                              "uniform mat4	      texture_matrix[%d];\n",
+                              n_texture_units);
+
+      g_string_append_printf (shader_source,
+                              "varying vec2       tex_coord[%d];",
+                              n_texture_units);
+    }
+
+  g_string_append (shader_source, cogl_fixed_vertex_shader_fogging_options);
+  g_string_append (shader_source, cogl_fixed_vertex_shader_main_start);
+
+  for (i = 0; i < COGL_GLES2_MAX_TEXTURE_UNITS; i++)
+    if (COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (settings->texture_units, i))
+      {
+        g_string_append_printf (shader_source,
+                                "transformed_tex_coord = "
+                                "texture_matrix[%d] "
+                                " * multi_tex_coord_attrib%d;\n",
+                                i, i);
+        g_string_append_printf (shader_source,
+                                "tex_coord[%d] = transformed_tex_coord.st "
+                                " / transformed_tex_coord.q;\n",
+                                i);
+      }
+
+  g_string_append (shader_source, cogl_fixed_vertex_shader_frag_color_start);
 
   if (settings->fog_enabled)
     {
@@ -206,10 +295,10 @@ cogl_gles2_get_vertex_shader (const CoglGles2WrapperSettings *settings)
 						 shader_source->str);
 
   g_string_free (shader_source, TRUE);
-  
+
   if (shader_obj == 0)
     return NULL;
-  
+
   shader = g_slice_new (CoglGles2WrapperShader);
   shader->shader = shader_obj;
   shader->settings = *settings;
@@ -227,6 +316,8 @@ cogl_gles2_get_fragment_shader (const CoglGles2WrapperSettings *settings)
   GLuint shader_obj;
   CoglGles2WrapperShader *shader;
   GSList *node;
+  int i;
+  int n_texture_units = 0;
 
   _COGL_GET_GLES2_WRAPPER (w, NULL);
 
@@ -239,18 +330,60 @@ cogl_gles2_get_fragment_shader (const CoglGles2WrapperSettings *settings)
       return (CoglGles2WrapperShader *) node->data;
 
   /* Otherwise create a new shader */
-  shader_source = g_string_new (cogl_fixed_fragment_shader_start);
-  if (settings->texture_2d_enabled)
+  shader_source = g_string_new (cogl_fixed_fragment_shader_variables_start);
+
+  /* Find the biggest enabled texture unit index */
+  for (i = 0; i < COGL_GLES2_MAX_TEXTURE_UNITS; i++)
+    if (COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (settings->texture_units, i))
+      n_texture_units = i + 1;
+
+  g_string_append (shader_source, cogl_fixed_fragment_shader_inputs);
+
+  if (n_texture_units > 0)
     {
-      if (settings->alpha_only)
-	g_string_append (shader_source,
-			 cogl_fixed_fragment_shader_texture_alpha_only);
-      else
-	g_string_append (shader_source,
-			 cogl_fixed_fragment_shader_texture);
+      g_string_append_printf (shader_source,
+                              "varying vec2       tex_coord[%d];\n",
+                              n_texture_units);
+
+      g_string_append (shader_source, cogl_fixed_fragment_shader_texturing_options);
+      g_string_append_printf (shader_source,
+                              "uniform sampler2D  texture_unit[%d];\n",
+                              n_texture_units);
     }
-  else
-    g_string_append (shader_source, cogl_fixed_fragment_shader_solid_color);
+
+  g_string_append (shader_source, cogl_fixed_fragment_shader_fogging_options);
+
+  g_string_append (shader_source, cogl_fixed_fragment_shader_main_declare);
+
+  g_string_append (shader_source, cogl_fixed_fragment_shader_main_start);
+  
+  /* This pointless extra variable is needed to work around an
+     apparent bug in the PowerVR drivers. Without it the alpha
+     blending seems to stop working */
+  g_string_append (shader_source,
+		   "vec4 frag_color_copy = frag_color;\n");
+  g_string_append (shader_source, "gl_FragColor = frag_color;\n");
+
+  for (i = 0; i < n_texture_units; i++)
+    if (COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (settings->texture_units, i))
+      {
+        if (COGL_GLES2_TEXTURE_UNIT_IS_ALPHA_ONLY (settings->texture_units, i))
+          /* If the texture only has an alpha channel (eg, with the textures
+             from the pango renderer) then the RGB components will be
+             black. We want to use the RGB from the current color in that
+             case */
+          g_string_append_printf (shader_source,
+                                  "gl_FragColor.a *= "
+                                  "texture2D (texture_unit[%d], "
+                                  "tex_coord[%d]).a;\n",
+                                  i, i);
+        else
+          g_string_append_printf (shader_source,
+                                  "gl_FragColor *= "
+                                  "texture2D (texture_unit[%d], "
+                                  "tex_coord[%d]);\n",
+                                  i, i);
+      }
 
   if (settings->fog_enabled)
     g_string_append (shader_source, cogl_fixed_fragment_shader_fog);
@@ -293,10 +426,10 @@ cogl_gles2_get_fragment_shader (const CoglGles2WrapperSettings *settings)
 						 shader_source->str);
 
   g_string_free (shader_source, TRUE);
-  
+
   if (shader_obj == 0)
     return NULL;
-  
+
   shader = g_slice_new (CoglGles2WrapperShader);
   shader->shader = shader_obj;
   shader->settings = *settings;
@@ -307,6 +440,69 @@ cogl_gles2_get_fragment_shader (const CoglGles2WrapperSettings *settings)
   return shader;
 }
 
+static void
+cogl_gles2_wrapper_get_locations (GLuint program,
+				  CoglGles2WrapperSettings *settings,
+				  CoglGles2WrapperUniforms *uniforms,
+				  CoglGles2WrapperAttributes *attribs)
+{
+  int i;
+
+  uniforms->mvp_matrix_uniform
+    = glGetUniformLocation (program, "mvp_matrix");
+  uniforms->modelview_matrix_uniform
+    = glGetUniformLocation (program, "modelview_matrix");
+
+  for (i = 0; i < COGL_GLES2_MAX_TEXTURE_UNITS; i++)
+    if (COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (settings->texture_units, i))
+      {
+        char *matrix_var_name = g_strdup_printf ("texture_matrix[%d]", i);
+        char *sampler_var_name = g_strdup_printf ("texture_unit[%d]", i);
+        char *tex_coord_var_name =
+          g_strdup_printf ("multi_tex_coord_attrib%d", i);
+
+        uniforms->texture_matrix_uniforms[i]
+          = glGetUniformLocation (program, matrix_var_name);
+        uniforms->texture_sampler_uniforms[i]
+          = glGetUniformLocation (program, sampler_var_name);
+        attribs->multi_texture_coords[i]
+          = glGetAttribLocation (program, tex_coord_var_name);
+
+        g_free (tex_coord_var_name);
+        g_free (sampler_var_name);
+        g_free (matrix_var_name);
+      }
+    else
+      {
+        uniforms->texture_matrix_uniforms[i] = -1;
+        uniforms->texture_sampler_uniforms[i] = -1;
+        attribs->multi_texture_coords[i] = -1;
+      }
+
+  uniforms->fog_density_uniform
+    = glGetUniformLocation (program, "fog_density");
+  uniforms->fog_start_uniform
+    = glGetUniformLocation (program, "fog_start");
+  uniforms->fog_end_uniform
+    = glGetUniformLocation (program, "fog_end");
+  uniforms->fog_color_uniform
+    = glGetUniformLocation (program, "fog_color");
+
+  uniforms->alpha_test_ref_uniform
+    = glGetUniformLocation (program, "alpha_test_ref");
+}
+
+static void
+cogl_gles2_wrapper_bind_attributes (GLuint program)
+{
+  glBindAttribLocation (program, COGL_GLES2_WRAPPER_VERTEX_ATTRIB,
+			"vertex_attrib");
+  glBindAttribLocation (program, COGL_GLES2_WRAPPER_COLOR_ATTRIB,
+			"color_attrib");
+  glBindAttribLocation (program, COGL_GLES2_WRAPPER_NORMAL_ATTRIB,
+			"normal_attrib");
+}
+
 static CoglGles2WrapperProgram *
 cogl_gles2_wrapper_get_program (const CoglGles2WrapperSettings *settings)
 {
@@ -402,63 +598,27 @@ cogl_gles2_wrapper_get_program (const CoglGles2WrapperSettings *settings)
     }
 
   program->settings = *settings;
-      
-  cogl_gles2_wrapper_get_uniforms (program->program, &program->uniforms);
+
+  cogl_gles2_wrapper_get_locations (program->program,
+				    &program->settings,
+				    &program->uniforms,
+				    &program->attributes);
 
   /* We haven't tried to get a location for any of the custom uniforms
      yet */
   for (i = 0; i < COGL_GLES2_NUM_CUSTOM_UNIFORMS; i++)
     program->custom_uniforms[i] = COGL_GLES2_UNBOUND_CUSTOM_UNIFORM;
-      
+
   w->compiled_programs = g_slist_append (w->compiled_programs, program);
-      
+
   return program;
 }
 
-void
-cogl_gles2_wrapper_bind_attributes (GLuint program)
-{
-  glBindAttribLocation (program, COGL_GLES2_WRAPPER_VERTEX_ATTRIB,
-			"vertex_attrib");
-  glBindAttribLocation (program, COGL_GLES2_WRAPPER_TEX_COORD_ATTRIB,
-			"tex_coord_attrib");
-  glBindAttribLocation (program, COGL_GLES2_WRAPPER_COLOR_ATTRIB,
-			"color_attrib");
-}
-
-void
-cogl_gles2_wrapper_get_uniforms (GLuint program,
-				 CoglGles2WrapperUniforms *uniforms)
-{
-  uniforms->mvp_matrix_uniform
-    = glGetUniformLocation (program, "mvp_matrix");
-  uniforms->modelview_matrix_uniform
-    = glGetUniformLocation (program, "modelview_matrix");
-  uniforms->texture_matrix_uniform
-    = glGetUniformLocation (program, "texture_matrix");
-  uniforms->bound_texture_uniform
-    = glGetUniformLocation (program, "texture_unit");
-
-  uniforms->fog_density_uniform
-    = glGetUniformLocation (program, "fog_density");
-  uniforms->fog_start_uniform
-    = glGetUniformLocation (program, "fog_start");
-  uniforms->fog_end_uniform
-    = glGetUniformLocation (program, "fog_end");
-  uniforms->fog_color_uniform
-    = glGetUniformLocation (program, "fog_color");
-
-  uniforms->alpha_test_ref_uniform
-    = glGetUniformLocation (program, "alpha_test_ref");
-
-  uniforms->texture_unit_uniform
-    = glGetUniformLocation (program, "texture_unit");
-}
-
 void
 cogl_gles2_wrapper_deinit (CoglGles2Wrapper *wrapper)
 {
   GSList *node, *next;
+  int i;
 
   for (node = wrapper->compiled_programs; node; node = next)
     {
@@ -483,11 +643,17 @@ cogl_gles2_wrapper_deinit (CoglGles2Wrapper *wrapper)
       g_slist_free1 (node);
     }
   wrapper->compiled_fragment_shaders = NULL;
+
+  for (i = 0; i < COGL_GLES2_NUM_CUSTOM_UNIFORMS; i++)
+    if (wrapper->custom_uniforms[i].count > 1)
+      g_free (wrapper->custom_uniforms[i].v.array);
 }
 
-void
+static void
 cogl_gles2_wrapper_update_matrix (CoglGles2Wrapper *wrapper, GLenum matrix_num)
 {
+  CoglGles2WrapperTextureUnit *texture_unit;
+
   switch (matrix_num)
     {
     default:
@@ -501,20 +667,13 @@ cogl_gles2_wrapper_update_matrix (CoglGles2Wrapper *wrapper, GLenum matrix_num)
       break;
 
     case GL_TEXTURE:
-      wrapper->dirty_uniforms |= COGL_GLES2_DIRTY_TEXTURE_MATRIX;
+      wrapper->dirty_uniforms |= COGL_GLES2_DIRTY_TEXTURE_MATRICES;
+      texture_unit = wrapper->texture_units + wrapper->active_texture_unit;
+      texture_unit->dirty_matrix = 1;
       break;
     }
 }
 
-void
-cogl_wrap_glClearColorx (GLclampx r, GLclampx g, GLclampx b, GLclampx a)
-{
-  glClearColor (CLUTTER_FIXED_TO_FLOAT (r),
-		CLUTTER_FIXED_TO_FLOAT (g),
-		CLUTTER_FIXED_TO_FLOAT (b),
-		CLUTTER_FIXED_TO_FLOAT (a));
-}
-
 void
 cogl_wrap_glPushMatrix ()
 {
@@ -543,11 +702,17 @@ cogl_wrap_glPushMatrix ()
       break;
 
     case GL_TEXTURE:
-      src = w->texture_stack + w->texture_stack_pos * 16;
-      w->texture_stack_pos = (w->texture_stack_pos + 1)
-	& (COGL_GLES2_TEXTURE_STACK_SIZE - 1);
-      dst = w->texture_stack + w->texture_stack_pos * 16;
-      break;
+      {
+	CoglGles2WrapperTextureUnit *texture_unit;
+	texture_unit = w->texture_units + w->active_texture_unit;
+	src = texture_unit->texture_stack
+	      + texture_unit->texture_stack_pos * 16;
+	texture_unit->texture_stack_pos = (texture_unit->texture_stack_pos + 1)
+	  & (COGL_GLES2_TEXTURE_STACK_SIZE - 1);
+	dst = texture_unit->texture_stack
+	      + texture_unit->texture_stack_pos * 16;
+	break;
+      }
     }
 
   /* Copy the old matrix to the new position */
@@ -557,6 +722,7 @@ cogl_wrap_glPushMatrix ()
 void
 cogl_wrap_glPopMatrix ()
 {
+  CoglGles2WrapperTextureUnit *texture_unit;
   _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
 
   /* Decrement the stack pointer */
@@ -574,7 +740,8 @@ cogl_wrap_glPopMatrix ()
       break;
 
     case GL_TEXTURE:
-      w->texture_stack_pos = (w->texture_stack_pos - 1)
+      texture_unit = w->texture_units + w->active_texture_unit;
+      texture_unit->texture_stack_pos = (texture_unit->texture_stack_pos - 1)
 	& (COGL_GLES2_TEXTURE_STACK_SIZE - 1);
       break;
     }
@@ -594,6 +761,8 @@ cogl_wrap_glMatrixMode (GLenum mode)
 static float *
 cogl_gles2_get_matrix_stack_top (CoglGles2Wrapper *wrapper)
 {
+  CoglGles2WrapperTextureUnit *texture_unit;
+
   switch (wrapper->matrix_mode)
     {
     default:
@@ -604,7 +773,9 @@ cogl_gles2_get_matrix_stack_top (CoglGles2Wrapper *wrapper)
       return wrapper->projection_stack + wrapper->projection_stack_pos * 16;
 
     case GL_TEXTURE:
-      return wrapper->texture_stack + wrapper->texture_stack_pos * 16;
+      texture_unit = wrapper->texture_units + wrapper->active_texture_unit;
+      return texture_unit->texture_stack
+	+ texture_unit->texture_stack_pos * 16;
     }
 }
 
@@ -658,58 +829,66 @@ cogl_wrap_glMultMatrix (const float *m)
 }
 
 void
-cogl_wrap_glMultMatrixx (const GLfixed *m)
+cogl_wrap_glMultMatrixf (const GLfloat *m)
 {
-  float new_matrix[16];
-  int i;
-
-  for (i = 0; i < 16; i++)
-    new_matrix[i] = CLUTTER_FIXED_TO_FLOAT (m[i]);
-
-  cogl_wrap_glMultMatrix (new_matrix);
+  cogl_wrap_glMultMatrix (m);
 }
 
 void
-cogl_wrap_glFrustumx (GLfixed left, GLfixed right,
-		      GLfixed bottom, GLfixed top,
-		      GLfixed z_near, GLfixed z_far)
+cogl_wrap_glLoadMatrixf (const GLfloat *m)
+{
+  float *old_matrix;
+
+  _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
+
+  old_matrix = cogl_gles2_get_matrix_stack_top (w);
+
+  memcpy (old_matrix, m, sizeof (float) * 16);
+
+  cogl_gles2_wrapper_update_matrix (w, w->matrix_mode);
+}
+
+void
+cogl_wrap_glFrustumf (GLfloat left, GLfloat right,
+		      GLfloat bottom, GLfloat top,
+		      GLfloat z_near, GLfloat z_far)
 {
   float matrix[16];
-  float two_near = CLUTTER_FIXED_TO_FLOAT (2 * z_near);
+  float two_near =  (2 * z_near);
 
   memset (matrix, 0, sizeof (matrix));
 
-  matrix[0] = two_near / CLUTTER_FIXED_TO_FLOAT (right - left);
-  matrix[5] = two_near / CLUTTER_FIXED_TO_FLOAT (top - bottom);
-  matrix[8] = CLUTTER_FIXED_TO_FLOAT (right + left)
-    / CLUTTER_FIXED_TO_FLOAT (right - left);
-  matrix[9] = CLUTTER_FIXED_TO_FLOAT (top + bottom)
-    / CLUTTER_FIXED_TO_FLOAT (top - bottom);
-  matrix[10] = -CLUTTER_FIXED_TO_FLOAT (z_far + z_near)
-    / CLUTTER_FIXED_TO_FLOAT (z_far - z_near);
+  matrix[0] = two_near /  (right - left);
+  matrix[5] = two_near /  (top - bottom);
+  matrix[8] =  (right + left)
+    /  (right - left);
+  matrix[9] =  (top + bottom)
+    /  (top - bottom);
+  matrix[10] = - (z_far + z_near)
+    /  (z_far - z_near);
   matrix[11] = -1.0f;
-  matrix[14] = -two_near * CLUTTER_FIXED_TO_FLOAT (z_far)
-    / CLUTTER_FIXED_TO_FLOAT (z_far - z_near);
+  matrix[14] = -two_near *  (z_far)
+    /  (z_far - z_near);
 
   cogl_wrap_glMultMatrix (matrix);
 }
 
 void
-cogl_wrap_glScalex (GLfixed x, GLfixed y, GLfixed z)
+cogl_wrap_glScalef (GLfloat x, GLfloat y, GLfloat z)
 {
   float matrix[16];
 
   memset (matrix, 0, sizeof (matrix));
-  matrix[0] = CLUTTER_FIXED_TO_FLOAT (x);
-  matrix[5] = CLUTTER_FIXED_TO_FLOAT (y);
-  matrix[10] = CLUTTER_FIXED_TO_FLOAT (z);
+  matrix[0] =  (x);
+  matrix[5] =  (y);
+  matrix[10] =  (z);
   matrix[15] = 1.0f;
 
   cogl_wrap_glMultMatrix (matrix);
 }
 
 void
-cogl_wrap_glTranslatex (GLfixed x, GLfixed y, GLfixed z)
+cogl_wrap_glTranslatef (GLfloat x, GLfloat y, GLfloat z)
 {
   float matrix[16];
 
@@ -717,25 +896,25 @@ cogl_wrap_glTranslatex (GLfixed x, GLfixed y, GLfixed z)
   matrix[0] = 1.0f;
   matrix[5] = 1.0f;
   matrix[10] = 1.0f;
-  matrix[12] = CLUTTER_FIXED_TO_FLOAT (x);
-  matrix[13] = CLUTTER_FIXED_TO_FLOAT (y);
-  matrix[14] = CLUTTER_FIXED_TO_FLOAT (z);
+  matrix[12] =  (x);
+  matrix[13] =  (y);
+  matrix[14] =  (z);
   matrix[15] = 1.0f;
 
   cogl_wrap_glMultMatrix (matrix);
 }
 
 void
-cogl_wrap_glRotatex (GLfixed angle, GLfixed x, GLfixed y, GLfixed z)
+cogl_wrap_glRotatef (GLfloat angle, GLfloat x, GLfloat y, GLfloat z)
 {
   float matrix[16];
-  float xf = CLUTTER_FIXED_TO_FLOAT (x);
-  float yf = CLUTTER_FIXED_TO_FLOAT (y);
-  float zf = CLUTTER_FIXED_TO_FLOAT (z);
-  float anglef = CLUTTER_FIXED_TO_FLOAT (angle) * G_PI / 180.0f;
+  float xf =  (x);
+  float yf =  (y);
+  float zf =  (z);
+  float anglef =  (angle) * G_PI / 180.0f;
   float c = cosf (anglef);
   float s = sinf (anglef);
-  
+
   matrix[0]  = xf * xf * (1.0f - c) + c;
   matrix[1]  = yf * xf * (1.0f - c) + zf * s;
   matrix[2]  = xf * zf * (1.0f - c) - yf * s;
@@ -760,21 +939,21 @@ cogl_wrap_glRotatex (GLfixed angle, GLfixed x, GLfixed y, GLfixed z)
 }
 
 void
-cogl_wrap_glOrthox (GLfixed left, GLfixed right, GLfixed bottom, GLfixed top,
-		    GLfixed near, GLfixed far)
+cogl_wrap_glOrthof (GLfloat left, GLfloat right, GLfloat bottom, GLfloat top,
+		    GLfloat near, GLfloat far)
 {
   float matrix[16];
-  float xrange = CLUTTER_FIXED_TO_FLOAT (right - left);
-  float yrange = CLUTTER_FIXED_TO_FLOAT (top - bottom);
-  float zrange = CLUTTER_FIXED_TO_FLOAT (far - near);
+  float xrange =  (right - left);
+  float yrange =  (top - bottom);
+  float zrange =  (far - near);
 
   memset (matrix, 0, sizeof (matrix));
   matrix[0] = 2.0f / xrange;
   matrix[5] = 2.0f / yrange;
   matrix[10] = 2.0f / zrange;
-  matrix[12] = CLUTTER_FIXED_TO_FLOAT (right + left) / xrange;
-  matrix[13] = CLUTTER_FIXED_TO_FLOAT (top + bottom) / yrange;
-  matrix[14] = CLUTTER_FIXED_TO_FLOAT (far + near) / zrange;
+  matrix[12] =  (right + left) / xrange;
+  matrix[13] =  (top + bottom) / yrange;
+  matrix[14] =  (far + near) / zrange;
   matrix[15] = 1.0f;
 
   cogl_wrap_glMultMatrix (matrix);
@@ -792,8 +971,20 @@ void
 cogl_wrap_glTexCoordPointer (GLint size, GLenum type, GLsizei stride,
 			     const GLvoid *pointer)
 {
-  glVertexAttribPointer (COGL_GLES2_WRAPPER_TEX_COORD_ATTRIB, size, type,
-			 GL_FALSE, stride, pointer);
+  int active_unit;
+  CoglGles2WrapperTextureUnit *texture_unit;
+  _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
+
+  active_unit = w->active_client_texture_unit;
+
+  texture_unit = w->texture_units + active_unit;
+  texture_unit->texture_coords_size = size;
+  texture_unit->texture_coords_type = type;
+  texture_unit->texture_coords_stride = stride;
+  texture_unit->texture_coords_pointer = pointer;
+
+  w->dirty_attribute_pointers
+    |= COGL_GLES2_DIRTY_TEX_COORD_VERTEX_ATTRIB;
 }
 
 void
@@ -801,9 +992,88 @@ cogl_wrap_glColorPointer (GLint size, GLenum type, GLsizei stride,
 			  const GLvoid *pointer)
 {
   glVertexAttribPointer (COGL_GLES2_WRAPPER_COLOR_ATTRIB, size, type,
+			 GL_TRUE, stride, pointer);
+}
+
+void
+cogl_wrap_glNormalPointer (GLenum type, GLsizei stride, const GLvoid *pointer)
+{
+  glVertexAttribPointer (COGL_GLES2_WRAPPER_NORMAL_ATTRIB, 1, type,
 			 GL_FALSE, stride, pointer);
 }
 
+static void
+cogl_gles2_do_set_uniform (GLint location, CoglBoxedValue *value)
+{
+  switch (value->type)
+    {
+    case COGL_BOXED_NONE:
+      break;
+
+    case COGL_BOXED_INT:
+      {
+	gint *ptr;
+
+	if (value->count == 1)
+	  ptr = value->v.int_value;
+	else
+	  ptr = value->v.int_array;
+
+	switch (value->size)
+	  {
+	  case 1: glUniform1iv (location, value->count, ptr); break;
+	  case 2: glUniform2iv (location, value->count, ptr); break;
+	  case 3: glUniform3iv (location, value->count, ptr); break;
+	  case 4: glUniform4iv (location, value->count, ptr); break;
+	  }
+      }
+      break;
+
+    case COGL_BOXED_FLOAT:
+      {
+	gfloat *ptr;
+
+	if (value->count == 1)
+	  ptr = value->v.float_value;
+	else
+	  ptr = value->v.float_array;
+
+	switch (value->size)
+	  {
+	  case 1: glUniform1fv (location, value->count, ptr); break;
+	  case 2: glUniform2fv (location, value->count, ptr); break;
+	  case 3: glUniform3fv (location, value->count, ptr); break;
+	  case 4: glUniform4fv (location, value->count, ptr); break;
+	  }
+      }
+      break;
+
+    case COGL_BOXED_MATRIX:
+      {
+	gfloat *ptr;
+
+	if (value->count == 1)
+	  ptr = value->v.matrix;
+	else
+	  ptr = value->v.float_array;
+
+	switch (value->size)
+	  {
+	  case 2:
+	    glUniformMatrix2fv (location, value->count, value->transpose, ptr);
+	    break;
+	  case 3:
+	    glUniformMatrix3fv (location, value->count, value->transpose, ptr);
+	    break;
+	  case 4:
+	    glUniformMatrix4fv (location, value->count, value->transpose, ptr);
+	    break;
+	  }
+      }
+      break;
+    }
+}
+
 static void
 cogl_wrap_prepare_for_draw (void)
 {
@@ -848,7 +1118,7 @@ cogl_wrap_prepare_for_draw (void)
 	  cogl_gles2_wrapper_mult_matrix (mvp_matrix,
 					  w->projection_stack
 					  + w->projection_stack_pos * 16,
-					  modelview_matrix);	  
+					  modelview_matrix);
 
 	  if (program->uniforms.mvp_matrix_uniform != -1)
 	    glUniformMatrix4fv (program->uniforms.mvp_matrix_uniform, 1,
@@ -857,11 +1127,24 @@ cogl_wrap_prepare_for_draw (void)
 	    glUniformMatrix4fv (program->uniforms.modelview_matrix_uniform, 1,
 				GL_FALSE, modelview_matrix);
 	}
-      if ((w->dirty_uniforms & COGL_GLES2_DIRTY_TEXTURE_MATRIX)
-	  && program->uniforms.texture_matrix_uniform != -1)
-	glUniformMatrix4fv (program->uniforms.texture_matrix_uniform, 1,
-			    GL_FALSE,
-			    w->texture_stack + w->texture_stack_pos * 16);
+      if ((w->dirty_uniforms & COGL_GLES2_DIRTY_TEXTURE_MATRICES))
+	{
+	  int i;
+
+	  /* TODO - we should probably have a per unit dirty flag too */
+
+	  for (i = 0; i < COGL_GLES2_MAX_TEXTURE_UNITS; i++)
+	    {
+	      CoglGles2WrapperTextureUnit *texture_unit;
+	      GLint uniform = program->uniforms.texture_matrix_uniforms[i];
+
+              texture_unit = w->texture_units + i;
+	      if (uniform != -1)
+		glUniformMatrix4fv (uniform, 1, GL_FALSE,
+				    texture_unit->texture_stack
+				      + texture_unit->texture_stack_pos * 16);
+	    }
+	}
 
       if ((w->dirty_uniforms & COGL_GLES2_DIRTY_FOG_DENSITY)
 	  && program->uniforms.fog_density_uniform != -1)
@@ -878,9 +1161,20 @@ cogl_wrap_prepare_for_draw (void)
 	glUniform1f (program->uniforms.alpha_test_ref_uniform,
 		     w->alpha_test_ref);
 
-      if ((w->dirty_uniforms & COGL_GLES2_DIRTY_TEXTURE_UNIT)
-          && program->uniforms.texture_unit_uniform != -1)
-        glUniform1i (program->uniforms.texture_unit_uniform, 0);
+      if ((w->dirty_uniforms & COGL_GLES2_DIRTY_TEXTURE_UNITS))
+        {
+          int i;
+
+          /* TODO - we should probably have a per unit dirty flag too */
+
+          for (i = 0; i < COGL_GLES2_MAX_TEXTURE_UNITS; i++)
+            {
+              GLint uniform = program->uniforms.texture_sampler_uniforms[i];
+
+              if (uniform != -1)
+                glUniform1i (uniform, i);
+            }
+        }
 
       w->dirty_uniforms = 0;
     }
@@ -904,13 +1198,68 @@ cogl_wrap_prepare_for_draw (void)
 		  program->custom_uniforms[i]
 		    = glGetUniformLocation (program->program, uniform_name);
 		if (program->custom_uniforms[i] >= 0)
-		  glUniform1f (program->custom_uniforms[i],
-			       w->custom_uniforms[i]);
+		  cogl_gles2_do_set_uniform (program->custom_uniforms[i],
+					     &w->custom_uniforms[i]);
 	      }
 	}
-      
+
       w->dirty_custom_uniforms = 0;
     }
+
+  if (w->dirty_attribute_pointers
+      & COGL_GLES2_DIRTY_TEX_COORD_VERTEX_ATTRIB)
+    {
+      int i;
+
+      /* TODO - coverage test */
+      for (i = 0; i < COGL_GLES2_MAX_TEXTURE_UNITS; i++)
+        if (COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (w->settings.texture_units, i))
+          {
+            GLint tex_coord_var_index;
+            CoglGles2WrapperTextureUnit *texture_unit;
+
+            texture_unit = w->texture_units + w->active_texture_unit;
+            if (!texture_unit->texture_coords_enabled)
+              continue;
+
+            /* TODO - we should probably have a per unit dirty flag too */
+
+            /* TODO - coverage test */
+            tex_coord_var_index = program->attributes.multi_texture_coords[i];
+            glVertexAttribPointer (tex_coord_var_index,
+                                   texture_unit->texture_coords_size,
+                                   texture_unit->texture_coords_type,
+                                   GL_FALSE,
+                                   texture_unit->texture_coords_stride,
+                                   texture_unit->texture_coords_pointer);
+          }
+    }
+
+  if (w->dirty_vertex_attrib_enables)
+    {
+      int i;
+
+      /* TODO - coverage test */
+
+      /* TODO - we should probably have a per unit dirty flag too */
+
+      for (i = 0; i < COGL_GLES2_MAX_TEXTURE_UNITS; i++)
+	{
+	  CoglGles2WrapperTextureUnit *texture_unit
+            = w->texture_units + w->active_texture_unit;
+          GLint attrib = program->attributes.multi_texture_coords[i];
+
+          if (attrib != -1)
+            {
+              if (texture_unit->texture_coords_enabled)
+                glEnableVertexAttribArray (attrib);
+              else
+                glDisableVertexAttribArray (attrib);
+            }
+	}
+
+      w->dirty_vertex_attrib_enables = 0;
+    }
 }
 
 void
@@ -941,17 +1290,47 @@ cogl_gles2_wrapper_bind_texture (GLenum target, GLuint texture,
   /* We need to keep track of whether the texture is alpha-only
      because the emulation of GL_MODULATE needs to work differently in
      that case */
-  _COGL_GLES2_CHANGE_SETTING (w, alpha_only, internal_format == GL_ALPHA);
+  if (COGL_GLES2_TEXTURE_UNIT_IS_ALPHA_ONLY (w->settings.texture_units,
+                                             w->active_texture_unit)
+      != (internal_format == GL_ALPHA))
+    {
+      COGL_GLES2_TEXTURE_UNIT_SET_ALPHA_ONLY (w->settings.texture_units,
+                                              w->active_texture_unit,
+                                              internal_format == GL_ALPHA);
+      w->settings_dirty = TRUE;
+    }
 }
 
 void
-cogl_wrap_glTexEnvx (GLenum target, GLenum pname, GLfixed param)
+cogl_wrap_glTexEnvi (GLenum target, GLenum pname, GLfloat param)
 {
   /* This function is only used to set the texture mode once to
      GL_MODULATE. The shader is hard-coded to modulate the texture so
      nothing needs to be done here. */
 }
 
+void
+cogl_wrap_glClientActiveTexture (GLenum texture)
+{
+  int texture_unit_index = texture - GL_TEXTURE0;
+  _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
+
+  if (texture_unit_index < COGL_GLES2_MAX_TEXTURE_UNITS)
+    w->active_client_texture_unit = texture_unit_index;
+}
+
+void
+cogl_wrap_glActiveTexture (GLenum texture)
+{
+  int texture_unit_index = texture - GL_TEXTURE0;
+  _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
+
+  glActiveTexture (texture);
+
+  if (texture_unit_index < COGL_GLES2_MAX_TEXTURE_UNITS)
+    w->active_texture_unit = texture_unit_index;
+}
+
 void
 cogl_wrap_glEnable (GLenum cap)
 {
@@ -960,7 +1339,14 @@ cogl_wrap_glEnable (GLenum cap)
   switch (cap)
     {
     case GL_TEXTURE_2D:
-      _COGL_GLES2_CHANGE_SETTING (w, texture_2d_enabled, TRUE);
+      if (!COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (w->settings.texture_units,
+                                               w->active_texture_unit))
+        {
+          COGL_GLES2_TEXTURE_UNIT_SET_ENABLED (w->settings.texture_units,
+                                               w->active_texture_unit,
+                                               TRUE);
+          w->settings_dirty = TRUE;
+        }
       break;
 
     case GL_FOG:
@@ -984,7 +1370,14 @@ cogl_wrap_glDisable (GLenum cap)
   switch (cap)
     {
     case GL_TEXTURE_2D:
-      _COGL_GLES2_CHANGE_SETTING (w, texture_2d_enabled, FALSE);
+      if (COGL_GLES2_TEXTURE_UNIT_IS_ENABLED (w->settings.texture_units,
+                                              w->active_texture_unit))
+        {
+          COGL_GLES2_TEXTURE_UNIT_SET_ENABLED (w->settings.texture_units,
+                                               w->active_texture_unit,
+                                               FALSE);
+          w->settings_dirty = TRUE;
+        }
       break;
 
     case GL_FOG:
@@ -1003,34 +1396,62 @@ cogl_wrap_glDisable (GLenum cap)
 void
 cogl_wrap_glEnableClientState (GLenum array)
 {
+  CoglGles2WrapperTextureUnit *texture_unit;
+  _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
+
   switch (array)
     {
     case GL_VERTEX_ARRAY:
       glEnableVertexAttribArray (COGL_GLES2_WRAPPER_VERTEX_ATTRIB);
       break;
     case GL_TEXTURE_COORD_ARRAY:
-      glEnableVertexAttribArray (COGL_GLES2_WRAPPER_TEX_COORD_ATTRIB);
+      /* TODO - review if this should be in w->settings? */
+
+      texture_unit = w->texture_units + w->active_texture_unit;
+      if (texture_unit->texture_coords_enabled != 1)
+	{
+	  texture_unit->texture_coords_enabled = 1;
+	  w->dirty_vertex_attrib_enables
+	    |= COGL_GLES2_DIRTY_TEX_COORD_ATTRIB_ENABLES;
+	}
       break;
     case GL_COLOR_ARRAY:
       glEnableVertexAttribArray (COGL_GLES2_WRAPPER_COLOR_ATTRIB);
       break;
+    case GL_NORMAL_ARRAY:
+      glEnableVertexAttribArray (COGL_GLES2_WRAPPER_NORMAL_ATTRIB);
+      break;
     }
 }
 
 void
 cogl_wrap_glDisableClientState (GLenum array)
 {
+  CoglGles2WrapperTextureUnit *texture_unit;
+  _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
+
   switch (array)
     {
     case GL_VERTEX_ARRAY:
       glDisableVertexAttribArray (COGL_GLES2_WRAPPER_VERTEX_ATTRIB);
       break;
     case GL_TEXTURE_COORD_ARRAY:
-      glDisableVertexAttribArray (COGL_GLES2_WRAPPER_TEX_COORD_ATTRIB);
+
+      texture_unit = w->texture_units + w->active_texture_unit;
+      /* TODO - review if this should be in w->settings? */
+      if (texture_unit->texture_coords_enabled != 0)
+	{
+	  texture_unit->texture_coords_enabled = 0;
+	  w->dirty_vertex_attrib_enables
+	    |= COGL_GLES2_DIRTY_TEX_COORD_ATTRIB_ENABLES;
+	}
       break;
     case GL_COLOR_ARRAY:
       glDisableVertexAttribArray (COGL_GLES2_WRAPPER_COLOR_ATTRIB);
       break;
+    case GL_NORMAL_ARRAY:
+      glDisableVertexAttribArray (COGL_GLES2_WRAPPER_NORMAL_ATTRIB);
+      break;
     }
 }
 
@@ -1049,29 +1470,17 @@ cogl_wrap_glAlphaFunc (GLenum func, GLclampf ref)
 }
 
 void
-cogl_wrap_glColor4x (GLclampx r, GLclampx g, GLclampx b, GLclampx a)
+cogl_wrap_glColor4f (GLclampf r, GLclampf g, GLclampf b, GLclampf a)
 {
-  glVertexAttrib4f (COGL_GLES2_WRAPPER_COLOR_ATTRIB,
-		    CLUTTER_FIXED_TO_FLOAT (r),
-		    CLUTTER_FIXED_TO_FLOAT (g),
-		    CLUTTER_FIXED_TO_FLOAT (b),
-		    CLUTTER_FIXED_TO_FLOAT (a));
+  glVertexAttrib4f (COGL_GLES2_WRAPPER_COLOR_ATTRIB, r, g, b, a);
 }
 
 void
-cogl_wrap_glClipPlanex (GLenum plane, GLfixed *equation)
+cogl_wrap_glClipPlanef (GLenum plane, GLfloat *equation)
 {
   /* FIXME */
 }
 
-static void
-cogl_gles2_float_array_to_fixed (int size, const GLfloat *floats,
-				 GLfixed *fixeds)
-{
-  while (size-- > 0)
-    *(fixeds++) = CLUTTER_FLOAT_TO_FIXED (*(floats++));
-}
-
 void
 cogl_wrap_glGetIntegerv (GLenum pname, GLint *params)
 {
@@ -1083,6 +1492,10 @@ cogl_wrap_glGetIntegerv (GLenum pname, GLint *params)
       *params = 0;
       break;
 
+    case CGL_MATRIX_MODE:
+      *params = w->matrix_mode;
+      break;
+
     default:
       glGetIntegerv (pname, params);
       break;
@@ -1090,37 +1503,40 @@ cogl_wrap_glGetIntegerv (GLenum pname, GLint *params)
 }
 
 void
-cogl_wrap_glGetFixedv (GLenum pname, GLfixed *params)
+cogl_wrap_glGetFloatv (GLenum pname, GLfloat *params)
 {
+  CoglGles2WrapperTextureUnit *texture_unit;
+
   _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
 
   switch (pname)
     {
     case GL_MODELVIEW_MATRIX:
-      cogl_gles2_float_array_to_fixed (16, w->modelview_stack
-				       + w->modelview_stack_pos * 16,
-				       params);
+      memcpy (params, w->modelview_stack + w->modelview_stack_pos * 16,
+              sizeof (GLfloat) * 16);
       break;
 
     case GL_PROJECTION_MATRIX:
-      cogl_gles2_float_array_to_fixed (16, w->projection_stack
-				       + w->projection_stack_pos * 16,
-				       params);
+      memcpy (params, w->projection_stack + w->projection_stack_pos * 16,
+              sizeof (GLfloat) * 16);
+      break;
+
+    case GL_TEXTURE_MATRIX:
+      texture_unit = w->texture_units + w->active_texture_unit;
+      memcpy (params,
+              texture_unit->texture_stack
+               + texture_unit->texture_stack_pos * 16,
+              sizeof (GLfloat) * 16);
       break;
 
     case GL_VIEWPORT:
-      {
-	GLfloat v[4];
-
-	glGetFloatv (GL_VIEWPORT, v);
-	cogl_gles2_float_array_to_fixed (4, v, params);
-      }
+      glGetFloatv (GL_VIEWPORT, params);
       break;
     }
 }
 
 void
-cogl_wrap_glFogx (GLenum pname, GLfixed param)
+cogl_wrap_glFogf (GLenum pname, GLfloat param)
 {
   _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
 
@@ -1129,26 +1545,26 @@ cogl_wrap_glFogx (GLenum pname, GLfixed param)
     case GL_FOG_MODE:
       _COGL_GLES2_CHANGE_SETTING (w, fog_mode, param);
       break;
-      
+
     case GL_FOG_DENSITY:
       _COGL_GLES2_CHANGE_UNIFORM (w, FOG_DENSITY, fog_density,
-				  CLUTTER_FIXED_TO_FLOAT (param));
+				   (param));
       break;
 
     case GL_FOG_START:
       _COGL_GLES2_CHANGE_UNIFORM (w, FOG_START, fog_start,
-				  CLUTTER_FIXED_TO_FLOAT (param));
+				   (param));
       break;
 
     case GL_FOG_END:
       _COGL_GLES2_CHANGE_UNIFORM (w, FOG_END, fog_end,
-				  CLUTTER_FIXED_TO_FLOAT (param));
+				   (param));
       break;
     }
 }
 
 void
-cogl_wrap_glFogxv (GLenum pname, const GLfixed *params)
+cogl_wrap_glFogfv (GLenum pname, const GLfloat *params)
 {
   int i;
   _COGL_GET_GLES2_WRAPPER (w, NO_RETVAL);
@@ -1156,7 +1572,8 @@ cogl_wrap_glFogxv (GLenum pname, const GLfixed *params)
   if (pname == GL_FOG_COLOR)
     {
       for (i = 0; i < 4; i++)
-	w->fog_color[i] = CLUTTER_FIXED_TO_FLOAT (params[i]);
+	w->fog_color[i] =  (params[i]);
+
       w->dirty_uniforms |= COGL_GLES2_DIRTY_FOG_COLOR;
     }
 }
@@ -1168,6 +1585,13 @@ cogl_wrap_glTexParameteri (GLenum target, GLenum pname, GLfloat param)
     glTexParameteri (target, pname, param);
 }
 
+void
+cogl_wrap_glMaterialfv (GLenum face, GLenum pname, const GLfloat *params)
+{
+  /* FIXME: the GLES 2 backend doesn't yet support lighting so this
+     function can't do anything */
+}
+
 void
 _cogl_gles2_clear_cache_for_program (CoglHandle user_program)
 {
