@@ -29,6 +29,8 @@
 #include <unistd.h>
 #endif
 
+#include <glib/gi18n-lib.h>
+
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <fcntl.h>
@@ -135,20 +137,22 @@ clutter_backend_glx_post_parse (ClutterBackend  *backend,
           g_set_error (error, CLUTTER_INIT_ERROR,
                        CLUTTER_INIT_ERROR_BACKEND,
                        "XServer appears to lack required GLX support");
-          return 1;
+
+          return FALSE;
         }
     }
+  else
+    return FALSE;
 
   return TRUE;
 }
 
-
 static const GOptionEntry entries[] =
 {
   { "vblank", 0, 
     0, 
     G_OPTION_ARG_STRING, &clutter_vblank_name,
-    "VBlank method to be used (none, dri or glx)", "METHOD" 
+    N_("VBlank method to be used (none, dri or glx)"), "METHOD"
   },
   { NULL }
 };
