@@ -40,6 +40,13 @@
  *
  * The class requires the GLX_EXT_texture_from_pixmap OpenGL extension
  * (http://people.freedesktop.org/~davidr/GLX_EXT_texture_from_pixmap.txt)
+ *
+ * The GL_ARB_texture_non_power_of_two extension will be used if it is
+ * available. Otherwise it will try to use the
+ * GL_ARB_texture_rectangle extension. If both are available you can
+ * force it to prefer the rectangle extension by setting the
+ * CLUTTER_PIXMAP_TEXTURE_RECTANGLE to 'force'. To prevent it ever
+ * using the rectangle extension you can set it to 'disable'.
  */
 
 
@@ -48,6 +55,8 @@
 #include "config.h"
 #endif
 
+#include <string.h>
+
 #include "../x11/clutter-x11-texture-pixmap.h"
 #include "clutter-glx-texture-pixmap.h"
 #include "clutter-glx.h"
@@ -66,10 +75,22 @@ typedef void    (*ReleaseTexImage) (Display     *display,
                                     GLXDrawable  drawable,
                                     int          buffer);
 
+typedef void    (*GenerateMipmap) (GLenum target);
+
+typedef enum
+{
+  CLUTTER_GLX_RECTANGLE_DISALLOW,
+  CLUTTER_GLX_RECTANGLE_ALLOW,
+  CLUTTER_GLX_RECTANGLE_FORCE
+} RectangleState;
+
 static BindTexImage      _gl_bind_tex_image = NULL;
 static ReleaseTexImage   _gl_release_tex_image = NULL;
+static GenerateMipmap    _gl_generate_mipmap = NULL;
 static gboolean          _have_tex_from_pixmap_ext = FALSE;
 static gboolean          _ext_check_done = FALSE;
+static gboolean          _have_tex_rectangle = FALSE;
+static RectangleState    _rectangle_state = CLUTTER_GLX_RECTANGLE_ALLOW;
 
 struct _ClutterGLXTexturePixmapPrivate
 {
@@ -80,7 +101,10 @@ struct _ClutterGLXTexturePixmapPrivate
   gboolean      use_fallback;
 
   gboolean      bound;
-  
+  gint          can_mipmap;
+  gint          mipmap_generate_queued;
+
+  gboolean      using_rectangle;
 };
 
 static void 
@@ -102,22 +126,45 @@ G_DEFINE_TYPE (ClutterGLXTexturePixmap,    \
 static gboolean
 texture_bind (ClutterGLXTexturePixmap *tex)
 {
-  GLuint     handle = 0;
-  GLenum     target = 0;
+  CoglHandle cogl_material;
   CoglHandle cogl_tex;
+
+  /* It might be better to track if we've succesfully set a texture yet
+   * explicitly, rather than doing it indirectly like this.
+   */
   cogl_tex = clutter_texture_get_cogl_texture (CLUTTER_TEXTURE(tex));
+  if (cogl_tex == COGL_INVALID_HANDLE)
+    return FALSE;
 
-  if (!cogl_texture_get_gl_texture (cogl_tex, &handle, &target))
-      return FALSE;
-
-  glEnable(target);
-
-  /* FIXME: fire off an error here? */
-  glBindTexture (target, handle);
+  cogl_material = clutter_texture_get_cogl_material (CLUTTER_TEXTURE(tex));
+  cogl_material_flush_gl_state (cogl_material, NULL);
 
   return TRUE;
 }
 
+static void
+on_glx_texture_pixmap_pre_paint (ClutterGLXTexturePixmap *texture,
+                                 gpointer                 user_data)
+{
+  if (texture->priv->mipmap_generate_queued)
+    {
+      GLuint     handle = 0;
+      GLenum     target = 0;
+      CoglHandle cogl_tex;
+      cogl_tex = clutter_texture_get_cogl_texture 
+        (CLUTTER_TEXTURE(texture));
+
+      texture_bind (texture);
+
+      cogl_texture_get_gl_texture (cogl_tex, &handle, &target);
+      
+      _gl_generate_mipmap (target);
+
+      texture->priv->mipmap_generate_queued = 0;
+    }
+
+}
+
 static void
 clutter_glx_texture_pixmap_init (ClutterGLXTexturePixmap *self)
 {
@@ -128,9 +175,15 @@ clutter_glx_texture_pixmap_init (ClutterGLXTexturePixmap *self)
                                    CLUTTER_GLX_TYPE_TEXTURE_PIXMAP,
                                    ClutterGLXTexturePixmapPrivate);
 
+  g_signal_connect (CLUTTER_ACTOR(self),
+                    "paint", G_CALLBACK (on_glx_texture_pixmap_pre_paint),
+                    NULL);
+
   if (_ext_check_done == FALSE)
     {
+      const char *gl_extensions = NULL;
       const gchar *glx_extensions = NULL;
+      const char *rect_env;
 
       glx_extensions =
         glXQueryExtensionsString (clutter_x11_get_default_display (),
@@ -148,16 +201,62 @@ clutter_glx_texture_pixmap_init (ClutterGLXTexturePixmap *self)
             _have_tex_from_pixmap_ext = TRUE;
         }
 
+      _gl_generate_mipmap =
+        (GenerateMipmap)cogl_get_proc_address ("glGenerateMipmapEXT");
+
+      gl_extensions = (char *) glGetString (GL_EXTENSIONS);
+      _have_tex_rectangle = cogl_check_extension ("GL_ARB_texture_rectangle",
+                                                  gl_extensions);
+
+      if ((rect_env = g_getenv ("CLUTTER_PIXMAP_TEXTURE_RECTANGLE")))
+        {
+          if (strcasecmp (rect_env, "force") == 0)
+            _rectangle_state = CLUTTER_GLX_RECTANGLE_FORCE;
+	  else if (strcasecmp (rect_env, "disable") == 0)
+            _rectangle_state = CLUTTER_GLX_RECTANGLE_DISALLOW;
+	  else if (rect_env[0])
+	    g_warning ("Unknown value for CLUTTER_PIXMAP_TEXTURE_RECTANGLE, "
+		       "should be 'force' or 'disable'");
+        }
+
       _ext_check_done = TRUE;
     }
 }
 
+static void
+clutter_glx_texture_pixmap_free_rectangle (ClutterGLXTexturePixmap *texture)
+{
+  ClutterGLXTexturePixmapPrivate *priv = texture->priv;
+  CoglHandle cogl_tex;
+
+  /* Cogl won't free the GL texture resource if it was created with
+     new_from_foreign so we need to free it manually */
+  if (priv->using_rectangle)
+    {
+      cogl_tex = clutter_texture_get_cogl_texture (CLUTTER_TEXTURE (texture));
+
+      if (cogl_tex != COGL_INVALID_HANDLE)
+        {
+          GLuint gl_handle;
+          GLenum gl_target;
+
+          cogl_texture_get_gl_texture (cogl_tex, &gl_handle, &gl_target);
+
+          if (gl_target == CGL_TEXTURE_RECTANGLE_ARB)
+            glDeleteTextures (1, &gl_handle);
+        }
+
+      priv->using_rectangle = FALSE;
+    }
+}
+
 static void
 clutter_glx_texture_pixmap_dispose (GObject *object)
 {
-  ClutterGLXTexturePixmapPrivate *priv;
+  ClutterGLXTexturePixmap *texture = CLUTTER_GLX_TEXTURE_PIXMAP (object);
+  ClutterGLXTexturePixmapPrivate *priv = texture->priv;
 
-  priv = CLUTTER_GLX_TEXTURE_PIXMAP (object)->priv;
+  clutter_glx_texture_pixmap_free_rectangle (texture);
 
   if (priv->glx_pixmap != None)
     {
@@ -185,22 +284,88 @@ clutter_glx_texture_pixmap_notify (GObject *object, GParamSpec *pspec)
     }
 }
 
+static gboolean
+should_use_rectangle (void)
+{
+  /* Use the rectangle only if it is available and either:
+
+     the CLUTTER_PIXMAP_TEXTURE_RECTANGLE environment variable is
+     set to 'force'
+
+     *or*
+
+     the env var is set to 'allow' (which is the default) and NPOTs
+     textures are not available */
+  return _have_tex_rectangle
+    && ((_rectangle_state == CLUTTER_GLX_RECTANGLE_ALLOW
+         && !clutter_feature_available (CLUTTER_FEATURE_TEXTURE_NPOT))
+        || _rectangle_state == CLUTTER_GLX_RECTANGLE_FORCE);
+}
+
 static gboolean
 create_cogl_texture (ClutterTexture *texture,
 		     guint width,
 		     guint height)
 {
-  CoglHandle  handle;
+  ClutterGLXTexturePixmap *texture_glx = CLUTTER_GLX_TEXTURE_PIXMAP (texture);
+  ClutterGLXTexturePixmapPrivate *priv = texture_glx->priv;
+  CoglHandle      handle;
+  gboolean        using_rectangle;
+  GLint           gl_format;
+  CoglPixelFormat cogl_format = COGL_PIXEL_FORMAT_RGBA_8888;
+  guint           depth;
 
-  handle 
-    = cogl_texture_new_with_size (width, height,
-                                  -1, FALSE,
-                                  COGL_PIXEL_FORMAT_RGBA_8888|COGL_BGR_BIT);
+  g_object_get (G_OBJECT (texture_glx), "pixmap-depth", &depth, NULL);
+  if (depth == 32)
+    {
+      gl_format = GL_RGBA;
+      cogl_format = COGL_PIXEL_FORMAT_RGBA_8888;
+    }
+  else if (depth == 24)
+    {
+      gl_format = GL_RGB;
+      cogl_format = COGL_PIXEL_FORMAT_RGB_888;
+    }
+  else
+    g_critical ("Can't create a TFP cogl texture for pixmap with depth < 24");
+
+  /* We want to use the GL_ARB_texture_rectangle extension on some
+     chipsets because GL_ARB_texture_non_power_of_two is not always
+     supported or might be buggy */
+  if (should_use_rectangle ())
+    {
+      GLuint tex = 0;
+
+      using_rectangle = TRUE;
+
+      glGenTextures (1, &tex);
+      glBindTexture (CGL_TEXTURE_RECTANGLE_ARB, tex);
+      glTexImage2D (CGL_TEXTURE_RECTANGLE_ARB, 0,
+                    GL_RGB, width, height,
+                    0, GL_RGB, GL_UNSIGNED_BYTE, NULL);
+
+      handle = cogl_texture_new_from_foreign (tex, CGL_TEXTURE_RECTANGLE_ARB,
+                                              width, height,
+                                              0, 0,
+                                              cogl_format | COGL_BGR_BIT);
+    }
+  else
+    {
+      handle
+        = cogl_texture_new_with_size (width, height,
+                                      -1, FALSE,
+                                      cogl_format | COGL_BGR_BIT);
+
+      using_rectangle = FALSE;
+    }
 
   if (handle)
     {
+      clutter_glx_texture_pixmap_free_rectangle (texture_glx);
+      priv->using_rectangle = using_rectangle;
+
       clutter_texture_set_cogl_texture (texture, handle);
-      
+
       CLUTTER_ACTOR_SET_FLAGS (texture, CLUTTER_ACTOR_REALIZED);
 
       clutter_glx_texture_pixmap_update_area
@@ -258,12 +423,14 @@ clutter_glx_texture_pixmap_realize (ClutterActor *actor)
 static void
 clutter_glx_texture_pixmap_unrealize (ClutterActor *actor)
 {
-  ClutterGLXTexturePixmapPrivate *priv;
+  ClutterGLXTexturePixmap        *texture = CLUTTER_GLX_TEXTURE_PIXMAP (actor);
+  ClutterGLXTexturePixmapPrivate *priv = texture->priv;
   Display                        *dpy;
 
-  priv = CLUTTER_GLX_TEXTURE_PIXMAP (actor)->priv;
   dpy = clutter_x11_get_default_display();
 
+  clutter_glx_texture_pixmap_free_rectangle (texture);
+
   if (!_have_tex_from_pixmap_ext)
     {
       CLUTTER_ACTOR_CLASS (clutter_glx_texture_pixmap_parent_class)->
@@ -292,13 +459,23 @@ clutter_glx_texture_pixmap_unrealize (ClutterActor *actor)
 }
 
 static GLXFBConfig *
-get_fbconfig_for_depth (guint depth)
+get_fbconfig_for_depth (ClutterGLXTexturePixmap *texture, guint depth)
 {
   GLXFBConfig *fbconfigs, *ret = NULL;
   int          n_elements, i, found;
   Display     *dpy;
   int          db, stencil, alpha, mipmap, rgba, value;
 
+  static GLXFBConfig *cached_config = NULL;
+  static gboolean     have_cached_config = FALSE;
+  static int          cached_mipmap = 0;
+
+  if (have_cached_config)
+    {
+      texture->priv->can_mipmap = cached_mipmap;
+      return cached_config;
+    }
+
   dpy = clutter_x11_get_default_display ();
 
   fbconfigs = glXGetFBConfigs (dpy,
@@ -312,7 +489,6 @@ get_fbconfig_for_depth (guint depth)
 
   found = n_elements;
 
-
   for (i = 0; i < n_elements; i++)
     {
       XVisualInfo *vi;
@@ -383,6 +559,19 @@ get_fbconfig_for_depth (guint depth)
 
       stencil = value;
 
+      if (_gl_generate_mipmap)
+        {
+          glXGetFBConfigAttrib (dpy,
+                                fbconfigs[i],
+                                GLX_BIND_TO_MIPMAP_TEXTURE_EXT,
+                                &value);
+
+          if (value < mipmap)
+            continue;
+
+          mipmap =  value;
+        }
+
       found = i;
     }
 
@@ -395,6 +584,10 @@ get_fbconfig_for_depth (guint depth)
   if (n_elements)
     XFree (fbconfigs);
 
+  have_cached_config = TRUE;
+  cached_config = ret;
+  texture->priv->can_mipmap = cached_mipmap = mipmap;
+
   return ret;
 }
 
@@ -455,8 +648,7 @@ clutter_glx_texture_pixmap_create_glx_pixmap (ClutterGLXTexturePixmap *texture)
 
   dpy = clutter_x11_get_default_display ();
 
-  if (priv->use_fallback == TRUE
-      || !clutter_glx_texture_pixmap_using_extension (texture))
+  if (!clutter_glx_texture_pixmap_using_extension (texture))
     goto cleanup;
 
   priv->use_fallback = FALSE;
@@ -473,7 +665,7 @@ clutter_glx_texture_pixmap_create_glx_pixmap (ClutterGLXTexturePixmap *texture)
       goto cleanup;
     }
 
-  fbconfig = get_fbconfig_for_depth (depth);
+  fbconfig = get_fbconfig_for_depth (texture, depth);
 
   if (!fbconfig)
     {
@@ -499,18 +691,26 @@ clutter_glx_texture_pixmap_create_glx_pixmap (ClutterGLXTexturePixmap *texture)
 
   quality = clutter_texture_get_filter_quality (CLUTTER_TEXTURE (texture));
 
-  if (quality == CLUTTER_TEXTURE_QUALITY_HIGH)
-    mipmap = 1;
-  
+  if (quality == CLUTTER_TEXTURE_QUALITY_HIGH && priv->can_mipmap)
+    mipmap = priv->can_mipmap;
+
   attribs[i++] = GLX_MIPMAP_TEXTURE_EXT;
   attribs[i++] = mipmap;
 
   attribs[i++] = GLX_TEXTURE_TARGET_EXT;
 
-  attribs[i++] = GLX_TEXTURE_2D_EXT;
+  attribs[i++] = (should_use_rectangle ()
+                  ? GLX_TEXTURE_RECTANGLE_EXT : GLX_TEXTURE_2D_EXT);
 
   attribs[i++] = None;
 
+  /* Note: some drivers (e.g. Nvidia) get upset if you effectivly create a glx
+   * pixmap for the same server side object, even though you might have unique
+   * client side names, we currently avoid hitting this problem by destroying
+   * the current glx pixmap first */
+  if (priv->glx_pixmap)
+    clutter_glx_texture_pixmap_free_glx_pixmap (texture);
+
   clutter_x11_trap_x_errors ();
   glx_pixmap = glXCreatePixmap (dpy,
                                 *fbconfig,
@@ -525,8 +725,6 @@ clutter_glx_texture_pixmap_create_glx_pixmap (ClutterGLXTexturePixmap *texture)
       glx_pixmap = None;
     }
 
-  g_free (fbconfig);
-
  cleanup:
 
   if (priv->glx_pixmap)
@@ -536,10 +734,20 @@ clutter_glx_texture_pixmap_create_glx_pixmap (ClutterGLXTexturePixmap *texture)
     {
       priv->glx_pixmap = glx_pixmap;
       
-      create_cogl_texture (CLUTTER_TEXTURE (texture), pixmap_width, pixmap_height);
+      create_cogl_texture (CLUTTER_TEXTURE (texture), 
+                           pixmap_width, pixmap_height);
 
       CLUTTER_NOTE (TEXTURE, "Created GLXPixmap");
 
+      /* Get ready to queue initial mipmap generation */
+      if (_gl_generate_mipmap
+          && priv->can_mipmap
+          &&  clutter_texture_get_filter_quality (CLUTTER_TEXTURE (texture))
+          == CLUTTER_TEXTURE_QUALITY_HIGH)
+        {
+          priv->mipmap_generate_queued++;
+        }
+
       return;
     }
   else
@@ -561,13 +769,12 @@ clutter_glx_texture_pixmap_update_area (ClutterX11TexturePixmap *texture,
                                         gint                     width,
                                         gint                     height)
 {
-  ClutterGLXTexturePixmapPrivate       *priv;
+  ClutterGLXTexturePixmap *texture_glx = CLUTTER_GLX_TEXTURE_PIXMAP (texture);
+  ClutterGLXTexturePixmapPrivate       *priv = texture_glx->priv;
   Display                              *dpy;
 
-
   CLUTTER_NOTE (TEXTURE, "Updating texture pixmap");
 
-  priv = CLUTTER_GLX_TEXTURE_PIXMAP (texture)->priv;
   dpy = clutter_x11_get_default_display();
 
   if (!CLUTTER_ACTOR_IS_REALIZED (texture))
@@ -576,6 +783,9 @@ clutter_glx_texture_pixmap_update_area (ClutterX11TexturePixmap *texture,
   if (priv->use_fallback)
     {
       CLUTTER_NOTE (TEXTURE, "Falling back to X11");
+
+      clutter_glx_texture_pixmap_free_rectangle (texture_glx);
+
       parent_class->update_area (texture,
                                  x, y,
                                  width, height);
@@ -605,6 +815,20 @@ clutter_glx_texture_pixmap_update_area (ClutterX11TexturePixmap *texture,
         CLUTTER_NOTE (TEXTURE, "Update bind_tex_image failed");
 
       priv->bound = TRUE;
+
+      if (_gl_generate_mipmap
+          && priv->can_mipmap
+          &&  clutter_texture_get_filter_quality (CLUTTER_TEXTURE (texture))
+              == CLUTTER_TEXTURE_QUALITY_HIGH)
+        {
+          /* FIXME: It may make more sense to set a flag here and only
+           *        generate the mipmap on a pre paint.. compressing need
+           *        to call generate mipmap 
+           *        May break clones however..
+          */
+          priv->mipmap_generate_queued++;
+        }
+
     }
   else
     g_warning ("Failed to bind initial tex");
@@ -653,7 +877,7 @@ clutter_glx_texture_pixmap_using_extension (ClutterGLXTexturePixmap *texture)
 
   priv = CLUTTER_GLX_TEXTURE_PIXMAP (texture)->priv;
 
-  return (_have_tex_from_pixmap_ext); 
+  return (_have_tex_from_pixmap_ext && !priv->use_fallback); 
   /* Assume NPOT TFP's are supported even if regular NPOT isn't advertised 
    * but tfp is. Seemingly some Intel drivers do this ?
   */
