@@ -189,6 +189,7 @@ clutter_stage_glx_realize (ClutterActor *actor)
                             FocusChangeMask |
                             ExposureMask |
                             KeyPressMask | KeyReleaseMask |
+                            EnterWindowMask | LeaveWindowMask |
                             PropertyChangeMask);
 #ifdef USE_XINPUT          
               _clutter_x11_select_events (stage_x11->xwin);
@@ -202,6 +203,7 @@ clutter_stage_glx_realize (ClutterActor *actor)
                           PointerMotionMask |
                           KeyPressMask | KeyReleaseMask |
                           ButtonPressMask | ButtonReleaseMask |
+                          EnterWindowMask | LeaveWindowMask |
                           PropertyChangeMask);
         }
 
@@ -222,6 +224,9 @@ clutter_stage_glx_realize (ClutterActor *actor)
               g_critical ("Unable to create suitable GL context.");
               goto fail;
             }
+
+          _cogl_set_indirect_context (!glXIsDirect (stage_x11->xdpy,
+                                                    backend_glx->gl_context));
         }
 
       CLUTTER_NOTE (BACKEND, "Marking stage as realized");
