@@ -204,6 +204,51 @@ _clutter_backend_x11_events_uninit (ClutterBackend *backend)
     }
 }
 
+static void
+update_last_event_time (ClutterBackendX11 *backend_x11,
+                        XEvent            *xevent)
+{
+  Time current_time = CurrentTime;
+  Time last_time = backend_x11->last_event_time;
+
+  switch (xevent->type)
+    {
+    case KeyPress:
+    case KeyRelease:
+      current_time = xevent->xkey.time;
+      break;
+
+    case ButtonPress:
+    case ButtonRelease:
+      current_time = xevent->xbutton.time;
+      break;
+
+    case MotionNotify:
+      current_time = xevent->xmotion.time;
+      break;
+
+    case EnterNotify:
+    case LeaveNotify:
+      current_time = xevent->xcrossing.time;
+      break;
+
+    case PropertyNotify:
+      current_time = xevent->xproperty.time;
+      break;
+
+    default:
+      break;
+    }
+
+  /* only change the current event time if it's after the previous event
+   * time, or if it is at least 30 seconds earlier - in case the system
+   * clock was changed
+   */
+  if ((current_time != CurrentTime) &&
+      (current_time > last_time || (last_time - current_time > (30 * 1000))))
+    backend_x11->last_event_time = current_time;
+}
+
 static void
 set_user_time (ClutterBackendX11 *backend_x11,
                Window            *xwindow,
@@ -419,27 +464,30 @@ event_translate (ClutterBackend *backend,
 
   event->any.stage = stage;
 
-
   res = TRUE;
 
+  update_last_event_time (backend_x11, xevent);
+
   switch (xevent->type)
     {
     case ConfigureNotify:
-      CLUTTER_SET_PRIVATE_FLAGS (stage_x11->wrapper,
-                                 CLUTTER_ACTOR_SYNC_MATRICES);
       if (!stage_x11->is_foreign_xwin)
         {
-	  /* Set a flag so that the stage will know the actor is being
-	     resized in response to the window size changing as
-	     opposed to a request from the application. This prevents
-	     it from trying to resize the window again */
-	  stage_x11->handling_configure = TRUE;
+          CLUTTER_NOTE (BACKEND, "%s: ConfigureNotify[%x] (%d, %d)",
+                        G_STRLOC,
+                        (unsigned int) stage_x11->xwin,
+                        xevent->xconfigure.width,
+                        xevent->xconfigure.height);
 
 	  clutter_actor_set_size (CLUTTER_ACTOR (stage),
 				  xevent->xconfigure.width,
 				  xevent->xconfigure.height);
 
-	  stage_x11->handling_configure = FALSE;
+          /* the resize process is complete, so we can ask the stage
+           * to set up the GL viewport with the new size
+           */
+          CLUTTER_SET_PRIVATE_FLAGS (CLUTTER_ACTOR (stage_x11->wrapper),
+                                     CLUTTER_ACTOR_SYNC_MATRICES);
         }
       res = FALSE;
       break;
@@ -664,6 +712,24 @@ event_translate (ClutterBackend *backend,
               event->motion.y = xevent->xmotion.y;
               event->motion.modifier_state = xevent->xmotion.state;
               break;
+
+            case EnterNotify:
+              /* Convert enter notifies to motion events because X
+                 doesn't emit the corresponding motion notify */
+              event->motion.type = event->type = CLUTTER_MOTION;
+              event->motion.time = xevent->xcrossing.time;
+              event->motion.x = xevent->xcrossing.x;
+              event->motion.y = xevent->xcrossing.y;
+              event->motion.modifier_state = xevent->xcrossing.state;
+              break;
+
+            case LeaveNotify:
+              event->crossing.type = event->type = CLUTTER_LEAVE;
+              event->crossing.time = xevent->xcrossing.time;
+              event->crossing.x = xevent->xcrossing.x;
+              event->crossing.y = xevent->xcrossing.y;
+              break;
+
             default:
               /* ignore every other event */
               res = FALSE;
@@ -862,6 +928,7 @@ clutter_x11_handle_event (XEvent *xevent)
   ClutterEvent        *event;
   ClutterMainContext  *clutter_context;
   ClutterX11FilterReturn result;
+  gint                 spin = 1;
 
   /* The return values here are someone approximate; we return
    * CLUTTER_X11_FILTER_REMOVE if and only if a clutter event is
@@ -893,13 +960,21 @@ clutter_x11_handle_event (XEvent *xevent)
       goto out;
     }
 
-  event = clutter_event_get ();
+  /*
+   * Motion events can generate synthetic enter and leave events, so if we
+   * are processing a motion event, we need to spin the event loop at least
+   * two extra times to pump the enter/leave events through (otherwise they
+   * just get pushed down the queue and never processed).
+   */
+  if (event->type == CLUTTER_MOTION)
+    spin += 2;
 
-  if (event)
+  while (spin > 0 && (event = clutter_event_get ()))
     {
       /* forward the event into clutter for emission etc. */
       clutter_do_event (event);
       clutter_event_free (event);
+      --spin;
     }
 
  out:
@@ -973,3 +1048,23 @@ clutter_event_dispatch (GSource     *source,
 
   return TRUE;
 }
+
+/**
+ * clutter_x11_get_current_event_time:
+ *
+ * Retrieves the timestamp of the last X11 event processed by
+ * Clutter. This might be different from the timestamp returned
+ * by clutter_get_current_event_time(), as Clutter may synthesize
+ * or throttle events.
+ *
+ * Return value: a timestamp, in milliseconds
+ *
+ * Since: 1.0
+ */
+Time
+clutter_x11_get_current_event_time (void)
+{
+  ClutterBackend *backend = clutter_get_default_backend ();
+
+  return CLUTTER_BACKEND_X11 (backend)->last_event_time;
+}
